searchData={"items":[{"type":"module","title":"Jido.Signal","doc":"Defines the core Signal structure in Jido, implementing the CloudEvents specification (v1.0.2)\nwith Jido-specific extensions for agent-based systems.\n\nhttps://cloudevents.io/","ref":"Jido.Signal.html"},{"type":"module","title":"Overview - Jido.Signal","doc":"Signals are the universal message format in Jido, serving as the nervous system of your\nagent-based application. Every event, command, and state change flows through the system\nas a Signal, providing:\n\n- Standardized event structure (CloudEvents v1.0.2 compatible)\n- Rich metadata and context tracking\n- Flexible dispatch configuration\n- Automatic serialization","ref":"Jido.Signal.html#module-overview"},{"type":"module","title":"CloudEvents Compliance - Jido.Signal","doc":"Each Signal implements the CloudEvents v1.0.2 specification with these required fields:\n\n- `specversion`: Always \"1.0.2\"\n- `id`: Unique identifier (UUID v4)\n- `source`: Origin of the event (\"/service/component\")\n- `type`: Classification of the event (\"domain.entity.action\")\n\nAnd optional fields:\n\n- `subject`: Specific subject of the event\n- `time`: Timestamp in ISO 8601 format\n- `datacontenttype`: Media type of the data (defaults to \"application/json\")\n- `dataschema`: Schema defining the data structure\n- `data`: The actual event payload","ref":"Jido.Signal.html#module-cloudevents-compliance"},{"type":"module","title":"Jido Extensions - Jido.Signal","doc":"Beyond the CloudEvents spec, Signals include Jido-specific fields:\n\n- `jido_dispatch`: Routing and delivery configuration (optional)","ref":"Jido.Signal.html#module-jido-extensions"},{"type":"module","title":"Creating Signals - Jido.Signal","doc":"Signals can be created in several ways:\n\n```elixir\n# Basic event\n{:ok, signal} = Signal.new(%{\n  type: \"user.created\",\n  source: \"/auth/registration\",\n  data: %{user_id: \"123\", email: \"user@example.com\"}\n})\n\n# With dispatch config\n{:ok, signal} = Signal.new(%{\n  type: \"metrics.collected\",\n  source: \"/monitoring\",\n  data: %{cpu: 80, memory: 70},\n  jido_dispatch: {:pubsub, topic: \"metrics\"}\n})\n```","ref":"Jido.Signal.html#module-creating-signals"},{"type":"module","title":"Custom Signal Types - Jido.Signal","doc":"You can define custom Signal types using the `use Jido.Signal` pattern:\n\n```elixir\ndefmodule MySignal do\n  use Jido.Signal,\n    type: \"my.custom.signal\",\n    default_source: \"/my/service\",\n    datacontenttype: \"application/json\",\n    schema: [\n      user_id: [type: :string, required: true],\n      message: [type: :string, required: true]\n    ]\nend\n\n# Create instances\n{:ok, signal} = MySignal.new(%{user_id: \"123\", message: \"Hello\"})\n\n# Override runtime fields\n{:ok, signal} = MySignal.new(\n  %{user_id: \"123\", message: \"Hello\"},\n  source: \"/different/source\",\n  subject: \"user-notification\",\n  jido_dispatch: {:pubsub, topic: \"events\"}\n)\n```","ref":"Jido.Signal.html#module-custom-signal-types"},{"type":"module","title":"Signal Types - Jido.Signal","doc":"Signal types are strings, but typically use a hierarchical dot notation:\n\n```\n . . [. ]\n```\n\nExamples:\n- `user.profile.updated`\n- `order.payment.processed.success`\n- `system.metrics.collected`\n\nGuidelines for type naming:\n- Use lowercase with dots\n- Keep segments meaningful\n- Order from general to specific\n- Include qualifiers when needed","ref":"Jido.Signal.html#module-signal-types"},{"type":"module","title":"Data Content Types - Jido.Signal","doc":"The `datacontenttype` field indicates the format of the `data` field:\n\n- `application/json` (default) - JSON-structured data\n- `text/plain` - Unstructured text\n- `application/octet-stream` - Binary data\n- Custom MIME types for specific formats","ref":"Jido.Signal.html#module-data-content-types"},{"type":"module","title":"Dispatch Configuration - Jido.Signal","doc":"The `jido_dispatch` field controls how the Signal is delivered:\n\n```elixir\n# Single dispatch config\njido_dispatch: {:pubsub, topic: \"events\"}\n\n# Multiple dispatch targets\njido_dispatch: [\n  {:pubsub, topic: \"events\"},\n  {:logger, level: :info},\n  {:webhook, url: \"https://api.example.com/webhook\"}\n]\n```","ref":"Jido.Signal.html#module-dispatch-configuration"},{"type":"module","title":"See Also - Jido.Signal","doc":"- `Jido.Signal.Router` - Signal routing\n- `Jido.Signal.Dispatch` - Dispatch handling\n- CloudEvents spec: https://cloudevents.io/","ref":"Jido.Signal.html#module-see-also"},{"type":"macro","title":"Jido.Signal.__using__/1","doc":"Defines a new Signal module.\n\nThis macro sets up the necessary structure and callbacks for a custom Signal,\nincluding configuration validation and default implementations.","ref":"Jido.Signal.html#__using__/1"},{"type":"macro","title":"Options - Jido.Signal.__using__/1","doc":"* `:type` (`t:String.t/0`) - Required. The type of the Signal\n\n* `:default_source` (`t:String.t/0`) - The default source of the Signal\n\n* `:datacontenttype` (`t:String.t/0`) - The content type of the data field\n\n* `:dataschema` (`t:String.t/0`) - Schema URI for the data field (optional)\n\n* `:schema` (`t:keyword/0`) - A NimbleOptions schema for validating the Signal's data parameters The default value is `[]`.","ref":"Jido.Signal.html#__using__/1-options"},{"type":"macro","title":"Examples - Jido.Signal.__using__/1","doc":"defmodule MySignal do\n      use Jido.Signal,\n        type: \"my.custom.signal\",\n        default_source: \"/my/service\",\n        schema: [\n          user_id: [type: :string, required: true],\n          message: [type: :string, required: true]\n        ]\n    end","ref":"Jido.Signal.html#__using__/1-examples"},{"type":"function","title":"Jido.Signal.deserialize/2","doc":"Deserializes binary data back into a Signal struct or list of Signal structs.","ref":"Jido.Signal.html#deserialize/2"},{"type":"function","title":"Parameters - Jido.Signal.deserialize/2","doc":"- `binary`: The serialized binary data to deserialize\n- `opts`: Optional configuration including:\n  - `:serializer` - The serializer module to use (defaults to configured serializer)\n  - `:type` - Specific type to deserialize to\n  - `:type_provider` - Custom type provider","ref":"Jido.Signal.html#deserialize/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.deserialize/2","doc":"`{:ok, Signal.t() | list(Signal.t())}` if successful, `{:error, reason}` otherwise","ref":"Jido.Signal.html#deserialize/2-returns"},{"type":"function","title":"Examples - Jido.Signal.deserialize/2","doc":"# JSON deserialization (default)\n    iex> json = ~s({\"type\":\"example.event\",\"source\":\"/example\",\"id\":\"123\"})\n    iex> {:ok, signal} = Jido.Signal.deserialize(json)\n    iex> signal.type\n    \"example.event\"\n\n    # Using a specific serializer\n    iex> {:ok, signal} = Jido.Signal.deserialize(binary, serializer: Jido.Signal.Serialization.ErlangTermSerializer)\n    iex> signal.type\n    \"example.event\"\n\n    # Deserializing multiple Signals\n    iex> json = ~s([{\"type\":\"event1\",\"source\":\"/ex1\"},{\"type\":\"event2\",\"source\":\"/ex2\"}])\n    iex> {:ok, signals} = Jido.Signal.deserialize(json)\n    iex> length(signals)\n    2","ref":"Jido.Signal.html#deserialize/2-examples"},{"type":"function","title":"Jido.Signal.from_map/1","doc":"Creates a new Signal struct from a map.","ref":"Jido.Signal.html#from_map/1"},{"type":"function","title":"Parameters - Jido.Signal.from_map/1","doc":"- `map`: A map containing the Signal attributes.","ref":"Jido.Signal.html#from_map/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.from_map/1","doc":"`{:ok, Signal.t()}` if the map is valid, `{:error, String.t()}` otherwise.","ref":"Jido.Signal.html#from_map/1-returns"},{"type":"function","title":"Examples - Jido.Signal.from_map/1","doc":"iex> Jido.Signal.from_map(%{\"type\" => \"example.event\", \"source\" => \"/example\", \"id\" => \"123\"})\n    {:ok, %Jido.Signal{type: \"example.event\", source: \"/example\", id: \"123\", ...}}","ref":"Jido.Signal.html#from_map/1-examples"},{"type":"function","title":"Jido.Signal.map_to_signal_data/2","doc":"Converts a struct or list of structs to Signal data format.\n\nThis function is useful for converting domain objects to Signal format\nwhile preserving their type information through the TypeProvider.","ref":"Jido.Signal.html#map_to_signal_data/2"},{"type":"function","title":"Parameters - Jido.Signal.map_to_signal_data/2","doc":"- `signals`: A struct or list of structs to convert\n- `fields`: Additional fields to include (currently unused)","ref":"Jido.Signal.html#map_to_signal_data/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.map_to_signal_data/2","doc":"Signal struct or list of Signal structs with the original data as payload","ref":"Jido.Signal.html#map_to_signal_data/2-returns"},{"type":"function","title":"Examples - Jido.Signal.map_to_signal_data/2","doc":"# Converting a single struct\n    iex> user = %User{id: 1, name: \"John\"}\n    iex> signal = Jido.Signal.map_to_signal_data(user)\n    iex> signal.data\n    %User{id: 1, name: \"John\"}\n\n    # Converting multiple structs\n    iex> users = [%User{id: 1}, %User{id: 2}]\n    iex> signals = Jido.Signal.map_to_signal_data(users)\n    iex> length(signals)\n    2","ref":"Jido.Signal.html#map_to_signal_data/2-examples"},{"type":"function","title":"Jido.Signal.new/1","doc":"Creates a new Signal struct.","ref":"Jido.Signal.html#new/1"},{"type":"function","title":"Parameters - Jido.Signal.new/1","doc":"- `attrs`: A map or keyword list containing the Signal attributes.","ref":"Jido.Signal.html#new/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.new/1","doc":"`{:ok, Signal.t()}` if the attributes are valid, `{:error, String.t()}` otherwise.","ref":"Jido.Signal.html#new/1-returns"},{"type":"function","title":"Examples - Jido.Signal.new/1","doc":"iex> Jido.Signal.new(%{type: \"example.event\", source: \"/example\", id: \"123\"})\n    {:ok, %Jido.Signal{type: \"example.event\", source: \"/example\", id: \"123\", ...}}\n\n    iex> Jido.Signal.new(type: \"example.event\", source: \"/example\")\n    {:ok, %Jido.Signal{type: \"example.event\", source: \"/example\", ...}}","ref":"Jido.Signal.html#new/1-examples"},{"type":"function","title":"Jido.Signal.new!/1","doc":"Creates a new Signal struct, raising an error if invalid.","ref":"Jido.Signal.html#new!/1"},{"type":"function","title":"Parameters - Jido.Signal.new!/1","doc":"- `attrs`: A map or keyword list containing the Signal attributes.","ref":"Jido.Signal.html#new!/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.new!/1","doc":"`Signal.t()` if the attributes are valid.","ref":"Jido.Signal.html#new!/1-returns"},{"type":"function","title":"Raises - Jido.Signal.new!/1","doc":"`RuntimeError` if the attributes are invalid.","ref":"Jido.Signal.html#new!/1-raises"},{"type":"function","title":"Examples - Jido.Signal.new!/1","doc":"iex> Jido.Signal.new!(%{type: \"example.event\", source: \"/example\"})\n    %Jido.Signal{type: \"example.event\", source: \"/example\", ...}\n\n    iex> Jido.Signal.new!(type: \"example.event\", source: \"/example\")\n    %Jido.Signal{type: \"example.event\", source: \"/example\", ...}","ref":"Jido.Signal.html#new!/1-examples"},{"type":"function","title":"Jido.Signal.serialize/2","doc":"Serializes a Signal or a list of Signals using the specified or default serializer.","ref":"Jido.Signal.html#serialize/2"},{"type":"function","title":"Parameters - Jido.Signal.serialize/2","doc":"- `signal_or_list`: A Signal struct or list of Signal structs\n- `opts`: Optional configuration including:\n  - `:serializer` - The serializer module to use (defaults to configured serializer)","ref":"Jido.Signal.html#serialize/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.serialize/2","doc":"`{:ok, binary}` on success, `{:error, reason}` on failure","ref":"Jido.Signal.html#serialize/2-returns"},{"type":"function","title":"Examples - Jido.Signal.serialize/2","doc":"iex> signal = %Jido.Signal{type: \"example.event\", source: \"/example\"}\n    iex> {:ok, binary} = Jido.Signal.serialize(signal)\n    iex> is_binary(binary)\n    true\n\n    # Using a specific serializer\n    iex> {:ok, binary} = Jido.Signal.serialize(signal, serializer: Jido.Signal.Serialization.ErlangTermSerializer)\n    iex> is_binary(binary)\n    true\n\n    # Serializing multiple Signals\n    iex> signals = [\n    ...>   %Jido.Signal{type: \"event1\", source: \"/ex1\"},\n    ...>   %Jido.Signal{type: \"event2\", source: \"/ex2\"}\n    ...> ]\n    iex> {:ok, binary} = Jido.Signal.serialize(signals)\n    iex> is_binary(binary)\n    true","ref":"Jido.Signal.html#serialize/2-examples"},{"type":"function","title":"Jido.Signal.serialize!/2","doc":"Legacy serialize function that returns binary directly (for backward compatibility).","ref":"Jido.Signal.html#serialize!/2"},{"type":"type","title":"Jido.Signal.t/0","doc":"","ref":"Jido.Signal.html#t:t/0"},{"type":"module","title":"Jido.Signal.Application","doc":"The main application module for Jido Signal.\n\nThis module handles the initialization and supervision of the signal processing\ninfrastructure, including the Registry and Task Supervisor.","ref":"Jido.Signal.Application.html"},{"type":"function","title":"Jido.Signal.Application.start/2","doc":"Starts the Jido Signal application.\n\nInitializes the supervision tree with the Registry for managing signal subscriptions\nand a Task Supervisor for handling asynchronous operations.","ref":"Jido.Signal.Application.html#start/2"},{"type":"function","title":"Parameters - Jido.Signal.Application.start/2","doc":"- `_type`: The application start type (ignored)\n- `_args`: Application start arguments (ignored)","ref":"Jido.Signal.Application.html#start/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Application.start/2","doc":"`{:ok, pid}` where pid is the supervisor process ID","ref":"Jido.Signal.Application.html#start/2-returns"},{"type":"module","title":"Jido.Signal.Bus","doc":"Implements a signal bus for routing, filtering, and distributing signals.\n\nThe Bus acts as a central hub for signals in the system, allowing components\nto publish and subscribe to signals. It handles routing based on signal paths,\nsubscription management, persistence, and signal filtering. The Bus maintains\nan internal log of signals and provides mechanisms for retrieving historical\nsignals and snapshots.","ref":"Jido.Signal.Bus.html"},{"type":"function","title":"Jido.Signal.Bus.ack/3","doc":"Acknowledges a signal for a persistent subscription.","ref":"Jido.Signal.Bus.html#ack/3"},{"type":"function","title":"Jido.Signal.Bus.child_spec/1","doc":"Returns a child specification for starting the bus under a supervisor.","ref":"Jido.Signal.Bus.html#child_spec/1"},{"type":"function","title":"Options - Jido.Signal.Bus.child_spec/1","doc":"- name: The name to register the bus under (required)\n- router: A custom router implementation (optional)","ref":"Jido.Signal.Bus.html#child_spec/1-options"},{"type":"macro","title":"Jido.Signal.Bus.dbug/2","doc":"","ref":"Jido.Signal.Bus.html#dbug/2"},{"type":"macro","title":"Jido.Signal.Bus.error/2","doc":"","ref":"Jido.Signal.Bus.html#error/2"},{"type":"function","title":"Jido.Signal.Bus.init/1","doc":"Starts a new bus process.\nOptions:\n- name: The name to register the bus under (required)\n- router: A custom router implementation (optional)\n- middleware: A list of {module, opts} tuples for middleware (optional)","ref":"Jido.Signal.Bus.html#init/1"},{"type":"type","title":"Jido.Signal.Bus.path/0","doc":"","ref":"Jido.Signal.Bus.html#t:path/0"},{"type":"function","title":"Jido.Signal.Bus.publish/2","doc":"Publishes a list of signals to the bus.\nReturns {:ok, recorded_signals} on success.","ref":"Jido.Signal.Bus.html#publish/2"},{"type":"function","title":"Jido.Signal.Bus.reconnect/3","doc":"Reconnects a client to a persistent subscription.","ref":"Jido.Signal.Bus.html#reconnect/3"},{"type":"function","title":"Jido.Signal.Bus.replay/4","doc":"Replays signals from the bus log that match the given path pattern.\nOptional start_timestamp to replay from a specific point in time.","ref":"Jido.Signal.Bus.html#replay/4"},{"type":"type","title":"Jido.Signal.Bus.server/0","doc":"","ref":"Jido.Signal.Bus.html#t:server/0"},{"type":"function","title":"Jido.Signal.Bus.snapshot_create/2","doc":"Creates a new snapshot of signals matching the given path pattern.","ref":"Jido.Signal.Bus.html#snapshot_create/2"},{"type":"function","title":"Jido.Signal.Bus.snapshot_delete/2","doc":"Deletes a snapshot by its ID.","ref":"Jido.Signal.Bus.html#snapshot_delete/2"},{"type":"function","title":"Jido.Signal.Bus.snapshot_list/1","doc":"Lists all available snapshots.","ref":"Jido.Signal.Bus.html#snapshot_list/1"},{"type":"function","title":"Jido.Signal.Bus.snapshot_read/2","doc":"Reads a snapshot by its ID.","ref":"Jido.Signal.Bus.html#snapshot_read/2"},{"type":"function","title":"Jido.Signal.Bus.start_link/1","doc":"Starts a new bus process and links it to the calling process.","ref":"Jido.Signal.Bus.html#start_link/1"},{"type":"function","title":"Parameters - Jido.Signal.Bus.start_link/1","doc":"- `opts`: Options for starting the bus:\n  - `:name` - The name to register the bus under (required)\n  - `:router` - A custom router implementation (optional)\n  - `:middleware` - A list of {module, opts} tuples for middleware (optional)","ref":"Jido.Signal.Bus.html#start_link/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Bus.start_link/1","doc":"- `{:ok, pid}` if the bus starts successfully\n- `{:error, reason}` if the bus fails to start","ref":"Jido.Signal.Bus.html#start_link/1-returns"},{"type":"function","title":"Examples - Jido.Signal.Bus.start_link/1","doc":"iex> {:ok, pid} = Jido.Signal.Bus.start_link(name: :my_bus)\n    iex> is_pid(pid)\n    true\n\n    iex> {:ok, pid} = Jido.Signal.Bus.start_link([\n    ...>   name: :my_bus,\n    ...>   router: custom_router,\n    ...>   middleware: [{MyMiddleware, []}]\n    ...> ])\n    iex> is_pid(pid)\n    true","ref":"Jido.Signal.Bus.html#start_link/1-examples"},{"type":"type","title":"Jido.Signal.Bus.start_option/0","doc":"","ref":"Jido.Signal.Bus.html#t:start_option/0"},{"type":"function","title":"Jido.Signal.Bus.subscribe/3","doc":"Subscribes to signals matching the given path pattern.\nOptions:\n- dispatch: How to dispatch signals to the subscriber (default: async to calling process)\n- persistent: Whether the subscription should persist across restarts (default: false)","ref":"Jido.Signal.Bus.html#subscribe/3"},{"type":"type","title":"Jido.Signal.Bus.subscription_id/0","doc":"","ref":"Jido.Signal.Bus.html#t:subscription_id/0"},{"type":"function","title":"Jido.Signal.Bus.unsubscribe/3","doc":"Unsubscribes from signals using the subscription ID.\nOptions:\n- delete_persistence: Whether to delete persistent subscription data (default: false)","ref":"Jido.Signal.Bus.html#unsubscribe/3"},{"type":"function","title":"Jido.Signal.Bus.via_tuple/2","doc":"See  Jido.Signal.Util.via_tuple/2 .","ref":"Jido.Signal.Bus.html#via_tuple/2"},{"type":"function","title":"Jido.Signal.Bus.whereis/2","doc":"See  Jido.Signal.Util.whereis/2 .","ref":"Jido.Signal.Bus.html#whereis/2"},{"type":"behaviour","title":"Jido.Signal.Bus.Middleware","doc":"Behavior for signal bus middleware modules.\n\nMiddleware modules can intercept and transform signals at various points in the bus lifecycle:\n- Before signals are published to the bus\n- After signals are published but before they are dispatched to subscribers\n- Before signals are dispatched to individual subscribers\n- After signals are dispatched to subscribers\n\nMiddleware modules implement callbacks that receive the signal and context, and can:\n- Transform the signal\n- Add metadata\n- Log signal activity\n- Filter signals\n- Perform side effects","ref":"Jido.Signal.Bus.Middleware.html"},{"type":"behaviour","title":"Example - Jido.Signal.Bus.Middleware","doc":"See `Jido.Signal.Bus.Middleware.Logger` for a complete implementation example.","ref":"Jido.Signal.Bus.Middleware.html#module-example"},{"type":"macro","title":"Jido.Signal.Bus.Middleware.__using__/1","doc":"Default implementation that can be used by middleware modules.","ref":"Jido.Signal.Bus.Middleware.html#__using__/1"},{"type":"callback","title":"Jido.Signal.Bus.Middleware.after_dispatch/5","doc":"Called after a signal is dispatched to a subscriber.\n\nCannot modify the signal but can react to dispatch results.","ref":"Jido.Signal.Bus.Middleware.html#c:after_dispatch/5"},{"type":"callback","title":"Parameters - Jido.Signal.Bus.Middleware.after_dispatch/5","doc":"- signal: The signal that was dispatched\n- subscriber: The subscriber that received the signal\n- result: The result of the dispatch operation\n- context: Context information about the operation\n- state: Current middleware state","ref":"Jido.Signal.Bus.Middleware.html#c:after_dispatch/5-parameters"},{"type":"callback","title":"Returns - Jido.Signal.Bus.Middleware.after_dispatch/5","doc":"- `{:cont, new_state}` - Continue processing","ref":"Jido.Signal.Bus.Middleware.html#c:after_dispatch/5-returns"},{"type":"callback","title":"Jido.Signal.Bus.Middleware.after_publish/3","doc":"Called after signals are successfully published to the bus.\n\nCannot modify signals but can perform side effects or update middleware state.","ref":"Jido.Signal.Bus.Middleware.html#c:after_publish/3"},{"type":"callback","title":"Parameters - Jido.Signal.Bus.Middleware.after_publish/3","doc":"- signals: List of signals that were published\n- context: Context information about the operation\n- state: Current middleware state","ref":"Jido.Signal.Bus.Middleware.html#c:after_publish/3-parameters"},{"type":"callback","title":"Returns - Jido.Signal.Bus.Middleware.after_publish/3","doc":"- `{:cont, signals, new_state}` - Continue processing","ref":"Jido.Signal.Bus.Middleware.html#c:after_publish/3-returns"},{"type":"callback","title":"Jido.Signal.Bus.Middleware.before_dispatch/4","doc":"Called before a signal is dispatched to a specific subscriber.\n\nCan transform the signal or prevent dispatch to this subscriber.","ref":"Jido.Signal.Bus.Middleware.html#c:before_dispatch/4"},{"type":"callback","title":"Parameters - Jido.Signal.Bus.Middleware.before_dispatch/4","doc":"- signal: The signal about to be dispatched\n- subscriber: The subscriber that will receive the signal\n- context: Context information about the operation\n- state: Current middleware state","ref":"Jido.Signal.Bus.Middleware.html#c:before_dispatch/4-parameters"},{"type":"callback","title":"Returns - Jido.Signal.Bus.Middleware.before_dispatch/4","doc":"- `{:cont, signal, new_state}` - Continue with potentially modified signal\n- `{:skip, state}` - Skip this subscriber\n- `{:halt, reason, state}` - Stop all dispatch for this signal","ref":"Jido.Signal.Bus.Middleware.html#c:before_dispatch/4-returns"},{"type":"callback","title":"Jido.Signal.Bus.Middleware.before_publish/3","doc":"Called before signals are published to the bus.\n\nCan transform signals or prevent publication by returning :halt.","ref":"Jido.Signal.Bus.Middleware.html#c:before_publish/3"},{"type":"callback","title":"Parameters - Jido.Signal.Bus.Middleware.before_publish/3","doc":"- signals: List of signals about to be published\n- context: Context information about the operation\n- state: Current middleware state","ref":"Jido.Signal.Bus.Middleware.html#c:before_publish/3-parameters"},{"type":"callback","title":"Returns - Jido.Signal.Bus.Middleware.before_publish/3","doc":"- `{:cont, signals, new_state}` - Continue with potentially modified signals\n- `{:halt, reason, state}` - Stop processing and return error","ref":"Jido.Signal.Bus.Middleware.html#c:before_publish/3-returns"},{"type":"type","title":"Jido.Signal.Bus.Middleware.context/0","doc":"","ref":"Jido.Signal.Bus.Middleware.html#t:context/0"},{"type":"type","title":"Jido.Signal.Bus.Middleware.dispatch_result/0","doc":"","ref":"Jido.Signal.Bus.Middleware.html#t:dispatch_result/0"},{"type":"callback","title":"Jido.Signal.Bus.Middleware.init/1","doc":"Initialize the middleware with the given options.\n\nThis callback is called when the middleware is added to the bus.\nIt should return {:ok, state} where state will be passed to all other callbacks.","ref":"Jido.Signal.Bus.Middleware.html#c:init/1"},{"type":"type","title":"Jido.Signal.Bus.Middleware.middleware_state/0","doc":"","ref":"Jido.Signal.Bus.Middleware.html#t:middleware_state/0"},{"type":"module","title":"Jido.Signal.Bus.Middleware.Logger","doc":"A middleware that logs signal activity using Elixir's Logger.\n\nThis middleware provides comprehensive logging of signal flows through the bus,\nincluding publishing events, dispatch events, and errors. It's useful for\ndebugging, monitoring, and auditing signal activity.","ref":"Jido.Signal.Bus.Middleware.Logger.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Bus.Middleware.Logger","doc":"- `:level` - Log level to use (default: `:info`)\n- `:log_publish` - Whether to log publish events (default: `true`)\n- `:log_dispatch` - Whether to log dispatch events (default: `true`)\n- `:log_errors` - Whether to log errors (default: `true`)\n- `:include_signal_data` - Whether to include signal data in logs (default: `false`)\n- `:max_data_length` - Maximum length of signal data to log (default: `100`)","ref":"Jido.Signal.Bus.Middleware.Logger.html#module-configuration-options"},{"type":"module","title":"Examples - Jido.Signal.Bus.Middleware.Logger","doc":"# Basic logging at info level\n    middleware = [{Jido.Signal.Bus.Middleware.Logger, []}]\n\n    # Debug level with signal data\n    middleware = [\n      {Jido.Signal.Bus.Middleware.Logger, [\n        level: :debug,\n        include_signal_data: true,\n        max_data_length: 200\n      ]}\n    ]\n\n    # Only log errors\n    middleware = [\n      {Jido.Signal.Bus.Middleware.Logger, [\n        log_publish: false,\n        log_dispatch: false,\n        log_errors: true\n      ]}\n    ]","ref":"Jido.Signal.Bus.Middleware.Logger.html#module-examples"},{"type":"type","title":"Jido.Signal.Bus.Middleware.Logger.context/0","doc":"","ref":"Jido.Signal.Bus.Middleware.Logger.html#t:context/0"},{"type":"type","title":"Jido.Signal.Bus.Middleware.Logger.dispatch_result/0","doc":"","ref":"Jido.Signal.Bus.Middleware.Logger.html#t:dispatch_result/0"},{"type":"module","title":"Jido.Signal.Bus.MiddlewarePipeline","doc":"Handles execution of middleware chains for signal bus operations.\n\nThis module provides functions to execute middleware callbacks in sequence,\nallowing each middleware to transform signals or control the flow of execution.","ref":"Jido.Signal.Bus.MiddlewarePipeline.html"},{"type":"function","title":"Jido.Signal.Bus.MiddlewarePipeline.after_dispatch/5","doc":"Executes the after_dispatch middleware chain.\n\nThis is called for side effects only after a signal has been dispatched.","ref":"Jido.Signal.Bus.MiddlewarePipeline.html#after_dispatch/5"},{"type":"function","title":"Jido.Signal.Bus.MiddlewarePipeline.after_publish/3","doc":"Executes the after_publish middleware chain.\n\nThis is called for side effects only - signals cannot be modified.","ref":"Jido.Signal.Bus.MiddlewarePipeline.html#after_publish/3"},{"type":"function","title":"Jido.Signal.Bus.MiddlewarePipeline.before_dispatch/4","doc":"Executes the before_dispatch middleware chain for a single signal and subscriber.\n\nReturns the potentially modified signal, or indicates if dispatch should be skipped/halted.","ref":"Jido.Signal.Bus.MiddlewarePipeline.html#before_dispatch/4"},{"type":"function","title":"Jido.Signal.Bus.MiddlewarePipeline.before_publish/3","doc":"Executes the before_publish middleware chain.\n\nStops execution if any middleware returns :halt.","ref":"Jido.Signal.Bus.MiddlewarePipeline.html#before_publish/3"},{"type":"type","title":"Jido.Signal.Bus.MiddlewarePipeline.context/0","doc":"","ref":"Jido.Signal.Bus.MiddlewarePipeline.html#t:context/0"},{"type":"function","title":"Jido.Signal.Bus.MiddlewarePipeline.init_middleware/1","doc":"Initializes a list of middleware modules with their options.\n\nReturns a list of {module, state} tuples that can be used in the pipeline.","ref":"Jido.Signal.Bus.MiddlewarePipeline.html#init_middleware/1"},{"type":"type","title":"Jido.Signal.Bus.MiddlewarePipeline.middleware_config/0","doc":"","ref":"Jido.Signal.Bus.MiddlewarePipeline.html#t:middleware_config/0"},{"type":"module","title":"Jido.Signal.Bus.PersistentSubscription","doc":"A GenServer that manages persistent subscription state and checkpoints for a single subscriber.\n\nThis module maintains the subscription state for a client, tracking which signals have been\nacknowledged and allowing clients to resume from their last checkpoint after disconnection.\nEach instance maps 1:1 to a bus subscriber and is managed as a child of the Bus's dynamic supervisor.","ref":"Jido.Signal.Bus.PersistentSubscription.html"},{"type":"function","title":"Jido.Signal.Bus.PersistentSubscription.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Jido.Signal.Bus.PersistentSubscription.html#child_spec/1"},{"type":"macro","title":"Jido.Signal.Bus.PersistentSubscription.dbug/2","doc":"","ref":"Jido.Signal.Bus.PersistentSubscription.html#dbug/2"},{"type":"macro","title":"Jido.Signal.Bus.PersistentSubscription.error/2","doc":"","ref":"Jido.Signal.Bus.PersistentSubscription.html#error/2"},{"type":"function","title":"Jido.Signal.Bus.PersistentSubscription.start_link/1","doc":"Starts a new persistent subscription process.\n\nOptions:\n- id: Unique identifier for this subscription (required)\n- bus_pid: PID of the bus this subscription belongs to (required)\n- path: Signal path pattern to subscribe to (required)\n- start_from: Where to start reading signals from (:origin, :current, or timestamp)\n- max_in_flight: Maximum number of unacknowledged signals\n- client_pid: PID of the client process (required)\n- dispatch_opts: Additional dispatch options for the subscription","ref":"Jido.Signal.Bus.PersistentSubscription.html#start_link/1"},{"type":"type","title":"Jido.Signal.Bus.PersistentSubscription.t/0","doc":"","ref":"Jido.Signal.Bus.PersistentSubscription.html#t:t/0"},{"type":"function","title":"Jido.Signal.Bus.PersistentSubscription.via_tuple/1","doc":"See  Jido.Signal.Util.via_tuple/1 .","ref":"Jido.Signal.Bus.PersistentSubscription.html#via_tuple/1"},{"type":"function","title":"Jido.Signal.Bus.PersistentSubscription.whereis/1","doc":"See  Jido.Signal.Util.whereis/1 .","ref":"Jido.Signal.Bus.PersistentSubscription.html#whereis/1"},{"type":"module","title":"Jido.Signal.Bus.RecordedSignal","doc":"Represents a signal that has been recorded in the bus log.\n\nThis struct wraps a signal with additional metadata about when it was recorded.","ref":"Jido.Signal.Bus.RecordedSignal.html"},{"type":"function","title":"Jido.Signal.Bus.RecordedSignal.deserialize/1","doc":"Deserializes a JSON string back into a RecordedSignal struct or list of RecordedSignal structs.","ref":"Jido.Signal.Bus.RecordedSignal.html#deserialize/1"},{"type":"function","title":"Parameters - Jido.Signal.Bus.RecordedSignal.deserialize/1","doc":"- `json`: The JSON string to deserialize","ref":"Jido.Signal.Bus.RecordedSignal.html#deserialize/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Bus.RecordedSignal.deserialize/1","doc":"`{:ok, RecordedSignal.t() | list(RecordedSignal.t())}` if successful, `{:error, reason}` otherwise","ref":"Jido.Signal.Bus.RecordedSignal.html#deserialize/1-returns"},{"type":"function","title":"Examples - Jido.Signal.Bus.RecordedSignal.deserialize/1","doc":"iex> json = ~s({\"id\":\"rec123\",\"type\":\"example.event\",\"created_at\":\"2023-01-01T00:00:00Z\",\"signal\":{\"type\":\"example.event\",\"source\":\"/example\"}})\n    iex> {:ok, recorded} = Jido.Signal.Bus.RecordedSignal.deserialize(json)\n    iex> recorded.id\n    \"rec123\"\n\n    iex> # Deserializing multiple RecordedSignals\n    iex> json = ~s([{\"id\":\"rec1\",\"type\":\"event1\",\"created_at\":\"2023-01-01T00:00:00Z\",\"signal\":{\"type\":\"event1\",\"source\":\"/ex\"}}])\n    iex> {:ok, records} = Jido.Signal.Bus.RecordedSignal.deserialize(json)\n    iex> length(records)\n    1","ref":"Jido.Signal.Bus.RecordedSignal.html#deserialize/1-examples"},{"type":"function","title":"Jido.Signal.Bus.RecordedSignal.serialize/1","doc":"Serializes a RecordedSignal or a list of RecordedSignals to JSON string.","ref":"Jido.Signal.Bus.RecordedSignal.html#serialize/1"},{"type":"function","title":"Parameters - Jido.Signal.Bus.RecordedSignal.serialize/1","doc":"- `recorded_signal_or_list`: A RecordedSignal struct or list of RecordedSignal structs","ref":"Jido.Signal.Bus.RecordedSignal.html#serialize/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Bus.RecordedSignal.serialize/1","doc":"A JSON string representing the RecordedSignal(s)","ref":"Jido.Signal.Bus.RecordedSignal.html#serialize/1-returns"},{"type":"function","title":"Examples - Jido.Signal.Bus.RecordedSignal.serialize/1","doc":"iex> signal = %Jido.Signal{type: \"example.event\", source: \"/example\"}\n    iex> recorded = %Jido.Signal.Bus.RecordedSignal{id: \"rec123\", type: \"example.event\", created_at: DateTime.utc_now(), signal: signal}\n    iex> json = Jido.Signal.Bus.RecordedSignal.serialize(recorded)\n    iex> is_binary(json)\n    true\n\n    iex> # Serializing multiple RecordedSignals\n    iex> signal = %Jido.Signal{type: \"example.event\", source: \"/example\"}\n    iex> records = [\n    ...>   %Jido.Signal.Bus.RecordedSignal{id: \"rec1\", type: \"event1\", created_at: DateTime.utc_now(), signal: signal},\n    ...>   %Jido.Signal.Bus.RecordedSignal{id: \"rec2\", type: \"event2\", created_at: DateTime.utc_now(), signal: signal}\n    ...> ]\n    iex> json = Jido.Signal.Bus.RecordedSignal.serialize(records)\n    iex> is_binary(json)\n    true","ref":"Jido.Signal.Bus.RecordedSignal.html#serialize/1-examples"},{"type":"type","title":"Jido.Signal.Bus.RecordedSignal.t/0","doc":"A recorded signal with metadata","ref":"Jido.Signal.Bus.RecordedSignal.html#t:t/0"},{"type":"module","title":"Jido.Signal.Bus.Snapshot","doc":"Manages snapshots of the bus's signal log. A snapshot represents a filtered view\nof signals at a particular point in time, filtered by a path pattern.\n\nSnapshots are immutable once created and are stored in :persistent_term for efficiency.\nThe bus state only maintains lightweight references to the snapshots.","ref":"Jido.Signal.Bus.Snapshot.html"},{"type":"module","title":"Usage - Jido.Signal.Bus.Snapshot","doc":"```elixir\n# Create a snapshot of all signals\n{:ok, snapshot_ref, new_state} = Snapshot.create(state, \"*\")\n\n# Create a snapshot of specific signal types\n{:ok, snapshot_ref, new_state} = Snapshot.create(state, \"user.created\")\n\n# Create a snapshot with a custom ID\n{:ok, snapshot_ref, new_state} = Snapshot.create(state, \"user.created\", id: \"my-custom-id\")\n\n# Create a snapshot with signals after a specific timestamp\ntimestamp = DateTime.utc_now() |> DateTime.to_unix(:millisecond)\n{:ok, snapshot_ref, new_state} = Snapshot.create(state, \"*\", start_timestamp: timestamp)\n\n# List all snapshots\nsnapshots = Snapshot.list(state)\n\n# Read a snapshot\n{:ok, snapshot_data} = Snapshot.read(state, snapshot_ref.id)\n\n# Delete a snapshot\n{:ok, new_state} = Snapshot.delete(state, snapshot_ref.id)\n\n# Clean up all snapshots\n{:ok, new_state} = Snapshot.cleanup(state)\n\n# Clean up snapshots matching a filter\n{:ok, new_state} = Snapshot.cleanup(state, fn ref -> ref.path == \"user.created\" end)\n```","ref":"Jido.Signal.Bus.Snapshot.html#module-usage"},{"type":"function","title":"Jido.Signal.Bus.Snapshot.cleanup/1","doc":"Cleans up all snapshots from the bus state.\nReturns {:ok, new_state} with all snapshots removed.","ref":"Jido.Signal.Bus.Snapshot.html#cleanup/1"},{"type":"function","title":"Examples - Jido.Signal.Bus.Snapshot.cleanup/1","doc":"iex> Snapshot.cleanup(state)\n    {:ok, %BusState{snapshots: %{}}}","ref":"Jido.Signal.Bus.Snapshot.html#cleanup/1-examples"},{"type":"function","title":"Jido.Signal.Bus.Snapshot.cleanup/2","doc":"Cleans up snapshots from the bus state based on a filter function.\nThe filter function should return true for snapshots that should be removed.\nReturns {:ok, new_state} with filtered snapshots removed.","ref":"Jido.Signal.Bus.Snapshot.html#cleanup/2"},{"type":"function","title":"Examples - Jido.Signal.Bus.Snapshot.cleanup/2","doc":"iex> Snapshot.cleanup(state, fn ref -> ref.path == \"user.created\" end)\n    {:ok, %BusState{}}\n\n    iex> Snapshot.cleanup(state, fn ref -> DateTime.compare(ref.created_at, cutoff_time) == :lt end)\n    {:ok, %BusState{}}","ref":"Jido.Signal.Bus.Snapshot.html#cleanup/2-examples"},{"type":"function","title":"Jido.Signal.Bus.Snapshot.create/3","doc":"Creates a new snapshot of signals matching the given path pattern.\nStores the snapshot data in :persistent_term and returns a reference.\nReturns {:ok, snapshot_ref, new_state} on success or {:error, reason} on failure.","ref":"Jido.Signal.Bus.Snapshot.html#create/3"},{"type":"function","title":"Options - Jido.Signal.Bus.Snapshot.create/3","doc":"* `:id` - Custom ID for the snapshot (optional)\n* `:start_timestamp` - Only include signals after this timestamp in milliseconds (optional)\n* `:correlation_id` - Only include signals with this correlation ID (optional)\n* `:batch_size` - Maximum number of signals to include (optional, defaults to 1000)","ref":"Jido.Signal.Bus.Snapshot.html#create/3-options"},{"type":"function","title":"Examples - Jido.Signal.Bus.Snapshot.create/3","doc":"iex> Snapshot.create(state, \"*\")\n    {:ok, %SnapshotRef{}, %BusState{}}\n\n    iex> Snapshot.create(state, \"user.created\", id: \"my-snapshot\")\n    {:ok, %SnapshotRef{id: \"my-snapshot\"}, %BusState{}}\n\n    iex> Snapshot.create(state, \"*\", start_timestamp: 1612345678000)\n    {:ok, %SnapshotRef{}, %BusState{}}","ref":"Jido.Signal.Bus.Snapshot.html#create/3-examples"},{"type":"macro","title":"Jido.Signal.Bus.Snapshot.dbug/2","doc":"","ref":"Jido.Signal.Bus.Snapshot.html#dbug/2"},{"type":"function","title":"Jido.Signal.Bus.Snapshot.delete/2","doc":"Deletes a snapshot by its ID.\nRemoves both the reference from the state and the data from persistent_term.\nReturns {:ok, new_state} on success or {:error, :not_found} if snapshot doesn't exist.","ref":"Jido.Signal.Bus.Snapshot.html#delete/2"},{"type":"function","title":"Examples - Jido.Signal.Bus.Snapshot.delete/2","doc":"iex> Snapshot.delete(state, \"snapshot-id\")\n    {:ok, %BusState{}}\n\n    iex> Snapshot.delete(state, \"non-existent-id\")\n    {:error, :not_found}","ref":"Jido.Signal.Bus.Snapshot.html#delete/2-examples"},{"type":"macro","title":"Jido.Signal.Bus.Snapshot.error/2","doc":"","ref":"Jido.Signal.Bus.Snapshot.html#error/2"},{"type":"function","title":"Jido.Signal.Bus.Snapshot.list/1","doc":"Lists all snapshot references in the bus state, sorted by creation time (newest first).\nReturns a list of snapshot references.","ref":"Jido.Signal.Bus.Snapshot.html#list/1"},{"type":"function","title":"Examples - Jido.Signal.Bus.Snapshot.list/1","doc":"iex> Snapshot.list(state)\n    [%SnapshotRef{}, %SnapshotRef{}]\n\n    iex> Snapshot.list(empty_state)\n    []","ref":"Jido.Signal.Bus.Snapshot.html#list/1-examples"},{"type":"function","title":"Jido.Signal.Bus.Snapshot.read/2","doc":"Reads a snapshot by its ID.\nReturns {:ok, snapshot_data} if found or {:error, :not_found} if not found.","ref":"Jido.Signal.Bus.Snapshot.html#read/2"},{"type":"function","title":"Examples - Jido.Signal.Bus.Snapshot.read/2","doc":"iex> Snapshot.read(state, \"snapshot-id\")\n    {:ok, %SnapshotData{}}\n\n    iex> Snapshot.read(state, \"non-existent-id\")\n    {:error, :not_found}","ref":"Jido.Signal.Bus.Snapshot.html#read/2-examples"},{"type":"module","title":"Jido.Signal.Bus.Snapshot.SnapshotData","doc":"The actual snapshot data stored in :persistent_term.\nContains the full signal list and metadata.","ref":"Jido.Signal.Bus.Snapshot.SnapshotData.html"},{"type":"module","title":"Fields - Jido.Signal.Bus.Snapshot.SnapshotData","doc":"* `id` - Unique identifier for the snapshot\n* `path` - The path pattern used to filter signals\n* `signals` - Map of recorded signals matching the path pattern, keyed by signal ID\n* `created_at` - When the snapshot was created","ref":"Jido.Signal.Bus.Snapshot.SnapshotData.html#module-fields"},{"type":"type","title":"Jido.Signal.Bus.Snapshot.SnapshotData.t/0","doc":"","ref":"Jido.Signal.Bus.Snapshot.SnapshotData.html#t:t/0"},{"type":"module","title":"Jido.Signal.Bus.Snapshot.SnapshotRef","doc":"A lightweight reference to a snapshot stored in :persistent_term.\nContains only the metadata needed for listing and lookup.","ref":"Jido.Signal.Bus.Snapshot.SnapshotRef.html"},{"type":"module","title":"Fields - Jido.Signal.Bus.Snapshot.SnapshotRef","doc":"* `id` - Unique identifier for the snapshot\n* `path` - The path pattern used to filter signals\n* `created_at` - When the snapshot was created","ref":"Jido.Signal.Bus.Snapshot.SnapshotRef.html#module-fields"},{"type":"type","title":"Jido.Signal.Bus.Snapshot.SnapshotRef.t/0","doc":"","ref":"Jido.Signal.Bus.Snapshot.SnapshotRef.html#t:t/0"},{"type":"module","title":"Jido.Signal.Bus.State","doc":"Defines the state structure for the signal bus.\n\nThis module contains the type definitions and operations for managing\nthe internal state of the signal bus, including signal logs, subscriptions,\nsnapshots, and router configuration. It provides functions for manipulating\nand querying this state.","ref":"Jido.Signal.Bus.State.html"},{"type":"function","title":"Jido.Signal.Bus.State.add_route/2","doc":"Adds a route to the router in the bus state.","ref":"Jido.Signal.Bus.State.html#add_route/2"},{"type":"function","title":"Parameters - Jido.Signal.Bus.State.add_route/2","doc":"- `state`: The current bus state\n- `route`: The route to add to the router","ref":"Jido.Signal.Bus.State.html#add_route/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Bus.State.add_route/2","doc":"- `{:ok, new_state}` if successful\n- `{:error, reason}` if the route addition fails","ref":"Jido.Signal.Bus.State.html#add_route/2-returns"},{"type":"function","title":"Jido.Signal.Bus.State.add_subscription/3","doc":"Adds a subscription to the bus state and creates a corresponding route.","ref":"Jido.Signal.Bus.State.html#add_subscription/3"},{"type":"function","title":"Parameters - Jido.Signal.Bus.State.add_subscription/3","doc":"- `state`: The current bus state\n- `subscription_id`: The unique ID for the subscription\n- `subscription`: The subscription struct to add","ref":"Jido.Signal.Bus.State.html#add_subscription/3-parameters"},{"type":"function","title":"Returns - Jido.Signal.Bus.State.add_subscription/3","doc":"- `{:ok, new_state}` if successful\n- `{:error, :subscription_exists}` if a subscription with this ID already exists","ref":"Jido.Signal.Bus.State.html#add_subscription/3-returns"},{"type":"function","title":"Jido.Signal.Bus.State.append_signals/2","doc":"Merges a list of recorded signals into the existing log.\nSignals are added to the log keyed by their IDs.\nIf a signal with the same ID already exists, it will be overwritten.","ref":"Jido.Signal.Bus.State.html#append_signals/2"},{"type":"function","title":"Parameters - Jido.Signal.Bus.State.append_signals/2","doc":"- state: The current bus state\n  - signals: List of recorded signals to merge","ref":"Jido.Signal.Bus.State.html#append_signals/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Bus.State.append_signals/2","doc":"- `{:ok, new_state, recorded_signals}` with signals merged into log\n  - `{:error, reason}` if there was an error processing the signals","ref":"Jido.Signal.Bus.State.html#append_signals/2-returns"},{"type":"function","title":"Jido.Signal.Bus.State.clear_log/1","doc":"Clears all signals from the log.","ref":"Jido.Signal.Bus.State.html#clear_log/1"},{"type":"macro","title":"Jido.Signal.Bus.State.dbug/2","doc":"","ref":"Jido.Signal.Bus.State.html#dbug/2"},{"type":"macro","title":"Jido.Signal.Bus.State.error/2","doc":"","ref":"Jido.Signal.Bus.State.html#error/2"},{"type":"function","title":"Jido.Signal.Bus.State.get_subscription/2","doc":"Retrieves a subscription from the bus state.","ref":"Jido.Signal.Bus.State.html#get_subscription/2"},{"type":"function","title":"Parameters - Jido.Signal.Bus.State.get_subscription/2","doc":"- `state`: The current bus state\n- `subscription_id`: The ID of the subscription to retrieve","ref":"Jido.Signal.Bus.State.html#get_subscription/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Bus.State.get_subscription/2","doc":"The subscription struct if found, `nil` otherwise","ref":"Jido.Signal.Bus.State.html#get_subscription/2-returns"},{"type":"function","title":"Jido.Signal.Bus.State.has_subscription?/2","doc":"Checks if a subscription exists in the bus state.","ref":"Jido.Signal.Bus.State.html#has_subscription?/2"},{"type":"function","title":"Parameters - Jido.Signal.Bus.State.has_subscription?/2","doc":"- `state`: The current bus state\n- `subscription_id`: The ID of the subscription to check","ref":"Jido.Signal.Bus.State.html#has_subscription?/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Bus.State.has_subscription?/2","doc":"`true` if the subscription exists, `false` otherwise","ref":"Jido.Signal.Bus.State.html#has_subscription?/2-returns"},{"type":"function","title":"Jido.Signal.Bus.State.log_to_list/1","doc":"Converts the signal log from a map to a sorted list.","ref":"Jido.Signal.Bus.State.html#log_to_list/1"},{"type":"function","title":"Parameters - Jido.Signal.Bus.State.log_to_list/1","doc":"- `state`: The current bus state","ref":"Jido.Signal.Bus.State.html#log_to_list/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Bus.State.log_to_list/1","doc":"A list of signals sorted by their IDs","ref":"Jido.Signal.Bus.State.html#log_to_list/1-returns"},{"type":"function","title":"Examples - Jido.Signal.Bus.State.log_to_list/1","doc":"iex> state = %Jido.Signal.Bus.State{log: %{\"2\" => signal2, \"1\" => signal1}}\n    iex> signals = Jido.Signal.Bus.State.log_to_list(state)\n    iex> length(signals)\n    2\n    iex> Enum.map(signals, & &1.id)\n    [\"1\", \"2\"]","ref":"Jido.Signal.Bus.State.html#log_to_list/1-examples"},{"type":"function","title":"Jido.Signal.Bus.State.remove_route/2","doc":"Removes a route from the router in the bus state.","ref":"Jido.Signal.Bus.State.html#remove_route/2"},{"type":"function","title":"Parameters - Jido.Signal.Bus.State.remove_route/2","doc":"- `state`: The current bus state\n- `route`: The route to remove (can be a Route struct or path string)","ref":"Jido.Signal.Bus.State.html#remove_route/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Bus.State.remove_route/2","doc":"- `{:ok, new_state}` if successful\n- `{:error, :route_not_found}` if the route doesn't exist","ref":"Jido.Signal.Bus.State.html#remove_route/2-returns"},{"type":"function","title":"Jido.Signal.Bus.State.remove_subscription/3","doc":"Removes a subscription from the bus state and its corresponding route.","ref":"Jido.Signal.Bus.State.html#remove_subscription/3"},{"type":"function","title":"Parameters - Jido.Signal.Bus.State.remove_subscription/3","doc":"- `state`: The current bus state\n- `subscription_id`: The ID of the subscription to remove\n- `opts`: Options including:\n  - `:delete_persistence` - Whether to delete persistence (default: true)","ref":"Jido.Signal.Bus.State.html#remove_subscription/3-parameters"},{"type":"function","title":"Returns - Jido.Signal.Bus.State.remove_subscription/3","doc":"- `{:ok, new_state}` if successful\n- `{:error, :subscription_not_found}` if the subscription doesn't exist","ref":"Jido.Signal.Bus.State.html#remove_subscription/3-returns"},{"type":"type","title":"Jido.Signal.Bus.State.t/0","doc":"","ref":"Jido.Signal.Bus.State.html#t:t/0"},{"type":"function","title":"Jido.Signal.Bus.State.truncate_log/2","doc":"Truncates the signal log to the specified maximum size.\nKeeps the most recent signals and discards older ones.","ref":"Jido.Signal.Bus.State.html#truncate_log/2"},{"type":"module","title":"Jido.Signal.Bus.Stream","doc":"Provides streaming functionality for the signal bus.\n\nThis module contains functions for filtering, processing, and publishing\nsignals through the bus streaming interface. It supports operations like\nfiltering signals by type pattern and timestamp, as well as publishing\nsignals to subscribers.","ref":"Jido.Signal.Bus.Stream.html"},{"type":"function","title":"Jido.Signal.Bus.Stream.ack/3","doc":"Acknowledges a signal for a given subscription.","ref":"Jido.Signal.Bus.Stream.html#ack/3"},{"type":"function","title":"Jido.Signal.Bus.Stream.clear/1","doc":"Clears all signals from the log.","ref":"Jido.Signal.Bus.Stream.html#clear/1"},{"type":"macro","title":"Jido.Signal.Bus.Stream.dbug/2","doc":"","ref":"Jido.Signal.Bus.Stream.html#dbug/2"},{"type":"macro","title":"Jido.Signal.Bus.Stream.error/2","doc":"","ref":"Jido.Signal.Bus.Stream.html#error/2"},{"type":"function","title":"Jido.Signal.Bus.Stream.filter/4","doc":"Filters signals from the bus state's log based on type pattern and timestamp.\nThe type pattern is used for matching against the signal's type field.","ref":"Jido.Signal.Bus.Stream.html#filter/4"},{"type":"function","title":"Jido.Signal.Bus.Stream.publish/2","doc":"Publishes signals to the bus, recording them and routing them to subscribers.\nEach signal is routed based on its own type field.\nOnly accepts proper Jido.Signal structs to ensure system integrity.\nSignals are recorded and routed in the exact order they are received.","ref":"Jido.Signal.Bus.Stream.html#publish/2"},{"type":"function","title":"Jido.Signal.Bus.Stream.truncate/2","doc":"Truncates the signal log to the specified maximum size.\nKeeps the most recent signals and discards older ones.","ref":"Jido.Signal.Bus.Stream.html#truncate/2"},{"type":"module","title":"Jido.Signal.Bus.Subscriber","doc":"Defines the subscriber model and subscription management for the signal bus.\n\nThis module contains the subscriber type definition and functions for creating,\nmanaging, and dispatching signals to subscribers. It supports both regular and\npersistent subscriptions, handling subscription lifetime and signal delivery.","ref":"Jido.Signal.Bus.Subscriber.html"},{"type":"macro","title":"Jido.Signal.Bus.Subscriber.dbug/2","doc":"","ref":"Jido.Signal.Bus.Subscriber.html#dbug/2"},{"type":"macro","title":"Jido.Signal.Bus.Subscriber.error/2","doc":"","ref":"Jido.Signal.Bus.Subscriber.html#error/2"},{"type":"function","title":"Jido.Signal.Bus.Subscriber.subscribe/4","doc":"","ref":"Jido.Signal.Bus.Subscriber.html#subscribe/4"},{"type":"type","title":"Jido.Signal.Bus.Subscriber.t/0","doc":"","ref":"Jido.Signal.Bus.Subscriber.html#t:t/0"},{"type":"function","title":"Jido.Signal.Bus.Subscriber.unsubscribe/3","doc":"Unsubscribes from the bus by removing the subscription and cleaning up resources.\n\nFor persistent subscriptions, this also terminates the associated process.","ref":"Jido.Signal.Bus.Subscriber.html#unsubscribe/3"},{"type":"function","title":"Parameters - Jido.Signal.Bus.Subscriber.unsubscribe/3","doc":"- `state`: The current bus state\n- `subscription_id`: The unique identifier of the subscription to remove\n- `opts`: Additional options (currently unused)","ref":"Jido.Signal.Bus.Subscriber.html#unsubscribe/3-parameters"},{"type":"function","title":"Returns - Jido.Signal.Bus.Subscriber.unsubscribe/3","doc":"- `{:ok, new_state}` if successful\n- `{:error, Error.t()}` if the subscription doesn't exist or removal fails","ref":"Jido.Signal.Bus.Subscriber.html#unsubscribe/3-returns"},{"type":"module","title":"Jido.Signal.Dispatch","doc":"A flexible signal dispatching system that routes signals to various destinations using configurable adapters.\n\nThe Dispatch module serves as the central hub for signal delivery in the Jido system. It provides a unified\ninterface for sending signals to different destinations through various adapters. Each adapter implements\nspecific delivery mechanisms suited for different use cases.","ref":"Jido.Signal.Dispatch.html"},{"type":"module","title":"Built-in Adapters - Jido.Signal.Dispatch","doc":"The following adapters are provided out of the box:\n\n* `:pid` - Direct delivery to a specific process (see `Jido.Signal.Dispatch.PidAdapter`)\n* `:bus` - Delivery to an event bus (**UNSUPPORTED** - implementation pending)\n* `:named` - Delivery to a named process (see `Jido.Signal.Dispatch.Named`)\n* `:pubsub` - Delivery via PubSub mechanism (see `Jido.Signal.Dispatch.PubSub`)\n* `:logger` - Log signals using Logger (see `Jido.Signal.Dispatch.LoggerAdapter`)\n* `:console` - Print signals to console (see `Jido.Signal.Dispatch.ConsoleAdapter`)\n* `:noop` - No-op adapter for testing/development (see `Jido.Signal.Dispatch.NoopAdapter`)\n* `:http` - HTTP requests using :httpc (see `Jido.Signal.Dispatch.Http`)\n* `:webhook` - Webhook delivery with signatures (see `Jido.Signal.Dispatch.Webhook`)","ref":"Jido.Signal.Dispatch.html#module-built-in-adapters"},{"type":"module","title":"Configuration - Jido.Signal.Dispatch","doc":"Each adapter requires specific configuration options. A dispatch configuration is a tuple of\n`{adapter_type, options}` where:\n\n* `adapter_type` - One of the built-in adapter types above or a custom module implementing the `Jido.Signal.Dispatch.Adapter` behaviour\n* `options` - Keyword list of options specific to the chosen adapter\n\nMultiple dispatch configurations can be provided as a list to send signals to multiple destinations.","ref":"Jido.Signal.Dispatch.html#module-configuration"},{"type":"module","title":"Dispatch Modes - Jido.Signal.Dispatch","doc":"The module supports three dispatch modes:\n\n1. Synchronous (via `dispatch/2`) - Fire-and-forget dispatch that returns when all dispatches complete\n2. Asynchronous (via `dispatch_async/2`) - Returns immediately with a task that can be monitored\n3. Batched (via `dispatch_batch/3`) - Handles large numbers of dispatches in configurable batches","ref":"Jido.Signal.Dispatch.html#module-dispatch-modes"},{"type":"module","title":"Examples - Jido.Signal.Dispatch","doc":"# Synchronous dispatch\n    config = {:pid, [target: pid, delivery_mode: :async]}\n    :ok = Dispatch.dispatch(signal, config)\n\n    # Asynchronous dispatch\n    {:ok, task} = Dispatch.dispatch_async(signal, config)\n    :ok = Task.await(task)\n\n    # Batch dispatch\n    configs = List.duplicate({:pid, [target: pid]}, 1000)\n    :ok = Dispatch.dispatch_batch(signal, configs, batch_size: 100)\n\n    # HTTP dispatch\n    config = {:http, [\n      url: \"https://api.example.com/events\",\n      method: :post,\n      headers: [{\"x-api-key\", \"secret\"}]\n    ]}\n    :ok = Dispatch.dispatch(signal, config)\n\n    # Webhook dispatch\n    config = {:webhook, [\n      url: \"https://api.example.com/webhook\",\n      secret: \"webhook_secret\",\n      event_type_map: %{\"user:created\" => \"user.created\"}\n    ]}\n    :ok = Dispatch.dispatch(signal, config)","ref":"Jido.Signal.Dispatch.html#module-examples"},{"type":"type","title":"Jido.Signal.Dispatch.adapter/0","doc":"","ref":"Jido.Signal.Dispatch.html#t:adapter/0"},{"type":"type","title":"Jido.Signal.Dispatch.batch_opts/0","doc":"","ref":"Jido.Signal.Dispatch.html#t:batch_opts/0"},{"type":"function","title":"Jido.Signal.Dispatch.dispatch/2","doc":"Dispatches a signal using the provided configuration.\n\nThis is a synchronous operation that returns when all dispatches complete.\nFor asynchronous dispatch, use `dispatch_async/2`.\nFor batch dispatch, use `dispatch_batch/3`.","ref":"Jido.Signal.Dispatch.html#dispatch/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.dispatch/2","doc":"- `signal` - The signal to dispatch\n- `config` - Either a single dispatch configuration tuple or a list of configurations","ref":"Jido.Signal.Dispatch.html#dispatch/2-parameters"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.dispatch/2","doc":"# Single destination\n    iex> config = {:pid, [target: {:pid, pid}, delivery_mode: :async]}\n    iex> Jido.Signal.Dispatch.dispatch(signal, config)\n    :ok\n\n    # Multiple destinations\n    iex> config = [\n    ...>   {:bus, [target: {:bus, :default}, stream: \"events\"]},\n    ...>   {:pubsub, [target: :audit, topic: \"audit\"]}\n    ...> ]\n    iex> Jido.Signal.Dispatch.dispatch(signal, config)\n    :ok","ref":"Jido.Signal.Dispatch.html#dispatch/2-examples"},{"type":"function","title":"Jido.Signal.Dispatch.dispatch_async/2","doc":"Dispatches a signal asynchronously using the provided configuration.\n\nReturns immediately with a task that can be monitored for completion.","ref":"Jido.Signal.Dispatch.html#dispatch_async/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.dispatch_async/2","doc":"- `signal` - The signal to dispatch\n- `config` - Either a single dispatch configuration tuple or a list of configurations","ref":"Jido.Signal.Dispatch.html#dispatch_async/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.dispatch_async/2","doc":"- `{:ok, task}` where task is a Task that can be awaited\n- `{:error, reason}` if the configuration is invalid","ref":"Jido.Signal.Dispatch.html#dispatch_async/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.dispatch_async/2","doc":"{:ok, task} = Dispatch.dispatch_async(signal, config)\n    :ok = Task.await(task)","ref":"Jido.Signal.Dispatch.html#dispatch_async/2-examples"},{"type":"function","title":"Jido.Signal.Dispatch.dispatch_batch/3","doc":"Dispatches a signal to multiple destinations in batches.\n\nThis is useful when dispatching to a large number of destinations to avoid\noverwhelming the system. The dispatches are processed in batches of configurable\nsize with configurable concurrency.","ref":"Jido.Signal.Dispatch.html#dispatch_batch/3"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.dispatch_batch/3","doc":"- `signal` - The signal to dispatch\n- `configs` - List of dispatch configurations\n- `opts` - Batch options:\n  * `:batch_size` - Size of each batch (default: 50)\n  * `:max_concurrency` - Maximum number of concurrent batches (default: 5)","ref":"Jido.Signal.Dispatch.html#dispatch_batch/3-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.dispatch_batch/3","doc":"- `:ok` if all dispatches succeed\n- `{:error, errors}` where errors is a list of `{index, reason}` tuples","ref":"Jido.Signal.Dispatch.html#dispatch_batch/3-returns"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.dispatch_batch/3","doc":"configs = List.duplicate({:pid, [target: pid]}, 1000)\n    :ok = Dispatch.dispatch_batch(signal, configs, batch_size: 100)","ref":"Jido.Signal.Dispatch.html#dispatch_batch/3-examples"},{"type":"type","title":"Jido.Signal.Dispatch.dispatch_config/0","doc":"","ref":"Jido.Signal.Dispatch.html#t:dispatch_config/0"},{"type":"type","title":"Jido.Signal.Dispatch.dispatch_configs/0","doc":"","ref":"Jido.Signal.Dispatch.html#t:dispatch_configs/0"},{"type":"function","title":"Jido.Signal.Dispatch.validate_opts/1","doc":"Validates a dispatch configuration.","ref":"Jido.Signal.Dispatch.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.validate_opts/1","doc":"- `config` - Either a single dispatch configuration tuple or a list of dispatch configurations","ref":"Jido.Signal.Dispatch.html#validate_opts/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.validate_opts/1","doc":"- `{:ok, config}` if the configuration is valid\n- `{:error, reason}` if the configuration is invalid","ref":"Jido.Signal.Dispatch.html#validate_opts/1-returns"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.validate_opts/1","doc":"# Single config\n    iex> config = {:pid, [target: {:pid, self()}, delivery_mode: :async]}\n    iex> Jido.Signal.Dispatch.validate_opts(config)\n    {:ok, ^config}\n\n    # Multiple configs\n    iex> config = [\n    ...>   {:bus, [target: {:bus, :default}, stream: \"events\"]},\n    ...>   {:pubsub, [target: {:pubsub, :audit}, topic: \"audit\"]}\n    ...> ]\n    iex> Jido.Signal.Dispatch.validate_opts(config)\n    {:ok, ^config}","ref":"Jido.Signal.Dispatch.html#validate_opts/1-examples"},{"type":"behaviour","title":"Jido.Signal.Dispatch.Adapter","doc":"Defines the behaviour for signal dispatch adapters in the Jido system.\n\nThis behaviour specifies the contract that all signal dispatch adapters must implement.\nAdapters are responsible for validating their configuration options and delivering\nsignals to their respective destinations.","ref":"Jido.Signal.Dispatch.Adapter.html"},{"type":"behaviour","title":"Implementing an Adapter - Jido.Signal.Dispatch.Adapter","doc":"To create a custom adapter, implement this behaviour in your module:\n\n    defmodule MyApp.CustomAdapter do\n      @behaviour Jido.Signal.Dispatch.Adapter\n\n      @impl true\n      def validate_opts(opts) do\n        # Validate the options and return {:ok, validated_opts} or {:error, reason}\n      end\n\n      @impl true\n      def deliver(signal, opts) do\n        # Deliver the signal using the validated options\n        # Return :ok on success or {:error, reason} on failure\n      end\n    end","ref":"Jido.Signal.Dispatch.Adapter.html#module-implementing-an-adapter"},{"type":"behaviour","title":"Required Callbacks - Jido.Signal.Dispatch.Adapter","doc":"* `validate_opts/1` - Validates adapter-specific options before use\n* `deliver/2` - Handles the actual delivery of signals to their destination\n\nSee the callback documentation for detailed specifications.","ref":"Jido.Signal.Dispatch.Adapter.html#module-required-callbacks"},{"type":"callback","title":"Jido.Signal.Dispatch.Adapter.deliver/2","doc":"Delivers a signal using the validated options.\n\nThis callback handles the actual delivery of the signal to its destination. The options\npassed to this function will have already been validated by `validate_opts/1`.","ref":"Jido.Signal.Dispatch.Adapter.html#c:deliver/2"},{"type":"callback","title":"Parameters - Jido.Signal.Dispatch.Adapter.deliver/2","doc":"* `signal` - The signal to deliver (type: `Jido.Signal.t()`)\n* `opts` - Keyword list of validated adapter-specific options","ref":"Jido.Signal.Dispatch.Adapter.html#c:deliver/2-parameters"},{"type":"callback","title":"Returns - Jido.Signal.Dispatch.Adapter.deliver/2","doc":"* `:ok` - Signal was successfully delivered\n* `{:error, reason}` - Delivery failed with reason for failure","ref":"Jido.Signal.Dispatch.Adapter.html#c:deliver/2-returns"},{"type":"callback","title":"Jido.Signal.Dispatch.Adapter.validate_opts/1","doc":"Validates the adapter-specific options.\n\nThis callback is called before any signal delivery to ensure the adapter is properly configured.\nIt should validate all required options are present and have valid values.","ref":"Jido.Signal.Dispatch.Adapter.html#c:validate_opts/1"},{"type":"callback","title":"Parameters - Jido.Signal.Dispatch.Adapter.validate_opts/1","doc":"* `opts` - Keyword list of adapter-specific options","ref":"Jido.Signal.Dispatch.Adapter.html#c:validate_opts/1-parameters"},{"type":"callback","title":"Returns - Jido.Signal.Dispatch.Adapter.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid and possibly normalized\n* `{:error, reason}` - Options are invalid with reason for failure","ref":"Jido.Signal.Dispatch.Adapter.html#c:validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.ConsoleAdapter","doc":"An adapter for dispatching signals directly to the console output (stdout).\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and provides\nfunctionality to print signals in a human-readable format to the console. It's\nparticularly useful for:\n\n* Interactive development and debugging in IEx sessions\n* Local development and testing\n* Command-line tools and scripts\n* Direct observation of signal flow","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html"},{"type":"module","title":"Output Format - Jido.Signal.Dispatch.ConsoleAdapter","doc":"The adapter prints signals in a structured, easy-to-read format:\n\n```\n[2024-03-21T10:15:30Z] SIGNAL DISPATCHED\nid=signal_id_here\ntype=signal_type_here\nsource=signal_source_here\nmetadata={...}\ndata={...}\n```\n\nFeatures of the output:\n* ISO 8601 timestamp in UTC\n* Clear section headers\n* Pretty-printed metadata and data\n* Consistent formatting for easy parsing","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#module-output-format"},{"type":"module","title":"Configuration - Jido.Signal.Dispatch.ConsoleAdapter","doc":"This adapter requires no configuration options. Any provided options will be ignored.","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#module-configuration"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.ConsoleAdapter","doc":"# Basic usage\n    config = {:console, []}\n\n    # Options are ignored but allowed\n    config = {:console, [\n      any_option: :is_ignored\n    ]}","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#module-examples"},{"type":"module","title":"Use Cases - Jido.Signal.Dispatch.ConsoleAdapter","doc":"* Development and debugging\n* Local testing and verification\n* Signal flow monitoring\n* Educational purposes and demonstrations\n* Command-line tools","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#module-use-cases"},{"type":"module","title":"Notes - Jido.Signal.Dispatch.ConsoleAdapter","doc":"* Output is always sent to stdout\n* Data structures are pretty-printed for readability\n* Timestamps are always in UTC\n* No configuration options are required or used","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#module-notes"},{"type":"function","title":"Jido.Signal.Dispatch.ConsoleAdapter.deliver/2","doc":"Prints a signal to the console in a human-readable format.","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.ConsoleAdapter.deliver/2","doc":"* `signal` - The signal to print\n* `_opts` - Options (ignored)","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#deliver/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.ConsoleAdapter.deliver/2","doc":"* `:ok` - Signal was printed successfully","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#deliver/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.ConsoleAdapter.deliver/2","doc":"iex> signal = %Jido.Signal{\n    ...>   id: \"123\",\n    ...>   type: \"user:created\",\n    ...>   source: \"user_service\",\n    ...>   data: %{id: 456, name: \"John\"}\n    ...> }\n    iex> ConsoleAdapter.deliver(signal, [])\n    [2024-03-21T10:15:30Z] SIGNAL DISPATCHED\n    id=123\n    type=user:created\n    source=user_service\n    metadata={}\n    data=%{id: 456, name: \"John\"}\n    :ok","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#deliver/2-examples"},{"type":"function","title":"Jido.Signal.Dispatch.ConsoleAdapter.validate_opts/1","doc":"Validates the console adapter configuration options.\n\nThis adapter accepts any options but doesn't use them. All options are\nconsidered valid.","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.ConsoleAdapter.validate_opts/1","doc":"* `opts` - Keyword list of options (ignored)","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#validate_opts/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.ConsoleAdapter.validate_opts/1","doc":"* `{:ok, opts}` - Always returns ok with the unchanged options","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.Http","doc":"An adapter for dispatching signals via HTTP requests using Erlang's built-in :httpc client.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and provides\nfunctionality to send signals as HTTP requests to specified endpoints. It uses the\nbuilt-in :httpc client to avoid external dependencies.","ref":"Jido.Signal.Dispatch.Http.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Dispatch.Http","doc":"* `:url` - (required) The URL to send the request to\n* `:method` - (optional) HTTP method to use, one of [:post, :put, :patch], defaults to :post\n* `:headers` - (optional) List of headers to include in the request\n* `:timeout` - (optional) Request timeout in milliseconds, defaults to 5000\n* `:retry` - (optional) Retry configuration map with keys:\n  * `:max_attempts` - Maximum number of retry attempts (default: 3)\n  * `:base_delay` - Base delay between retries in milliseconds (default: 1000)\n  * `:max_delay` - Maximum delay between retries in milliseconds (default: 5000)","ref":"Jido.Signal.Dispatch.Http.html#module-configuration-options"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.Http","doc":"# Basic POST request\n    config = {:http, [\n      url: \"https://api.example.com/events\",\n    ]}\n\n    # Custom configuration\n    config = {:http, [\n      url: \"https://api.example.com/events\",\n      method: :put,\n      headers: [{\"content-type\", \"application/json\"}, {\"x-api-key\", \"secret\"}],\n      timeout: 10_000,\n      retry: %{\n        max_attempts: 5,\n        base_delay: 2000,\n        max_delay: 10000\n      }\n    ]}","ref":"Jido.Signal.Dispatch.Http.html#module-examples"},{"type":"module","title":"Error Handling - Jido.Signal.Dispatch.Http","doc":"The adapter handles these error conditions:\n\n* `:invalid_url` - The URL is not valid\n* `:connection_error` - Failed to establish connection\n* `:timeout` - Request timed out\n* `:retry_failed` - All retry attempts failed\n* Other HTTP status codes and errors","ref":"Jido.Signal.Dispatch.Http.html#module-error-handling"},{"type":"function","title":"Jido.Signal.Dispatch.Http.deliver/2","doc":"Delivers a signal via HTTP request.","ref":"Jido.Signal.Dispatch.Http.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.Http.deliver/2","doc":"* `signal` - The signal to deliver\n* `opts` - Validated options from `validate_opts/1`","ref":"Jido.Signal.Dispatch.Http.html#deliver/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.Http.deliver/2","doc":"* `:ok` - Signal was delivered successfully\n* `{:error, reason}` - Delivery failed with reason","ref":"Jido.Signal.Dispatch.Http.html#deliver/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.Http.deliver/2","doc":"iex> signal = %Jido.Signal{type: \"user:created\", data: %{id: 123}}\n    iex> Http.deliver(signal, [url: \"https://api.example.com/events\"])\n    :ok","ref":"Jido.Signal.Dispatch.Http.html#deliver/2-examples"},{"type":"type","title":"Jido.Signal.Dispatch.Http.delivery_error/0","doc":"","ref":"Jido.Signal.Dispatch.Http.html#t:delivery_error/0"},{"type":"type","title":"Jido.Signal.Dispatch.Http.delivery_opts/0","doc":"","ref":"Jido.Signal.Dispatch.Http.html#t:delivery_opts/0"},{"type":"type","title":"Jido.Signal.Dispatch.Http.header/0","doc":"","ref":"Jido.Signal.Dispatch.Http.html#t:header/0"},{"type":"type","title":"Jido.Signal.Dispatch.Http.http_method/0","doc":"","ref":"Jido.Signal.Dispatch.Http.html#t:http_method/0"},{"type":"type","title":"Jido.Signal.Dispatch.Http.retry_config/0","doc":"","ref":"Jido.Signal.Dispatch.Http.html#t:retry_config/0"},{"type":"function","title":"Jido.Signal.Dispatch.Http.validate_opts/1","doc":"Validates the HTTP adapter configuration options.","ref":"Jido.Signal.Dispatch.Http.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.Http.validate_opts/1","doc":"* `opts` - Keyword list of options to validate","ref":"Jido.Signal.Dispatch.Http.html#validate_opts/1-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.Http.validate_opts/1","doc":"* `:url` - Must be a valid URL string\n* `:method` - Must be one of [:post, :put, :patch]\n* `:headers` - Must be a list of string tuples\n* `:timeout` - Must be a positive integer\n* `:retry` - Must be a valid retry configuration map","ref":"Jido.Signal.Dispatch.Http.html#validate_opts/1-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.Http.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid\n* `{:error, reason}` - Options are invalid with reason","ref":"Jido.Signal.Dispatch.Http.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.LoggerAdapter","doc":"An adapter for dispatching signals through Elixir's Logger system.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and provides\nfunctionality to log signals using Elixir's built-in Logger. It supports both\nstructured and unstructured logging formats and respects configured log levels.","ref":"Jido.Signal.Dispatch.LoggerAdapter.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Dispatch.LoggerAdapter","doc":"* `:level` - (optional) The log level to use, one of [:debug, :info, :warning, :error], defaults to `:info`\n* `:structured` - (optional) Whether to use structured logging format, defaults to `false`","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-configuration-options"},{"type":"module","title":"Logging Formats - Jido.Signal.Dispatch.LoggerAdapter","doc":"","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-logging-formats"},{"type":"module","title":"Unstructured (default) - Jido.Signal.Dispatch.LoggerAdapter","doc":"```\nSignal dispatched: signal_type from source with data={...}\n```","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-unstructured-default"},{"type":"module","title":"Structured - Jido.Signal.Dispatch.LoggerAdapter","doc":"```elixir\n%{\n  event: \"signal_dispatched\",\n  id: \"signal_id\",\n  type: \"signal_type\",\n  data: {...},\n  source: \"source\"\n}\n```","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-structured"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.LoggerAdapter","doc":"# Basic usage with default level\n    config = {:logger, []}\n\n    # Custom log level\n    config = {:logger, [\n      level: :debug\n    ]}\n\n    # Structured logging\n    config = {:logger, [\n      level: :info,\n      structured: true\n    ]}","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-examples"},{"type":"module","title":"Integration with Logger - Jido.Signal.Dispatch.LoggerAdapter","doc":"The adapter integrates with Elixir's Logger system, which means:\n* Log messages respect the global Logger configuration\n* Metadata and formatting can be customized through Logger backends\n* Log levels can be filtered at runtime\n* Structured logging can be processed by log aggregators","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-integration-with-logger"},{"type":"module","title":"Notes - Jido.Signal.Dispatch.LoggerAdapter","doc":"* Consider using structured logging when integrating with log aggregation systems\n* Log levels should be chosen based on the signal's importance\n* High-volume signals should use `:debug` level to avoid log spam","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-notes"},{"type":"function","title":"Jido.Signal.Dispatch.LoggerAdapter.deliver/2","doc":"Logs a signal using the configured format and level.","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.LoggerAdapter.deliver/2","doc":"* `signal` - The signal to log\n* `opts` - Validated options from `validate_opts/1`","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#deliver/2-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.LoggerAdapter.deliver/2","doc":"* `:level` - (optional) The log level to use, defaults to `:info`\n* `:structured` - (optional) Whether to use structured format, defaults to `false`","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#deliver/2-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.LoggerAdapter.deliver/2","doc":"* `:ok` - Signal was logged successfully","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#deliver/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.LoggerAdapter.deliver/2","doc":"iex> signal = %Jido.Signal{type: \"user:created\", data: %{id: 123}}\n    iex> LoggerAdapter.deliver(signal, [level: :info])\n    :ok\n    # Logs: \"Signal dispatched: user:created from source with data=%{id: 123}\"\n\n    iex> LoggerAdapter.deliver(signal, [level: :info, structured: true])\n    :ok\n    # Logs structured map with event details","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#deliver/2-examples"},{"type":"function","title":"Jido.Signal.Dispatch.LoggerAdapter.validate_opts/1","doc":"Validates the logger adapter configuration options.","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.LoggerAdapter.validate_opts/1","doc":"* `opts` - Keyword list of options to validate","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#validate_opts/1-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.LoggerAdapter.validate_opts/1","doc":"* `:level` - (optional) One of [:debug, :info, :warning, :error], defaults to `:info`\n* `:structured` - (optional) Boolean, defaults to `false`","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#validate_opts/1-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.LoggerAdapter.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid\n* `{:error, reason}` - Options are invalid with string reason","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.Named","doc":"An adapter for dispatching signals to named processes in the Erlang registry.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and provides\nfunctionality to send signals to processes registered under a name in the Erlang\nprocess registry. It supports both synchronous and asynchronous delivery modes.","ref":"Jido.Signal.Dispatch.Named.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Dispatch.Named","doc":"* `:target` - (required) A tuple of `{:name, atom()}` specifying the registered process name\n* `:delivery_mode` - (optional) Either `:sync` or `:async`, defaults to `:async`\n* `:timeout` - (optional) Timeout for synchronous delivery in milliseconds, defaults to 5000\n* `:message_format` - (optional) Function to format the signal before sending, defaults to wrapping in `{:signal, signal}`","ref":"Jido.Signal.Dispatch.Named.html#module-configuration-options"},{"type":"module","title":"Delivery Modes - Jido.Signal.Dispatch.Named","doc":"* `:async` - Uses `send/2` to deliver the signal without waiting for a response\n* `:sync` - Uses `GenServer.call/3` to deliver the signal and wait for a response","ref":"Jido.Signal.Dispatch.Named.html#module-delivery-modes"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.Named","doc":"# Asynchronous delivery to named process\n    config = {:named, [\n      target: {:name, :my_process},\n      delivery_mode: :async\n    ]}\n\n    # Synchronous delivery with custom timeout\n    config = {:named, [\n      target: {:name, :my_process},\n      delivery_mode: :sync,\n      timeout: 10_000\n    ]}\n\n    # Custom message format\n    config = {:named, [\n      target: {:name, :my_process},\n      message_format: fn signal -> {:custom_signal, signal} end\n    ]}","ref":"Jido.Signal.Dispatch.Named.html#module-examples"},{"type":"module","title":"Error Handling - Jido.Signal.Dispatch.Named","doc":"The adapter handles these error conditions:\n\n* `:process_not_found` - The named process is not registered\n* `:process_not_alive` - The process exists but is not alive\n* `:timeout` - Synchronous delivery timed out\n* Other errors from the target process","ref":"Jido.Signal.Dispatch.Named.html#module-error-handling"},{"type":"function","title":"Jido.Signal.Dispatch.Named.deliver/2","doc":"Delivers a signal to the named process.","ref":"Jido.Signal.Dispatch.Named.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.Named.deliver/2","doc":"* `signal` - The signal to deliver\n* `opts` - Validated options from `validate_opts/1`","ref":"Jido.Signal.Dispatch.Named.html#deliver/2-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.Named.deliver/2","doc":"* `:target` - (required) Tuple of `{:name, atom()}` identifying the process\n* `:delivery_mode` - (required) Either `:sync` or `:async`\n* `:timeout` - (optional) Timeout for sync delivery, defaults to 5000ms\n* `:message_format` - (optional) Function to format the signal","ref":"Jido.Signal.Dispatch.Named.html#deliver/2-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.Named.deliver/2","doc":"* `:ok` - Signal delivered successfully (async mode)\n* `{:ok, term()}` - Signal delivered and response received (sync mode)\n* `{:error, reason}` - Delivery failed","ref":"Jido.Signal.Dispatch.Named.html#deliver/2-returns"},{"type":"type","title":"Jido.Signal.Dispatch.Named.delivery_error/0","doc":"","ref":"Jido.Signal.Dispatch.Named.html#t:delivery_error/0"},{"type":"type","title":"Jido.Signal.Dispatch.Named.delivery_mode/0","doc":"","ref":"Jido.Signal.Dispatch.Named.html#t:delivery_mode/0"},{"type":"type","title":"Jido.Signal.Dispatch.Named.delivery_opts/0","doc":"","ref":"Jido.Signal.Dispatch.Named.html#t:delivery_opts/0"},{"type":"type","title":"Jido.Signal.Dispatch.Named.delivery_target/0","doc":"","ref":"Jido.Signal.Dispatch.Named.html#t:delivery_target/0"},{"type":"type","title":"Jido.Signal.Dispatch.Named.message_format/0","doc":"","ref":"Jido.Signal.Dispatch.Named.html#t:message_format/0"},{"type":"function","title":"Jido.Signal.Dispatch.Named.validate_opts/1","doc":"Validates the named process adapter configuration options.","ref":"Jido.Signal.Dispatch.Named.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.Named.validate_opts/1","doc":"* `opts` - Keyword list of options to validate","ref":"Jido.Signal.Dispatch.Named.html#validate_opts/1-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.Named.validate_opts/1","doc":"* `:target` - Must be a tuple of `{:name, atom()}`\n* `:delivery_mode` - Must be either `:sync` or `:async`","ref":"Jido.Signal.Dispatch.Named.html#validate_opts/1-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.Named.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid\n* `{:error, :invalid_target}` - Target is not a valid name tuple\n* `{:error, :invalid_delivery_mode}` - Delivery mode is invalid","ref":"Jido.Signal.Dispatch.Named.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.NoopAdapter","doc":"A no-operation adapter that silently discards all signals.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour but performs\nno actual signal delivery. It's useful for:\n\n* Testing and development environments where signals should be ignored\n* Disabling signal delivery without changing configuration structure\n* Performance testing without signal delivery overhead\n* Mocking signal dispatch in test environments","ref":"Jido.Signal.Dispatch.NoopAdapter.html"},{"type":"module","title":"Configuration - Jido.Signal.Dispatch.NoopAdapter","doc":"This adapter accepts any options but ignores them all. No configuration is needed\nor used.","ref":"Jido.Signal.Dispatch.NoopAdapter.html#module-configuration"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.NoopAdapter","doc":"# Basic usage\n    config = {:noop, []}\n\n    # Options are allowed but ignored\n    config = {:noop, [\n      any_option: :is_ignored\n    ]}","ref":"Jido.Signal.Dispatch.NoopAdapter.html#module-examples"},{"type":"module","title":"Use Cases - Jido.Signal.Dispatch.NoopAdapter","doc":"* Test environments\n* Development configurations\n* Performance benchmarking\n* Signal flow testing\n* Temporary signal suppression","ref":"Jido.Signal.Dispatch.NoopAdapter.html#module-use-cases"},{"type":"module","title":"Notes - Jido.Signal.Dispatch.NoopAdapter","doc":"* All signals are silently discarded\n* No errors are ever returned\n* No side effects are produced\n* Completely thread-safe and concurrent","ref":"Jido.Signal.Dispatch.NoopAdapter.html#module-notes"},{"type":"function","title":"Jido.Signal.Dispatch.NoopAdapter.deliver/2","doc":"Silently discards the signal without performing any operation.","ref":"Jido.Signal.Dispatch.NoopAdapter.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.NoopAdapter.deliver/2","doc":"* `_signal` - The signal to discard (ignored)\n* `_opts` - Options (ignored)","ref":"Jido.Signal.Dispatch.NoopAdapter.html#deliver/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.NoopAdapter.deliver/2","doc":"* `:ok` - Always returns `:ok`","ref":"Jido.Signal.Dispatch.NoopAdapter.html#deliver/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.NoopAdapter.deliver/2","doc":"iex> signal = %Jido.Signal{type: \"test:event\"}\n    iex> NoopAdapter.deliver(signal, [])\n    :ok","ref":"Jido.Signal.Dispatch.NoopAdapter.html#deliver/2-examples"},{"type":"function","title":"Jido.Signal.Dispatch.NoopAdapter.validate_opts/1","doc":"Validates the noop adapter configuration options.\n\nThis adapter accepts any options but doesn't use them. All options are\nconsidered valid.","ref":"Jido.Signal.Dispatch.NoopAdapter.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.NoopAdapter.validate_opts/1","doc":"* `opts` - Keyword list of options (ignored)","ref":"Jido.Signal.Dispatch.NoopAdapter.html#validate_opts/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.NoopAdapter.validate_opts/1","doc":"* `{:ok, opts}` - Always returns ok with the unchanged options","ref":"Jido.Signal.Dispatch.NoopAdapter.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.PidAdapter","doc":"An adapter for dispatching signals directly to Erlang processes using PIDs.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and provides\nfunctionality to send signals to specific processes either synchronously or\nasynchronously.","ref":"Jido.Signal.Dispatch.PidAdapter.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Dispatch.PidAdapter","doc":"* `:target` - (required) The PID of the destination process\n* `:delivery_mode` - (optional) Either `:sync` or `:async`, defaults to `:async`\n* `:timeout` - (optional) Timeout for synchronous delivery in milliseconds, defaults to 5000\n* `:message_format` - (optional) Function to format the signal before sending, defaults to wrapping in `{:signal, signal}`","ref":"Jido.Signal.Dispatch.PidAdapter.html#module-configuration-options"},{"type":"module","title":"Delivery Modes - Jido.Signal.Dispatch.PidAdapter","doc":"* `:async` - Uses `send/2` to deliver the signal without waiting for a response\n* `:sync` - Uses `GenServer.call/3` to deliver the signal and wait for a response","ref":"Jido.Signal.Dispatch.PidAdapter.html#module-delivery-modes"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.PidAdapter","doc":"# Asynchronous delivery\n    config = {:pid, [\n      target: destination_pid,\n      delivery_mode: :async\n    ]}\n\n    # Synchronous delivery with custom timeout\n    config = {:pid, [\n      target: destination_pid,\n      delivery_mode: :sync,\n      timeout: 10_000\n    ]}\n\n    # Custom message format\n    config = {:pid, [\n      target: destination_pid,\n      message_format: fn signal -> {:custom_signal, signal} end\n    ]}","ref":"Jido.Signal.Dispatch.PidAdapter.html#module-examples"},{"type":"module","title":"Error Handling - Jido.Signal.Dispatch.PidAdapter","doc":"The adapter handles various error conditions:\n\n* `:process_not_alive` - The target process is not alive\n* `:timeout` - Synchronous delivery timed out\n* Other errors from the target process","ref":"Jido.Signal.Dispatch.PidAdapter.html#module-error-handling"},{"type":"function","title":"Jido.Signal.Dispatch.PidAdapter.deliver/2","doc":"Delivers a signal to the target process.","ref":"Jido.Signal.Dispatch.PidAdapter.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.PidAdapter.deliver/2","doc":"* `signal` - The signal to deliver\n* `opts` - Validated options from `validate_opts/1`","ref":"Jido.Signal.Dispatch.PidAdapter.html#deliver/2-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.PidAdapter.deliver/2","doc":"* `:target` - (required) Target PID to deliver to\n* `:delivery_mode` - (required) Either `:sync` or `:async`\n* `:timeout` - (optional) Timeout for sync delivery, defaults to 5000ms\n* `:message_format` - (optional) Function to format the signal","ref":"Jido.Signal.Dispatch.PidAdapter.html#deliver/2-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.PidAdapter.deliver/2","doc":"* `:ok` - Signal delivered successfully (async mode)\n* `{:ok, term()}` - Signal delivered and response received (sync mode)\n* `{:error, reason}` - Delivery failed","ref":"Jido.Signal.Dispatch.PidAdapter.html#deliver/2-returns"},{"type":"type","title":"Jido.Signal.Dispatch.PidAdapter.delivery_error/0","doc":"","ref":"Jido.Signal.Dispatch.PidAdapter.html#t:delivery_error/0"},{"type":"type","title":"Jido.Signal.Dispatch.PidAdapter.delivery_mode/0","doc":"","ref":"Jido.Signal.Dispatch.PidAdapter.html#t:delivery_mode/0"},{"type":"type","title":"Jido.Signal.Dispatch.PidAdapter.delivery_opts/0","doc":"","ref":"Jido.Signal.Dispatch.PidAdapter.html#t:delivery_opts/0"},{"type":"type","title":"Jido.Signal.Dispatch.PidAdapter.delivery_target/0","doc":"","ref":"Jido.Signal.Dispatch.PidAdapter.html#t:delivery_target/0"},{"type":"type","title":"Jido.Signal.Dispatch.PidAdapter.message_format/0","doc":"","ref":"Jido.Signal.Dispatch.PidAdapter.html#t:message_format/0"},{"type":"function","title":"Jido.Signal.Dispatch.PidAdapter.validate_opts/1","doc":"Validates the PID adapter configuration options.","ref":"Jido.Signal.Dispatch.PidAdapter.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.PidAdapter.validate_opts/1","doc":"* `opts` - Keyword list of options to validate","ref":"Jido.Signal.Dispatch.PidAdapter.html#validate_opts/1-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.PidAdapter.validate_opts/1","doc":"* `:target` - Must be a valid PID\n* `:delivery_mode` - Must be either `:sync` or `:async`","ref":"Jido.Signal.Dispatch.PidAdapter.html#validate_opts/1-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.PidAdapter.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid\n* `{:error, :invalid_target}` - Target is not a valid PID\n* `{:error, :invalid_delivery_mode}` - Delivery mode is invalid","ref":"Jido.Signal.Dispatch.PidAdapter.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.PubSub","doc":"An adapter for dispatching signals through Phoenix.PubSub.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and provides\nfunctionality to broadcast signals through Phoenix.PubSub to all subscribers of a\nspecific topic. It integrates with Phoenix's pub/sub system for distributed\nmessage broadcasting.","ref":"Jido.Signal.Dispatch.PubSub.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Dispatch.PubSub","doc":"* `:target` - (required) An atom specifying the PubSub server name\n* `:topic` - (required) A string specifying the topic to broadcast on","ref":"Jido.Signal.Dispatch.PubSub.html#module-configuration-options"},{"type":"module","title":"Phoenix.PubSub Integration - Jido.Signal.Dispatch.PubSub","doc":"The adapter uses `Phoenix.PubSub.broadcast/3` to:\n* Broadcast signals to all subscribers of a topic\n* Handle distributed message delivery across nodes\n* Manage subscription-based message routing","ref":"Jido.Signal.Dispatch.PubSub.html#module-phoenix-pubsub-integration"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.PubSub","doc":"# Basic usage\n    config = {:pubsub, [\n      target: :my_app_pubsub,\n      topic: \"events\"\n    ]}\n\n    # Using with specific event topics\n    config = {:pubsub, [\n      target: :my_app_pubsub,\n      topic: \"user:123:events\"\n    ]}","ref":"Jido.Signal.Dispatch.PubSub.html#module-examples"},{"type":"module","title":"Error Handling - Jido.Signal.Dispatch.PubSub","doc":"The adapter handles these error conditions:\n\n* `:pubsub_not_found` - The target PubSub server is not running\n* Other errors from the Phoenix.PubSub system","ref":"Jido.Signal.Dispatch.PubSub.html#module-error-handling"},{"type":"module","title":"Notes - Jido.Signal.Dispatch.PubSub","doc":"* Ensure the PubSub server is started in your application supervision tree\n* Topics can be any string, but consider using consistent naming patterns\n* Messages are broadcast to all subscribers, so consider message volume","ref":"Jido.Signal.Dispatch.PubSub.html#module-notes"},{"type":"function","title":"Jido.Signal.Dispatch.PubSub.deliver/2","doc":"Broadcasts a signal through Phoenix.PubSub.","ref":"Jido.Signal.Dispatch.PubSub.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.PubSub.deliver/2","doc":"* `signal` - The signal to broadcast\n* `opts` - Validated options from `validate_opts/1`","ref":"Jido.Signal.Dispatch.PubSub.html#deliver/2-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.PubSub.deliver/2","doc":"* `:target` - (required) Atom identifying the PubSub server\n* `:topic` - (required) String topic to broadcast on","ref":"Jido.Signal.Dispatch.PubSub.html#deliver/2-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.PubSub.deliver/2","doc":"* `:ok` - Signal broadcast successfully\n* `{:error, :pubsub_not_found}` - PubSub server not found\n* `{:error, reason}` - Other broadcast failure","ref":"Jido.Signal.Dispatch.PubSub.html#deliver/2-returns"},{"type":"type","title":"Jido.Signal.Dispatch.PubSub.delivery_error/0","doc":"","ref":"Jido.Signal.Dispatch.PubSub.html#t:delivery_error/0"},{"type":"type","title":"Jido.Signal.Dispatch.PubSub.delivery_opts/0","doc":"","ref":"Jido.Signal.Dispatch.PubSub.html#t:delivery_opts/0"},{"type":"type","title":"Jido.Signal.Dispatch.PubSub.delivery_target/0","doc":"","ref":"Jido.Signal.Dispatch.PubSub.html#t:delivery_target/0"},{"type":"function","title":"Jido.Signal.Dispatch.PubSub.validate_opts/1","doc":"Validates the PubSub adapter configuration options.","ref":"Jido.Signal.Dispatch.PubSub.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.PubSub.validate_opts/1","doc":"* `opts` - Keyword list of options to validate","ref":"Jido.Signal.Dispatch.PubSub.html#validate_opts/1-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.PubSub.validate_opts/1","doc":"* `:target` - Must be an atom representing the PubSub server name\n* `:topic` - Must be a string","ref":"Jido.Signal.Dispatch.PubSub.html#validate_opts/1-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.PubSub.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid\n* `{:error, reason}` - Options are invalid with string reason","ref":"Jido.Signal.Dispatch.PubSub.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.Webhook","doc":"An adapter for dispatching signals to webhooks.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and extends\nthe HTTP adapter with webhook-specific functionality. It provides features like:\n\n* Signature generation for webhook payloads\n* Standard webhook headers\n* Webhook-specific retry policies\n* Event type mapping","ref":"Jido.Signal.Dispatch.Webhook.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Dispatch.Webhook","doc":"* `:url` - (required) The webhook URL to send the request to\n* `:secret` - (optional) Secret key for generating signatures\n* `:signature_header` - (optional) Header name for the signature, defaults to \"x-webhook-signature\"\n* `:event_type_header` - (optional) Header name for the event type, defaults to \"x-webhook-event\"\n* `:event_type_map` - (optional) Map of signal types to webhook event types\n* All other options from `Jido.Signal.Dispatch.Http`","ref":"Jido.Signal.Dispatch.Webhook.html#module-configuration-options"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.Webhook","doc":"# Basic webhook with signature\n    config = {:webhook, [\n      url: \"https://api.example.com/webhook\",\n      secret: \"webhook_secret\"\n    ]}\n\n    # Advanced configuration\n    config = {:webhook, [\n      url: \"https://api.example.com/webhook\",\n      secret: \"webhook_secret\",\n      signature_header: \"x-signature\",\n      event_type_header: \"x-event-type\",\n      event_type_map: %{\n        \"user:created\" => \"user.created\",\n        \"user:updated\" => \"user.updated\"\n      },\n      timeout: 10_000,\n      retry: %{\n        max_attempts: 5,\n        base_delay: 2000,\n        max_delay: 10000\n      }\n    ]}","ref":"Jido.Signal.Dispatch.Webhook.html#module-examples"},{"type":"module","title":"Webhook Signatures - Jido.Signal.Dispatch.Webhook","doc":"When a secret is provided, the adapter will:\n1. Generate a signature of the payload using HMAC-SHA256\n2. Add the signature to the request headers\n3. Include a timestamp to prevent replay attacks","ref":"Jido.Signal.Dispatch.Webhook.html#module-webhook-signatures"},{"type":"module","title":"Event Type Mapping - Jido.Signal.Dispatch.Webhook","doc":"The adapter can map internal signal types to external webhook event types:\n\n    event_type_map = %{\n      \"user:created\" => \"user.created\",\n      \"user:updated\" => \"user.updated\"\n    }\n\nIf no mapping is provided, the original signal type is used.","ref":"Jido.Signal.Dispatch.Webhook.html#module-event-type-mapping"},{"type":"module","title":"Error Handling - Jido.Signal.Dispatch.Webhook","doc":"In addition to HTTP adapter errors, this adapter handles:\n\n* `:invalid_secret` - Secret key is not valid\n* `:invalid_event_type_map` - Event type mapping is invalid","ref":"Jido.Signal.Dispatch.Webhook.html#module-error-handling"},{"type":"function","title":"Jido.Signal.Dispatch.Webhook.deliver/2","doc":"Delivers a signal via webhook.","ref":"Jido.Signal.Dispatch.Webhook.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.Webhook.deliver/2","doc":"* `signal` - The signal to deliver\n* `opts` - Validated options from `validate_opts/1`","ref":"Jido.Signal.Dispatch.Webhook.html#deliver/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.Webhook.deliver/2","doc":"* `:ok` - Signal was delivered successfully\n* `{:error, reason}` - Delivery failed with reason","ref":"Jido.Signal.Dispatch.Webhook.html#deliver/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.Webhook.deliver/2","doc":"iex> signal = %Jido.Signal{type: \"user:created\", data: %{id: 123}}\n    iex> Webhook.deliver(signal, [url: \"https://api.example.com/webhook\", secret: \"secret\"])\n    :ok","ref":"Jido.Signal.Dispatch.Webhook.html#deliver/2-examples"},{"type":"function","title":"Jido.Signal.Dispatch.Webhook.validate_opts/1","doc":"Validates the webhook adapter configuration options.","ref":"Jido.Signal.Dispatch.Webhook.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.Webhook.validate_opts/1","doc":"* `opts` - Keyword list of options to validate","ref":"Jido.Signal.Dispatch.Webhook.html#validate_opts/1-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.Webhook.validate_opts/1","doc":"* `:url` - Must be a valid URL string\n* `:secret` - Must be a string or nil\n* `:signature_header` - Must be a string\n* `:event_type_header` - Must be a string\n* `:event_type_map` - Must be a map of strings to strings","ref":"Jido.Signal.Dispatch.Webhook.html#validate_opts/1-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.Webhook.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid\n* `{:error, reason}` - Options are invalid with reason","ref":"Jido.Signal.Dispatch.Webhook.html#validate_opts/1-returns"},{"type":"type","title":"Jido.Signal.Dispatch.Webhook.webhook_error/0","doc":"","ref":"Jido.Signal.Dispatch.Webhook.html#t:webhook_error/0"},{"type":"type","title":"Jido.Signal.Dispatch.Webhook.webhook_opts/0","doc":"","ref":"Jido.Signal.Dispatch.Webhook.html#t:webhook_opts/0"},{"type":"module","title":"Jido.Signal.Error","doc":"Defines error structures and helper functions for Jido\n\nThis module provides a standardized way to create and handle errors within the Jido system.\nIt offers a set of predefined error types and functions to create, manipulate, and convert\nerror structures consistently across the application.\n\n> Why not use Exceptions?\n>\n> Jido is designed to be a functional system, strictly adhering to the use of result tuples.\n> This approach provides several benefits:\n>\n> 1. Consistent error handling: By using `{:ok, result}` or `{:error, reason}` tuples,\n>    we ensure a uniform way of handling success and failure cases throughout the system.\n>\n> 2. Composability: Monadic actions can be easily chained together, allowing for\n>    cleaner and more maintainable code.\n>\n> 3. Explicit error paths: The use of result tuples makes error cases explicit,\n>    reducing the likelihood of unhandled errors.\n>\n> 4. No silent failures: Unlike exceptions, which can be silently caught and ignored,\n>    result tuples require explicit handling of both success and error cases.\n>\n> 5. Better testability: Monadic actions are easier to test, as both success and\n>    error paths can be explicitly verified.\n>\n> By using this approach instead of exceptions, we gain more control over the flow of our\n> actions and ensure that errors are handled consistently across the entire system.","ref":"Jido.Signal.Error.html"},{"type":"module","title":"Usage - Jido.Signal.Error","doc":"Use this module to create specific error types when exceptions occur in your Jido actions.\nThis allows for consistent error handling and reporting throughout the system.\n\nExample:\n\n    defmodule MyExec do\n      alias Jido.Signal.Error\n\n      def run(params) do\n        case validate(params) do\n          :ok -> perform_action(params)\n          {:error, reason} -> Error.validation_error(\"Invalid parameters\")\n        end\n      end\n    end","ref":"Jido.Signal.Error.html#module-usage"},{"type":"function","title":"Jido.Signal.Error.bad_request/3","doc":"Creates a new bad request error.\n\nUse this when the client sends an invalid or malformed request.","ref":"Jido.Signal.Error.html#bad_request/3"},{"type":"function","title":"Parameters - Jido.Signal.Error.bad_request/3","doc":"- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Signal.Error.html#bad_request/3-parameters"},{"type":"function","title":"Example - Jido.Signal.Error.bad_request/3","doc":"iex> Jido.Signal.Error.bad_request(\"Missing required parameter 'user_id'\")\n    %Jido.Signal.Error{\n      type: :bad_request,\n      message: \"Missing required parameter 'user_id'\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Signal.Error.html#bad_request/3-example"},{"type":"function","title":"Jido.Signal.Error.capture_stacktrace/0","doc":"Captures the current stacktrace.\n\nThis function is useful when you want to capture the stacktrace at a specific point\nin your code, rather than at the point where the error is created. It drops the first\ntwo entries from the stacktrace to remove internal function calls related to this module.","ref":"Jido.Signal.Error.html#capture_stacktrace/0"},{"type":"function","title":"Returns - Jido.Signal.Error.capture_stacktrace/0","doc":"The current stacktrace as a list.","ref":"Jido.Signal.Error.html#capture_stacktrace/0-returns"},{"type":"function","title":"Example - Jido.Signal.Error.capture_stacktrace/0","doc":"iex> stacktrace = Jido.Signal.Error.capture_stacktrace()\n    iex> is_list(stacktrace)\n    true","ref":"Jido.Signal.Error.html#capture_stacktrace/0-example"},{"type":"function","title":"Jido.Signal.Error.dispatch_error/3","doc":"Creates a new dispatch error.\n\nUse this when an error occurs during signal dispatching.","ref":"Jido.Signal.Error.html#dispatch_error/3"},{"type":"function","title":"Parameters - Jido.Signal.Error.dispatch_error/3","doc":"- `message`: A string describing the dispatch error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Signal.Error.html#dispatch_error/3-parameters"},{"type":"function","title":"Example - Jido.Signal.Error.dispatch_error/3","doc":"iex> Jido.Signal.Error.dispatch_error(\"Failed to deliver signal\", %{adapter: :http, reason: :timeout})\n    %Jido.Signal.Error{\n      type: :dispatch_error,\n      message: \"Failed to deliver signal\",\n      details: %{adapter: :http, reason: :timeout},\n      stacktrace: [...]\n    }","ref":"Jido.Signal.Error.html#dispatch_error/3-example"},{"type":"type","title":"Jido.Signal.Error.error_type/0","doc":"Defines the possible error types in the Jido system.\n\n- `:validation_error`: Used when input validation fails.\n- `:execution_error`: Used when an error occurs during action execution.\n- `:timeout`: Used when an action exceeds its time limit.\n- `:planning_error`: Used when an error occurs during action planning.\n- `:routing_error`: Used when an error occurs during action routing.\n- `:dispatch_error`: Used when an error occurs during signal dispatching.","ref":"Jido.Signal.Error.html#t:error_type/0"},{"type":"function","title":"Jido.Signal.Error.execution_error/3","doc":"Creates a new execution error.\n\nUse this when an error occurs during the execution of an action.","ref":"Jido.Signal.Error.html#execution_error/3"},{"type":"function","title":"Parameters - Jido.Signal.Error.execution_error/3","doc":"- `message`: A string describing the execution error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Signal.Error.html#execution_error/3-parameters"},{"type":"function","title":"Example - Jido.Signal.Error.execution_error/3","doc":"iex> Jido.Signal.Error.execution_error(\"Failed to process data\", %{step: \"data_transformation\"})\n    %Jido.Signal.Error{\n      type: :execution_error,\n      message: \"Failed to process data\",\n      details: %{step: \"data_transformation\"},\n      stacktrace: [...]\n    }","ref":"Jido.Signal.Error.html#execution_error/3-example"},{"type":"function","title":"Jido.Signal.Error.format_nimble_config_error/3","doc":"Formats a NimbleOptions validation error for configuration validation.\nUsed when validating configuration options at compile or runtime.","ref":"Jido.Signal.Error.html#format_nimble_config_error/3"},{"type":"function","title":"Parameters - Jido.Signal.Error.format_nimble_config_error/3","doc":"- `error`: The NimbleOptions.ValidationError to format\n- `module_type`: String indicating the module type (e.g. \"Action\", \"Agent\", \"Sensor\")","ref":"Jido.Signal.Error.html#format_nimble_config_error/3-parameters"},{"type":"function","title":"Examples - Jido.Signal.Error.format_nimble_config_error/3","doc":"iex> error = %NimbleOptions.ValidationError{keys_path: [:name], message: \"is required\"}\n    iex> Jido.Signal.Error.format_nimble_config_error(error, \"Action\")\n    \"Invalid configuration given to use Jido.Action for key [:name]: is required\"","ref":"Jido.Signal.Error.html#format_nimble_config_error/3-examples"},{"type":"function","title":"Jido.Signal.Error.format_nimble_validation_error/3","doc":"Formats a NimbleOptions validation error for parameter validation.\nUsed when validating runtime parameters.","ref":"Jido.Signal.Error.html#format_nimble_validation_error/3"},{"type":"function","title":"Parameters - Jido.Signal.Error.format_nimble_validation_error/3","doc":"- `error`: The NimbleOptions.ValidationError to format\n- `module_type`: String indicating the module type (e.g. \"Action\", \"Agent\", \"Sensor\")","ref":"Jido.Signal.Error.html#format_nimble_validation_error/3-parameters"},{"type":"function","title":"Examples - Jido.Signal.Error.format_nimble_validation_error/3","doc":"iex> error = %NimbleOptions.ValidationError{keys_path: [:input], message: \"is required\"}\n    iex> Jido.Signal.Error.format_nimble_validation_error(error, \"Action\")\n    \"Invalid parameters for Action at [:input]: is required\"","ref":"Jido.Signal.Error.html#format_nimble_validation_error/3-examples"},{"type":"function","title":"Jido.Signal.Error.new/4","doc":"Creates a new error struct with the given type and message.\n\nThis is a low-level function used by other error creation functions in this module.\nConsider using the specific error creation functions unless you need fine-grained control.","ref":"Jido.Signal.Error.html#new/4"},{"type":"function","title":"Parameters - Jido.Signal.Error.new/4","doc":"- `type`: The error type (see `t:error_type/0`).\n- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Signal.Error.html#new/4-parameters"},{"type":"function","title":"Examples - Jido.Signal.Error.new/4","doc":"iex> Jido.Signal.Error.new(:config_error, \"Invalid configuration\")\n    %Jido.Signal.Error{\n      type: :config_error,\n      message: \"Invalid configuration\",\n      details: nil,\n      stacktrace: [...]\n    }\n\n    iex> Jido.Signal.Error.new(:execution_error, \"Exec failed\", %{step: \"data_processing\"})\n    %Jido.Signal.Error{\n      type: :execution_error,\n      message: \"Exec failed\",\n      details: %{step: \"data_processing\"},\n      stacktrace: [...]\n    }","ref":"Jido.Signal.Error.html#new/4-examples"},{"type":"function","title":"Jido.Signal.Error.routing_error/3","doc":"Creates a new routing error.\n\nUse this when an error occurs during action routing.","ref":"Jido.Signal.Error.html#routing_error/3"},{"type":"function","title":"Parameters - Jido.Signal.Error.routing_error/3","doc":"- `message`: A string describing the routing error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Signal.Error.html#routing_error/3-parameters"},{"type":"function","title":"Example - Jido.Signal.Error.routing_error/3","doc":"iex> Jido.Signal.Error.routing_error(\"Invalid route configuration\", %{route: \"user_action\"})\n    %Jido.Signal.Error{\n      type: :routing_error,\n      message: \"Invalid route configuration\",\n      details: %{route: \"user_action\"},\n      stacktrace: [...]\n    }","ref":"Jido.Signal.Error.html#routing_error/3-example"},{"type":"type","title":"Jido.Signal.Error.t/0","doc":"Represents a structured error in the Jido system.\n\nFields:\n- `type`: The category of the error (see `t:error_type/0`).\n- `message`: A human-readable description of the error.\n- `details`: Optional map containing additional error context.\n- `stacktrace`: Optional list representing the error's stacktrace.","ref":"Jido.Signal.Error.html#t:t/0"},{"type":"function","title":"Jido.Signal.Error.timeout/3","doc":"Creates a new timeout error.\n\nUse this when an action exceeds its allocated time limit.","ref":"Jido.Signal.Error.html#timeout/3"},{"type":"function","title":"Parameters - Jido.Signal.Error.timeout/3","doc":"- `message`: A string describing the timeout error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Signal.Error.html#timeout/3-parameters"},{"type":"function","title":"Example - Jido.Signal.Error.timeout/3","doc":"iex> Jido.Signal.Error.timeout(\"Exec timed out after 30 seconds\", %{action: \"FetchUserData\"})\n    %Jido.Signal.Error{\n      type: :timeout,\n      message: \"Exec timed out after 30 seconds\",\n      details: %{action: \"FetchUserData\"},\n      stacktrace: [...]\n    }","ref":"Jido.Signal.Error.html#timeout/3-example"},{"type":"function","title":"Jido.Signal.Error.to_map/1","doc":"Converts the error struct to a plain map.\n\nThis function transforms the error struct into a plain map,\nincluding the error type and stacktrace if available. It's useful\nfor serialization or when working with APIs that expect plain maps.","ref":"Jido.Signal.Error.html#to_map/1"},{"type":"function","title":"Parameters - Jido.Signal.Error.to_map/1","doc":"- `error`: An error struct of type `t:t/0`.","ref":"Jido.Signal.Error.html#to_map/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Error.to_map/1","doc":"A map representation of the error.","ref":"Jido.Signal.Error.html#to_map/1-returns"},{"type":"function","title":"Example - Jido.Signal.Error.to_map/1","doc":"iex> error = Jido.Signal.Error.validation_error(\"Invalid input\")\n    iex> Jido.Signal.Error.to_map(error)\n    %{\n      type: :validation_error,\n      message: \"Invalid input\",\n      stacktrace: [...]\n    }","ref":"Jido.Signal.Error.html#to_map/1-example"},{"type":"function","title":"Jido.Signal.Error.validation_error/3","doc":"Creates a new validation error.\n\nUse this when input validation fails for an action.","ref":"Jido.Signal.Error.html#validation_error/3"},{"type":"function","title":"Parameters - Jido.Signal.Error.validation_error/3","doc":"- `message`: A string describing the validation error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Signal.Error.html#validation_error/3-parameters"},{"type":"function","title":"Example - Jido.Signal.Error.validation_error/3","doc":"iex> Jido.Signal.Error.validation_error(\"Invalid email format\", %{field: \"email\", value: \"not-an-email\"})\n    %Jido.Signal.Error{\n      type: :validation_error,\n      message: \"Invalid email format\",\n      details: %{field: \"email\", value: \"not-an-email\"},\n      stacktrace: [...]\n    }","ref":"Jido.Signal.Error.html#validation_error/3-example"},{"type":"module","title":"Jido.Signal.ID","doc":"Manages UUID7-based signal IDs for the bus system. Provides utilities for\ngenerating, comparing, and extracting information from signal IDs.\n\nUUID7s provide several benefits for signal IDs:\n- Monotonically increasing (timestamp-based)\n- Contain embedded timestamps and sequence numbers\n- High uniqueness guarantees within the same millisecond\n- Good sequential scanning performance\n\nUUID7 Format:\n- First 48 bits: Unix timestamp in milliseconds\n- Next 12 bits: Sequence number (monotonic counter within each millisecond)\n- Remaining bits: Random data\n\nThis ensures that:\n1. IDs are globally ordered by timestamp\n2. IDs generated in the same millisecond are ordered by sequence number\n3. IDs with same timestamp and sequence are ordered lexicographically","ref":"Jido.Signal.ID.html"},{"type":"module","title":"Usage Examples - Jido.Signal.ID","doc":"# Generate UUID7\n    {id, timestamp} = Jido.Signal.ID.generate()\n\n    # Compare IDs chronologically\n    :lt = Jido.Signal.ID.compare(older_id, newer_id)\n\n    # Extract components\n    timestamp = Jido.Signal.ID.extract_timestamp(id)\n    sequence = Jido.Signal.ID.sequence_number(id)","ref":"Jido.Signal.ID.html#module-usage-examples"},{"type":"function","title":"Jido.Signal.ID.compare/2","doc":"Compares two UUID7s chronologically.\nReturns `:lt`, `:eq`, or `:gt` based on the following order:\n1. Timestamp comparison\n2. If timestamps match, sequence number comparison\n3. If both match, lexicographical comparison of remaining bits","ref":"Jido.Signal.ID.html#compare/2"},{"type":"function","title":"Examples - Jido.Signal.ID.compare/2","doc":"Jido.Signal.ID.compare(older_id, newer_id)\n    #=> :lt\n\n    Jido.Signal.ID.compare(newer_id, older_id)\n    #=> :gt\n\n    Jido.Signal.ID.compare(id, id)\n    #=> :eq","ref":"Jido.Signal.ID.html#compare/2-examples"},{"type":"type","title":"Jido.Signal.ID.comparison_result/0","doc":"","ref":"Jido.Signal.ID.html#t:comparison_result/0"},{"type":"function","title":"Jido.Signal.ID.extract_timestamp/1","doc":"Extracts the Unix timestamp (in milliseconds) from a UUID7 string.\nUUID7 embeds a 48-bit timestamp in its first 6 bytes.","ref":"Jido.Signal.ID.html#extract_timestamp/1"},{"type":"function","title":"Examples - Jido.Signal.ID.extract_timestamp/1","doc":"ts = Jido.Signal.ID.extract_timestamp(\"018df6f0-1234-7890-abcd-ef0123456789\")\n    #=> 1677721600000","ref":"Jido.Signal.ID.html#extract_timestamp/1-examples"},{"type":"function","title":"Jido.Signal.ID.format_sortable/1","doc":"Formats a timestamp and sequence as a sortable string.\nUseful for version strings or ordering.\n\nReturns a string in the format \"timestamp-sequence\".","ref":"Jido.Signal.ID.html#format_sortable/1"},{"type":"function","title":"Examples - Jido.Signal.ID.format_sortable/1","doc":"Jido.Signal.ID.format_sortable(\"018df6f0-1234-7890-abcd-ef0123456789\")\n    #=> \"1677721600000-42\"","ref":"Jido.Signal.ID.html#format_sortable/1-examples"},{"type":"function","title":"Jido.Signal.ID.generate/0","doc":"Generates a new signal ID using UUID7.\n\nReturns a tuple containing the generated ID and its embedded timestamp.\nThe timestamp is in Unix milliseconds.","ref":"Jido.Signal.ID.html#generate/0"},{"type":"function","title":"Examples - Jido.Signal.ID.generate/0","doc":"{id, ts} = Jido.Signal.ID.generate()\n    #=> {\"018df6f0-1234-7890-abcd-ef0123456789\", 1677721600000}","ref":"Jido.Signal.ID.html#generate/0-examples"},{"type":"function","title":"Jido.Signal.ID.generate!/0","doc":"Generates a new signal ID using UUID7, returning just the UUID string.","ref":"Jido.Signal.ID.html#generate!/0"},{"type":"function","title":"Examples - Jido.Signal.ID.generate!/0","doc":"id = Jido.Signal.ID.generate!()\n    #=> \"018df6f0-1234-7890-abcd-ef0123456789\"","ref":"Jido.Signal.ID.html#generate!/0-examples"},{"type":"function","title":"Jido.Signal.ID.generate_batch/1","doc":"Generates multiple sequential UUIDs in a batch.\nAll UUIDs will be strictly ordered and unique within the same millisecond.\nIf the batch size would exceed the available sequence numbers (4096),\nit will use multiple milliseconds.","ref":"Jido.Signal.ID.html#generate_batch/1"},{"type":"function","title":"Examples - Jido.Signal.ID.generate_batch/1","doc":"{ids, timestamp} = Jido.Signal.ID.generate_batch(5)\n    #=> {[\"018df6f0-0001-...\", \"018df6f0-0002-...\"], 1677721600000}\n\n    # Verify ordering\n    [id1, id2 | _] = ids\n    :lt = Jido.Signal.ID.compare(id1, id2)","ref":"Jido.Signal.ID.html#generate_batch/1-examples"},{"type":"function","title":"Jido.Signal.ID.generate_sequential/2","doc":"Generates a UUID7 with a specific timestamp and sequence number.\nThis ensures strict ordering of IDs within the same millisecond.","ref":"Jido.Signal.ID.html#generate_sequential/2"},{"type":"function","title":"Parameters - Jido.Signal.ID.generate_sequential/2","doc":"* timestamp - Unix timestamp in milliseconds\n  * sequence - A number between 0 and 4095 (12 bits)","ref":"Jido.Signal.ID.html#generate_sequential/2-parameters"},{"type":"function","title":"Examples - Jido.Signal.ID.generate_sequential/2","doc":"id = Jido.Signal.ID.generate_sequential(timestamp, 1)\n    #=> \"018df6f0-0001-7890-abcd-ef0123456789\"","ref":"Jido.Signal.ID.html#generate_sequential/2-examples"},{"type":"function","title":"Jido.Signal.ID.sequence_number/1","doc":"Returns the sequence number portion of the UUID7.\nThis is a 12-bit monotonic counter within each millisecond.","ref":"Jido.Signal.ID.html#sequence_number/1"},{"type":"function","title":"Examples - Jido.Signal.ID.sequence_number/1","doc":"seq = Jido.Signal.ID.sequence_number(\"018df6f0-1234-7890-abcd-ef0123456789\")\n    #=> 42","ref":"Jido.Signal.ID.html#sequence_number/1-examples"},{"type":"type","title":"Jido.Signal.ID.timestamp/0","doc":"","ref":"Jido.Signal.ID.html#t:timestamp/0"},{"type":"type","title":"Jido.Signal.ID.uuid7/0","doc":"","ref":"Jido.Signal.ID.html#t:uuid7/0"},{"type":"function","title":"Jido.Signal.ID.valid?/1","doc":"Validates that a string is a valid UUID7.\nReturns true if the input is a valid UUID7 string, false otherwise.","ref":"Jido.Signal.ID.html#valid?/1"},{"type":"function","title":"Examples - Jido.Signal.ID.valid?/1","doc":"Jido.Signal.ID.valid?(\"018df6f0-1234-7890-abcd-ef0123456789\")\n    #=> true\n\n    Jido.Signal.ID.valid?(\"not-a-uuid\")\n    #=> false","ref":"Jido.Signal.ID.html#valid?/1-examples"},{"type":"module","title":"Jido.Signal.Journal","doc":"The Signal Journal tracks and manages signals between agents, maintaining causality\nand conversation relationships. It provides a directed graph of signals that captures\ntemporal ordering and causal relationships.","ref":"Jido.Signal.Journal.html"},{"type":"function","title":"Jido.Signal.Journal.get_cause/2","doc":"Gets the cause of a given signal.","ref":"Jido.Signal.Journal.html#get_cause/2"},{"type":"function","title":"Parameters - Jido.Signal.Journal.get_cause/2","doc":"* journal - The current journal state\n  * signal_id - The ID of the signal to get the cause for\n\nReturns the causing signal or nil if none exists","ref":"Jido.Signal.Journal.html#get_cause/2-parameters"},{"type":"function","title":"Jido.Signal.Journal.get_conversation/2","doc":"Gets all signals in a conversation.","ref":"Jido.Signal.Journal.html#get_conversation/2"},{"type":"function","title":"Parameters - Jido.Signal.Journal.get_conversation/2","doc":"* journal - The current journal state\n  * conversation_id - The ID of the conversation to fetch\n\nReturns a list of signals in chronological order","ref":"Jido.Signal.Journal.html#get_conversation/2-parameters"},{"type":"function","title":"Jido.Signal.Journal.get_effects/2","doc":"Gets all effects (signals caused by) of a given signal.","ref":"Jido.Signal.Journal.html#get_effects/2"},{"type":"function","title":"Parameters - Jido.Signal.Journal.get_effects/2","doc":"* journal - The current journal state\n  * signal_id - The ID of the signal to get effects for\n\nReturns a list of signals in chronological order","ref":"Jido.Signal.Journal.html#get_effects/2-parameters"},{"type":"function","title":"Jido.Signal.Journal.new/1","doc":"Creates a new journal with the specified persistence adapter.","ref":"Jido.Signal.Journal.html#new/1"},{"type":"function","title":"Jido.Signal.Journal.query/2","doc":"Queries signals based on criteria.","ref":"Jido.Signal.Journal.html#query/2"},{"type":"function","title":"Options - Jido.Signal.Journal.query/2","doc":"* type - Filter by signal type\n  * source - Filter by signal source\n  * after - Filter signals after this time\n  * before - Filter signals before this time\n\nReturns a list of signals matching all criteria, in chronological order","ref":"Jido.Signal.Journal.html#query/2-options"},{"type":"type","title":"Jido.Signal.Journal.query_opts/0","doc":"","ref":"Jido.Signal.Journal.html#t:query_opts/0"},{"type":"function","title":"Jido.Signal.Journal.record/3","doc":"Records a new signal in the journal.","ref":"Jido.Signal.Journal.html#record/3"},{"type":"function","title":"Parameters - Jido.Signal.Journal.record/3","doc":"* journal - The current journal state\n  * signal - The signal to record\n  * cause_id - Optional ID of the signal that caused this one\n\nReturns `{:ok, journal}` or `{:error, reason}`","ref":"Jido.Signal.Journal.html#record/3-parameters"},{"type":"type","title":"Jido.Signal.Journal.t/0","doc":"The journal maintains the graph of signals and their relationships","ref":"Jido.Signal.Journal.html#t:t/0"},{"type":"function","title":"Jido.Signal.Journal.trace_chain/3","doc":"Traces the complete causal chain starting from a signal.","ref":"Jido.Signal.Journal.html#trace_chain/3"},{"type":"function","title":"Parameters - Jido.Signal.Journal.trace_chain/3","doc":"* journal - The current journal state\n  * signal_id - The ID of the signal to trace from\n  * direction - :forward for effects chain, :backward for causes chain\n\nReturns a list of signals in causal order","ref":"Jido.Signal.Journal.html#trace_chain/3-parameters"},{"type":"module","title":"Jido.Signal.Journal.Adapters.ETS","doc":"ETS-based implementation of the Journal persistence behavior.\nUses separate ETS tables for signals, causes, effects, and conversations.","ref":"Jido.Signal.Journal.Adapters.ETS.html"},{"type":"module","title":"Configuration - Jido.Signal.Journal.Adapters.ETS","doc":"The adapter requires a prefix for table names to allow multiple instances:\n\n    {:ok, _pid} = Jido.Signal.Journal.Adapters.ETS.start_link(\"my_journal_\")\n    {:ok, journal} = Jido.Signal.Journal.new(Jido.Signal.Journal.Adapters.ETS)\n\nThis will create tables with names:\n  - :my_journal_signals\n  - :my_journal_causes\n  - :my_journal_effects\n  - :my_journal_conversations","ref":"Jido.Signal.Journal.Adapters.ETS.html#module-configuration"},{"type":"function","title":"Jido.Signal.Journal.Adapters.ETS.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Jido.Signal.Journal.Adapters.ETS.html#child_spec/1"},{"type":"function","title":"Jido.Signal.Journal.Adapters.ETS.cleanup/1","doc":"Cleans up all ETS tables used by this adapter instance.","ref":"Jido.Signal.Journal.Adapters.ETS.html#cleanup/1"},{"type":"function","title":"Jido.Signal.Journal.Adapters.ETS.get_all_signals/1","doc":"Gets all signals in the journal.","ref":"Jido.Signal.Journal.Adapters.ETS.html#get_all_signals/1"},{"type":"function","title":"Jido.Signal.Journal.Adapters.ETS.handle_call/3","doc":"Handles GenServer calls for signal operations.","ref":"Jido.Signal.Journal.Adapters.ETS.html#handle_call/3"},{"type":"function","title":"Parameters - Jido.Signal.Journal.Adapters.ETS.handle_call/3","doc":"- `{:put_signal, signal}` - Stores a signal in the journal\n- `{:get_signal, signal_id}` - Retrieves a signal by ID\n- `{:put_cause, cause_id, effect_id}` - Records a cause-effect relationship\n- `{:get_effects, signal_id}` - Gets all effects for a signal\n- `{:get_cause, signal_id}` - Gets the cause for a signal\n- `{:put_conversation, conversation_id, signal_id}` - Adds a signal to a conversation\n- `{:get_conversation, conversation_id}` - Gets all signals in a conversation\n- `:get_all_signals` - Gets all signals in the journal\n- `:cleanup` - Cleans up all ETS tables","ref":"Jido.Signal.Journal.Adapters.ETS.html#handle_call/3-parameters"},{"type":"function","title":"Returns - Jido.Signal.Journal.Adapters.ETS.handle_call/3","doc":"- `{:reply, result, adapter}` for successful operations\n- `{:reply, {:error, reason}, adapter}` for failed operations","ref":"Jido.Signal.Journal.Adapters.ETS.html#handle_call/3-returns"},{"type":"function","title":"Jido.Signal.Journal.Adapters.ETS.init/1","doc":"Initializes the ETS adapter with the given table name prefix.","ref":"Jido.Signal.Journal.Adapters.ETS.html#init/1"},{"type":"function","title":"Parameters - Jido.Signal.Journal.Adapters.ETS.init/1","doc":"- `prefix`: The prefix to use for table names","ref":"Jido.Signal.Journal.Adapters.ETS.html#init/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Journal.Adapters.ETS.init/1","doc":"- `{:ok, adapter}` if initialization succeeds\n- `{:error, reason}` if initialization fails","ref":"Jido.Signal.Journal.Adapters.ETS.html#init/1-returns"},{"type":"function","title":"Examples - Jido.Signal.Journal.Adapters.ETS.init/1","doc":"iex> {:ok, adapter} = Jido.Signal.Journal.Adapters.ETS.init(\"my_journal_\")\n    iex> adapter.signals_table\n    :my_journal_signals_...","ref":"Jido.Signal.Journal.Adapters.ETS.html#init/1-examples"},{"type":"function","title":"Jido.Signal.Journal.Adapters.ETS.start_link/1","doc":"Starts the ETS adapter with the given table name prefix.","ref":"Jido.Signal.Journal.Adapters.ETS.html#start_link/1"},{"type":"type","title":"Jido.Signal.Journal.Adapters.ETS.t/0","doc":"","ref":"Jido.Signal.Journal.Adapters.ETS.html#t:t/0"},{"type":"module","title":"Jido.Signal.Journal.Adapters.InMemory","doc":"In-memory implementation of the Journal persistence behavior.\nUses Agent to maintain state.","ref":"Jido.Signal.Journal.Adapters.InMemory.html"},{"type":"function","title":"Jido.Signal.Journal.Adapters.InMemory.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Jido.Signal.Journal.Adapters.InMemory.html#child_spec/1"},{"type":"function","title":"Jido.Signal.Journal.Adapters.InMemory.get_all_signals/0","doc":"Gets all signals in the journal.","ref":"Jido.Signal.Journal.Adapters.InMemory.html#get_all_signals/0"},{"type":"function","title":"Returns - Jido.Signal.Journal.Adapters.InMemory.get_all_signals/0","doc":"A list of all signals stored in the journal","ref":"Jido.Signal.Journal.Adapters.InMemory.html#get_all_signals/0-returns"},{"type":"function","title":"Examples - Jido.Signal.Journal.Adapters.InMemory.get_all_signals/0","doc":"iex> Jido.Signal.Journal.Adapters.InMemory.put_signal(signal1)\n    iex> Jido.Signal.Journal.Adapters.InMemory.put_signal(signal2)\n    iex> signals = Jido.Signal.Journal.Adapters.InMemory.get_all_signals()\n    iex> length(signals)\n    2","ref":"Jido.Signal.Journal.Adapters.InMemory.html#get_all_signals/0-examples"},{"type":"behaviour","title":"Jido.Signal.Journal.Persistence","doc":"Defines the behavior for Journal persistence adapters.","ref":"Jido.Signal.Journal.Persistence.html"},{"type":"type","title":"Jido.Signal.Journal.Persistence.conversation_id/0","doc":"","ref":"Jido.Signal.Journal.Persistence.html#t:conversation_id/0"},{"type":"type","title":"Jido.Signal.Journal.Persistence.error/0","doc":"","ref":"Jido.Signal.Journal.Persistence.html#t:error/0"},{"type":"callback","title":"Jido.Signal.Journal.Persistence.get_cause/2","doc":"","ref":"Jido.Signal.Journal.Persistence.html#c:get_cause/2"},{"type":"callback","title":"Jido.Signal.Journal.Persistence.get_conversation/2","doc":"","ref":"Jido.Signal.Journal.Persistence.html#c:get_conversation/2"},{"type":"callback","title":"Jido.Signal.Journal.Persistence.get_effects/2","doc":"","ref":"Jido.Signal.Journal.Persistence.html#c:get_effects/2"},{"type":"callback","title":"Jido.Signal.Journal.Persistence.get_signal/2","doc":"","ref":"Jido.Signal.Journal.Persistence.html#c:get_signal/2"},{"type":"callback","title":"Jido.Signal.Journal.Persistence.init/0","doc":"","ref":"Jido.Signal.Journal.Persistence.html#c:init/0"},{"type":"callback","title":"Jido.Signal.Journal.Persistence.put_cause/3","doc":"","ref":"Jido.Signal.Journal.Persistence.html#c:put_cause/3"},{"type":"callback","title":"Jido.Signal.Journal.Persistence.put_conversation/3","doc":"","ref":"Jido.Signal.Journal.Persistence.html#c:put_conversation/3"},{"type":"callback","title":"Jido.Signal.Journal.Persistence.put_signal/2","doc":"","ref":"Jido.Signal.Journal.Persistence.html#c:put_signal/2"},{"type":"type","title":"Jido.Signal.Journal.Persistence.signal_id/0","doc":"","ref":"Jido.Signal.Journal.Persistence.html#t:signal_id/0"},{"type":"module","title":"Jido.Signal.Registry","doc":"Registry for managing signal subscriptions.\n\nProvides functionality to register, unregister, and manage subscriptions\nto signal paths with associated dispatch configurations.","ref":"Jido.Signal.Registry.html"},{"type":"function","title":"Jido.Signal.Registry.all/1","doc":"Returns a list of all subscriptions in the registry.","ref":"Jido.Signal.Registry.html#all/1"},{"type":"function","title":"Jido.Signal.Registry.count/1","doc":"Returns the count of subscriptions in the registry.","ref":"Jido.Signal.Registry.html#count/1"},{"type":"macro","title":"Jido.Signal.Registry.dbug/2","doc":"","ref":"Jido.Signal.Registry.html#dbug/2"},{"type":"macro","title":"Jido.Signal.Registry.error/2","doc":"","ref":"Jido.Signal.Registry.html#error/2"},{"type":"function","title":"Jido.Signal.Registry.find_by_path/2","doc":"Returns all subscriptions for a given path pattern.\n\nFinds all subscriptions that would match the given signal path.","ref":"Jido.Signal.Registry.html#find_by_path/2"},{"type":"function","title":"Jido.Signal.Registry.lookup/2","doc":"Looks up a subscription by its ID.","ref":"Jido.Signal.Registry.html#lookup/2"},{"type":"function","title":"Returns - Jido.Signal.Registry.lookup/2","doc":"* `{:ok, subscription}` - Subscription was found\n  * `{:error, :not_found}` - No subscription with this ID exists","ref":"Jido.Signal.Registry.html#lookup/2-returns"},{"type":"function","title":"Jido.Signal.Registry.new/0","doc":"Creates a new empty registry.","ref":"Jido.Signal.Registry.html#new/0"},{"type":"function","title":"Examples - Jido.Signal.Registry.new/0","doc":"iex> registry = Jido.Signal.Registry.new()","ref":"Jido.Signal.Registry.html#new/0-examples"},{"type":"function","title":"Jido.Signal.Registry.register/4","doc":"Registers a new subscription in the registry.","ref":"Jido.Signal.Registry.html#register/4"},{"type":"function","title":"Parameters - Jido.Signal.Registry.register/4","doc":"* registry - The current registry\n  * id - Unique identifier for the subscription\n  * path - Signal path pattern to subscribe to\n  * dispatch - Dispatch configuration for matched signals","ref":"Jido.Signal.Registry.html#register/4-parameters"},{"type":"function","title":"Returns - Jido.Signal.Registry.register/4","doc":"* `{:ok, updated_registry}` - Subscription was added successfully\n  * `{:error, :already_exists}` - A subscription with this ID already exists","ref":"Jido.Signal.Registry.html#register/4-returns"},{"type":"function","title":"Examples - Jido.Signal.Registry.register/4","doc":"iex> {:ok, registry} = Registry.register(registry, \"sub1\", \"user.created\", dispatch_config)","ref":"Jido.Signal.Registry.html#register/4-examples"},{"type":"type","title":"Jido.Signal.Registry.t/0","doc":"Registry containing a unique mapping of subscription IDs to subscriptions","ref":"Jido.Signal.Registry.html#t:t/0"},{"type":"function","title":"Jido.Signal.Registry.unregister/2","doc":"Unregisters a subscription from the registry.","ref":"Jido.Signal.Registry.html#unregister/2"},{"type":"function","title":"Parameters - Jido.Signal.Registry.unregister/2","doc":"* registry - The current registry\n  * id - ID of the subscription to remove","ref":"Jido.Signal.Registry.html#unregister/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Registry.unregister/2","doc":"* `{:ok, updated_registry}` - Subscription was removed successfully\n  * `{:error, :not_found}` - No subscription with this ID exists","ref":"Jido.Signal.Registry.html#unregister/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Registry.unregister/2","doc":"iex> {:ok, registry} = Registry.unregister(registry, \"sub1\")","ref":"Jido.Signal.Registry.html#unregister/2-examples"},{"type":"module","title":"Jido.Signal.Registry.Subscription","doc":"Represents a subscription to signal patterns in the registry.\n\nA subscription maps a signal path pattern to a dispatch configuration,\nallowing signals matching the pattern to be routed to the specified target.","ref":"Jido.Signal.Registry.Subscription.html"},{"type":"type","title":"Jido.Signal.Registry.Subscription.t/0","doc":"A single subscription mapping a path to dispatch configuration","ref":"Jido.Signal.Registry.Subscription.html#t:t/0"},{"type":"module","title":"Jido.Signal.Router","doc":"The Router module implements a high-performance, trie-based signal routing system designed specifically for agent-based architectures. It provides sophisticated message routing capabilities with support for exact matches, wildcards, pattern matching functions, and multiple dispatch targets.","ref":"Jido.Signal.Router.html"},{"type":"module","title":"Core Concepts - Jido.Signal.Router","doc":"The Router organizes signal handlers into an efficient trie (prefix tree) structure that enables:\n- Fast path-based routing using dot-notation (e.g., \"user.created.verified\")\n- Priority-based handler execution (-100 to 100)\n- Complexity-based ordering for wildcard resolution\n- Dynamic route management (add/remove at runtime)\n- Pattern matching through custom functions\n- Multiple dispatch targets per route","ref":"Jido.Signal.Router.html#module-core-concepts"},{"type":"module","title":"Path Patterns - Jido.Signal.Router","doc":"Routes use a dot-notation pattern system supporting:\n- Exact matches: `\"user.created\"`\n- Single wildcards: `\"user.*.updated\"` (matches one segment)\n- Multi-level wildcards: `\"audit.**\"` (matches zero or more segments)\n\nPattern rules:\n- Paths must match: `^[a-zA-Z0-9.*_-]+(.[a-zA-Z0-9.*_-]+)*$`\n- Cannot contain consecutive dots (..)\n- Cannot contain consecutive multi-wildcards (`**...**`)","ref":"Jido.Signal.Router.html#module-path-patterns"},{"type":"module","title":"Handler Priority - Jido.Signal.Router","doc":"Handlers execute in order based on:\n1. Path complexity (more specific paths execute first)\n2. Priority (-100 to 100, higher executes first)\n3. Registration order (for equal priority/complexity)","ref":"Jido.Signal.Router.html#module-handler-priority"},{"type":"module","title":"Target Types - Jido.Signal.Router","doc":"The router accepts any term as a target. Common patterns include:\n1. Atoms - Simple handler references\n2. Tuples - Configuration with options `{handler, opts}`\n3. Lists - Multiple targets `[{handler1, opts1}, {handler2, opts2}]`\n4. Any other term - Custom data structures","ref":"Jido.Signal.Router.html#module-target-types"},{"type":"module","title":"Usage Examples - Jido.Signal.Router","doc":"Basic route creation:\n```elixir\n{:ok, router} = Router.new([\n  # Simple route with default priority\n  {\"user.created\", HandleUserCreated},\n\n  # High-priority audit logging\n  {\"audit.**\", AuditLogger, 100},\n\n  # Pattern matching for large payments\n  {\"payment.processed\",\n    fn signal -> signal.data.amount > 1000 end,\n    HandleLargePayment},\n\n  # Single dispatch target\n  {\"metrics.collected\", {MetricsAdapter, [type: :counter]}},\n\n  # Multiple dispatch targets\n  {\"system.error\", [\n    {MetricsAdapter, [type: :error]},\n    {AlertAdapter, [priority: :high]},\n    {LogAdapter, [level: :error]}\n  ]}\n])\n```\n\nDynamic route management:\n```elixir\n# Add routes\n{:ok, router} = Router.add(router, [\n  {\"metrics.**\", {MetricsAdapter, [type: :gauge]}}\n])\n\n# Remove routes\n{:ok, router} = Router.remove(router, \"metrics.**\")\n```\n\nSignal routing:\n```elixir\n# Route to handler\n{:ok, [HandleUserCreated]} = Router.route(router, %Signal{\n  type: \"user.created\",\n  data: %{id: \"123\"}\n})\n\n# Route to multiple dispatch targets\n{:ok, [\n  {MetricsAdapter, [type: :error]},\n  {AlertAdapter, [priority: :high]},\n  {LogAdapter, [level: :error]}\n]} = Router.route(router, %Signal{\n  type: \"system.error\",\n  data: %{message: \"Critical error\"}\n})\n```","ref":"Jido.Signal.Router.html#module-usage-examples"},{"type":"module","title":"Path Complexity Scoring - Jido.Signal.Router","doc":"The router uses a sophisticated scoring system to determine handler execution order:\n\n1. Base score from segment count (length * 2000)\n2. Exact match bonuses (3000 per segment, weighted by position)\n3. Wildcard penalties:\n   - Single wildcard (*): 1000 - position_index * 100\n   - Multi-wildcard (**): 2000 - position_index * 200\n\nThis ensures more specific routes take precedence over wildcards, while maintaining predictable execution order.","ref":"Jido.Signal.Router.html#module-path-complexity-scoring"},{"type":"module","title":"Best Practices - Jido.Signal.Router","doc":"1. Route Design\n   - Use consistent, hierarchical path patterns\n   - Prefer exact matches over wildcards when possible\n   - Keep path segments meaningful and well-structured\n   - Document your path hierarchy\n\n2. Priority Management\n   - Reserve high priorities (75-100) for critical handlers\n   - Use default priority (0) for standard business logic\n   - Reserve low priorities (-100 to -75) for metrics/logging\n   - Document priority ranges for your application\n\n3. Pattern Matching\n   - Keep match functions simple and fast\n   - Handle nil/missing data gracefully\n   - Avoid side effects in match functions\n   - Test edge cases thoroughly\n\n4. Dispatch Configuration\n   - Use single dispatch for simple routing\n   - Use multiple dispatch for cross-cutting concerns\n   - Keep adapter options minimal and focused\n   - Document adapter requirements\n\n5. Performance Considerations\n   - Monitor route count in production\n   - Use pattern matching sparingly\n   - Consider complexity scores when designing paths\n   - Profile routing performance under load","ref":"Jido.Signal.Router.html#module-best-practices"},{"type":"module","title":"Error Handling - Jido.Signal.Router","doc":"The router provides detailed error feedback for:\n- Invalid path patterns\n- Priority out of bounds\n- Invalid match functions\n- Missing handlers\n- Malformed signals\n- Invalid dispatch configurations","ref":"Jido.Signal.Router.html#module-error-handling"},{"type":"module","title":"Implementation Details - Jido.Signal.Router","doc":"The router uses several specialized structs:\n- `Route` - Defines a single routing rule\n- `TrieNode` - Internal trie structure node\n- `HandlerInfo` - Stores handler metadata\n- `PatternMatch` - Encapsulates pattern matching rules\n\nSee the corresponding typespecs for detailed field information.","ref":"Jido.Signal.Router.html#module-implementation-details"},{"type":"module","title":"See Also - Jido.Signal.Router","doc":"- `Jido.Signal` - Signal structure and validation\n- `Jido.Signal.Error` - Error types and handling\n- `Jido.Signal.Dispatch` - Dispatch adapter interface","ref":"Jido.Signal.Router.html#module-see-also"},{"type":"function","title":"Jido.Signal.Router.add/2","doc":"Adds one or more routes to the router.","ref":"Jido.Signal.Router.html#add/2"},{"type":"function","title":"Parameters - Jido.Signal.Router.add/2","doc":"- router: The existing router struct\n- routes: A route specification or list of route specifications in one of these formats:\n  - %Route{}\n  - {path, target}\n  - {path, target, priority}\n  - {path, match_fn, target}\n  - {path, match_fn, target, priority}","ref":"Jido.Signal.Router.html#add/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.add/2","doc":"`{:ok, updated_router}` or `{:error, reason}`","ref":"Jido.Signal.Router.html#add/2-returns"},{"type":"function","title":"Jido.Signal.Router.filter/2","doc":"Filters a list of signals based on a pattern.","ref":"Jido.Signal.Router.html#filter/2"},{"type":"function","title":"Parameters - Jido.Signal.Router.filter/2","doc":"- signals: List of signals to filter\n- pattern: Pattern to filter by (e.g. \"user.*\" or \"audit.**\")","ref":"Jido.Signal.Router.html#filter/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.filter/2","doc":"- List of signals whose types match the pattern","ref":"Jido.Signal.Router.html#filter/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Router.filter/2","doc":"iex> signals = [\n    ...>   %Signal{type: \"user.created\"},\n    ...>   %Signal{type: \"payment.processed\"},\n    ...>   %Signal{type: \"user.updated\"}\n    ...> ]\n    iex> Router.filter(signals, \"user.*\")\n    [%Signal{type: \"user.created\"}, %Signal{type: \"user.updated\"}]\n\n    iex> Router.filter(nil, \"user.*\")\n    []\n\n    iex> Router.filter([], nil)\n    []\n\n    iex> Router.filter(\"not a list\", \"user.*\")\n    []","ref":"Jido.Signal.Router.html#filter/2-examples"},{"type":"function","title":"Jido.Signal.Router.has_route?/2","doc":"Checks if a route with the given ID exists in the router.","ref":"Jido.Signal.Router.html#has_route?/2"},{"type":"function","title":"Parameters - Jido.Signal.Router.has_route?/2","doc":"- router: The router struct to check\n- route_id: The ID of the route to check for","ref":"Jido.Signal.Router.html#has_route?/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.has_route?/2","doc":"- `true` if the route exists\n- `false` otherwise","ref":"Jido.Signal.Router.html#has_route?/2-returns"},{"type":"function","title":"Jido.Signal.Router.list/1","doc":"Lists all routes currently registered in the router.\n\nReturns a list of Route structs containing the path, target, priority and match function\nfor each registered route.","ref":"Jido.Signal.Router.html#list/1"},{"type":"function","title":"Returns - Jido.Signal.Router.list/1","doc":"`{:ok, [%Route{}]}` - List of Route structs","ref":"Jido.Signal.Router.html#list/1-returns"},{"type":"function","title":"Examples - Jido.Signal.Router.list/1","doc":"{:ok, routes} = Router.list(router)\n\n    # Returns:\n    [\n      %Route{\n        path: \"user.created\",\n        target: MyApp.Actions.HandleUserCreated,\n        priority: 0,\n        match: nil\n      },\n      %Route{\n        path: \"payment.processed\",\n        target: {:some_adapter, [opts: :here]},\n        priority: 90,\n        match: #Function<1.123456789/1>\n      }\n    ]","ref":"Jido.Signal.Router.html#list/1-examples"},{"type":"type","title":"Jido.Signal.Router.match/0","doc":"","ref":"Jido.Signal.Router.html#t:match/0"},{"type":"function","title":"Jido.Signal.Router.matches?/2","doc":"Checks if a signal type matches a pattern.","ref":"Jido.Signal.Router.html#matches?/2"},{"type":"function","title":"Parameters - Jido.Signal.Router.matches?/2","doc":"- type: The signal type to check (e.g. \"user.created\")\n- pattern: The pattern to match against (e.g. \"user.*\" or \"audit.**\")","ref":"Jido.Signal.Router.html#matches?/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.matches?/2","doc":"- `true` if the type matches the pattern\n- `false` otherwise","ref":"Jido.Signal.Router.html#matches?/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Router.matches?/2","doc":"iex> Router.matches?(\"user.created\", \"user.*\")\n    true\n\n    iex> Router.matches?(\"audit.user.created\", \"audit.**\")\n    true\n\n    iex> Router.matches?(\"user.created\", \"payment.*\")\n    false\n\n    iex> Router.matches?(\"user.profile.updated\", \"user.*\")\n    false\n\n    iex> Router.matches?(nil, \"user.*\")\n    false\n\n    iex> Router.matches?(\"user.created\", nil)\n    false","ref":"Jido.Signal.Router.html#matches?/2-examples"},{"type":"function","title":"Jido.Signal.Router.merge/2","doc":"Merges two routers by combining their routes.\n\nTakes a target router and a list of routes from another router (obtained via `list/1`) and\nmerges them together, preserving priorities and match functions.","ref":"Jido.Signal.Router.html#merge/2"},{"type":"function","title":"Parameters - Jido.Signal.Router.merge/2","doc":"- router: The target Router struct to merge into\n- routes: List of Route structs to merge in (from Router.list/1)","ref":"Jido.Signal.Router.html#merge/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.merge/2","doc":"`{:ok, merged_router}` or `{:error, reason}`","ref":"Jido.Signal.Router.html#merge/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Router.merge/2","doc":"{:ok, router1} = Router.new([{\"user.created\", target1}])\n    {:ok, router2} = Router.new([{\"payment.processed\", target2}])\n    {:ok, routes2} = Router.list(router2)\n\n    # Merge router2's routes into router1\n    {:ok, merged} = Router.merge(router1, routes2)","ref":"Jido.Signal.Router.html#merge/2-examples"},{"type":"function","title":"Jido.Signal.Router.new/1","doc":"Creates a new router with the given routes.","ref":"Jido.Signal.Router.html#new/1"},{"type":"function","title":"Jido.Signal.Router.new!/1","doc":"Creates a new router with the given routes, raising on error.","ref":"Jido.Signal.Router.html#new!/1"},{"type":"function","title":"Jido.Signal.Router.normalize/1","doc":"Normalizes route specifications into Route structs.","ref":"Jido.Signal.Router.html#normalize/1"},{"type":"function","title":"Parameters - Jido.Signal.Router.normalize/1","doc":"* `input` - One of:\n  * Single Route struct\n  * List of Route structs\n  * List of route_spec tuples:\n    * {path, target} tuple where target can be any term\n    * {path, target, priority} tuple\n    * {path, match_fn, target} tuple\n    * {path, match_fn, target, priority} tuple","ref":"Jido.Signal.Router.html#normalize/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.normalize/1","doc":"* `{:ok, [%Route{}]}` - List of normalized Route structs\n  * `{:error, term()}` - If normalization fails","ref":"Jido.Signal.Router.html#normalize/1-returns"},{"type":"type","title":"Jido.Signal.Router.path/0","doc":"","ref":"Jido.Signal.Router.html#t:path/0"},{"type":"type","title":"Jido.Signal.Router.priority/0","doc":"","ref":"Jido.Signal.Router.html#t:priority/0"},{"type":"function","title":"Jido.Signal.Router.remove/2","doc":"Removes one or more routes from the router.","ref":"Jido.Signal.Router.html#remove/2"},{"type":"function","title":"Parameters - Jido.Signal.Router.remove/2","doc":"- router: The Router struct to modify\n- paths: A path string or list of path strings to remove","ref":"Jido.Signal.Router.html#remove/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.remove/2","doc":"- `{:ok, updated_router}` - Routes removed successfully","ref":"Jido.Signal.Router.html#remove/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Router.remove/2","doc":"{:ok, router} = Router.remove(router, \"metrics.collected\")\n    {:ok, router} = Router.remove(router, [\"user.created\", \"user.updated\"])","ref":"Jido.Signal.Router.html#remove/2-examples"},{"type":"function","title":"Jido.Signal.Router.route/2","doc":"Routes a signal through the router to find and execute matching handlers.","ref":"Jido.Signal.Router.html#route/2"},{"type":"function","title":"Parameters - Jido.Signal.Router.route/2","doc":"- router: The router struct to use for routing\n- signal: The signal to route","ref":"Jido.Signal.Router.html#route/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.route/2","doc":"* `{:ok, [term()]}` - List of matching targets, may be empty if no matches\n* `{:error, term()}` - Other errors that occurred during routing","ref":"Jido.Signal.Router.html#route/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Router.route/2","doc":"{:ok, targets} = Router.route(router, %Signal{\n      type: \"payment.processed\",\n      data: %{amount: 100}\n    })","ref":"Jido.Signal.Router.html#route/2-examples"},{"type":"type","title":"Jido.Signal.Router.route_spec/0","doc":"","ref":"Jido.Signal.Router.html#t:route_spec/0"},{"type":"type","title":"Jido.Signal.Router.target/0","doc":"","ref":"Jido.Signal.Router.html#t:target/0"},{"type":"function","title":"Jido.Signal.Router.validate/1","doc":"Validates one or more Route structs.","ref":"Jido.Signal.Router.html#validate/1"},{"type":"function","title":"Parameters - Jido.Signal.Router.validate/1","doc":"- routes: A %Route{} struct or list of %Route{} structs to validate","ref":"Jido.Signal.Router.html#validate/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.validate/1","doc":"* `{:ok, %Route{}}` - Single validated Route struct\n* `{:ok, [%Route{}]}` - List of validated Route structs\n* `{:error, term()}` - If validation fails","ref":"Jido.Signal.Router.html#validate/1-returns"},{"type":"type","title":"Jido.Signal.Router.wildcard_type/0","doc":"","ref":"Jido.Signal.Router.html#t:wildcard_type/0"},{"type":"module","title":"Jido.Signal.Router.Engine","doc":"The routing engine that matches signals to handlers.","ref":"Jido.Signal.Router.Engine.html"},{"type":"function","title":"Jido.Signal.Router.Engine.build_trie/2","doc":"Builds the trie structure from validated routes.","ref":"Jido.Signal.Router.Engine.html#build_trie/2"},{"type":"function","title":"Jido.Signal.Router.Engine.collect_routes/1","doc":"Collects all routes from the trie into a list of Route structs.","ref":"Jido.Signal.Router.Engine.html#collect_routes/1"},{"type":"function","title":"Jido.Signal.Router.Engine.count_routes/1","doc":"Counts total routes in the trie.","ref":"Jido.Signal.Router.Engine.html#count_routes/1"},{"type":"macro","title":"Jido.Signal.Router.Engine.dbug/2","doc":"","ref":"Jido.Signal.Router.Engine.html#dbug/2"},{"type":"macro","title":"Jido.Signal.Router.Engine.error/2","doc":"","ref":"Jido.Signal.Router.Engine.html#error/2"},{"type":"function","title":"Jido.Signal.Router.Engine.remove_path/2","doc":"Removes a path from the trie.","ref":"Jido.Signal.Router.Engine.html#remove_path/2"},{"type":"function","title":"Jido.Signal.Router.Engine.route_signal/2","doc":"Routes a signal through the trie to find matching handlers.","ref":"Jido.Signal.Router.Engine.html#route_signal/2"},{"type":"module","title":"Jido.Signal.Router.HandlerInfo","doc":"Router Helper struct to store handler metadata","ref":"Jido.Signal.Router.HandlerInfo.html"},{"type":"type","title":"Jido.Signal.Router.HandlerInfo.t/0","doc":"","ref":"Jido.Signal.Router.HandlerInfo.html#t:t/0"},{"type":"module","title":"Jido.Signal.Router.NodeHandlers","doc":"Router Helper struct to store node handler metadata","ref":"Jido.Signal.Router.NodeHandlers.html"},{"type":"type","title":"Jido.Signal.Router.NodeHandlers.t/0","doc":"","ref":"Jido.Signal.Router.NodeHandlers.html#t:t/0"},{"type":"module","title":"Jido.Signal.Router.PatternMatch","doc":"Router Helper struct to store pattern match metadata","ref":"Jido.Signal.Router.PatternMatch.html"},{"type":"type","title":"Jido.Signal.Router.PatternMatch.t/0","doc":"","ref":"Jido.Signal.Router.PatternMatch.html#t:t/0"},{"type":"module","title":"Jido.Signal.Router.Route","doc":"Router Helper struct to store route metadata","ref":"Jido.Signal.Router.Route.html"},{"type":"type","title":"Jido.Signal.Router.Route.t/0","doc":"","ref":"Jido.Signal.Router.Route.html#t:t/0"},{"type":"module","title":"Jido.Signal.Router.Router","doc":"Router Helper struct to store router metadata","ref":"Jido.Signal.Router.Router.html"},{"type":"type","title":"Jido.Signal.Router.Router.t/0","doc":"","ref":"Jido.Signal.Router.Router.html#t:t/0"},{"type":"module","title":"Jido.Signal.Router.TrieNode","doc":"Router Helper struct to store trie node metadata","ref":"Jido.Signal.Router.TrieNode.html"},{"type":"type","title":"Jido.Signal.Router.TrieNode.t/0","doc":"","ref":"Jido.Signal.Router.TrieNode.html#t:t/0"},{"type":"module","title":"Jido.Signal.Router.Validator","doc":"Validates router configuration and normalizes route specifications.","ref":"Jido.Signal.Router.Validator.html"},{"type":"macro","title":"Jido.Signal.Router.Validator.dbug/2","doc":"","ref":"Jido.Signal.Router.Validator.html#dbug/2"},{"type":"macro","title":"Jido.Signal.Router.Validator.error/2","doc":"","ref":"Jido.Signal.Router.Validator.html#error/2"},{"type":"function","title":"Jido.Signal.Router.Validator.normalize/1","doc":"Normalizes route specifications into Route structs.","ref":"Jido.Signal.Router.Validator.html#normalize/1"},{"type":"function","title":"Parameters - Jido.Signal.Router.Validator.normalize/1","doc":"* `input` - One of:\n    * Single Route struct\n    * List of Route structs\n    * List of route_spec tuples\n    * {path, target} tuple where target can be any term\n    * {path, target, priority} tuple\n    * {path, match_fn, target} tuple\n    * {path, match_fn, target, priority} tuple","ref":"Jido.Signal.Router.Validator.html#normalize/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.Validator.normalize/1","doc":"* `{:ok, [%Route{}]}` - List of normalized Route structs\n  * `{:error, term()}` - If normalization fails","ref":"Jido.Signal.Router.Validator.html#normalize/1-returns"},{"type":"function","title":"Jido.Signal.Router.Validator.validate_match/1","doc":"Validates that a match function returns boolean for a test signal.","ref":"Jido.Signal.Router.Validator.html#validate_match/1"},{"type":"function","title":"Parameters - Jido.Signal.Router.Validator.validate_match/1","doc":"- match_fn: A function that takes a Signal struct and returns a boolean","ref":"Jido.Signal.Router.Validator.html#validate_match/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.Validator.validate_match/1","doc":"- `{:ok, match_fn}` if valid\n- `{:error, reason}` if invalid","ref":"Jido.Signal.Router.Validator.html#validate_match/1-returns"},{"type":"function","title":"Jido.Signal.Router.Validator.validate_path/1","doc":"Validates a path string against the allowed format.","ref":"Jido.Signal.Router.Validator.html#validate_path/1"},{"type":"function","title":"Rules - Jido.Signal.Router.Validator.validate_path/1","doc":"- Must be a string\n- Cannot contain consecutive dots (..)\n- Cannot have consecutive ** segments\n- Each segment must be either:\n  - A valid identifier (alphanumeric + underscore + hyphen)\n  - A single wildcard (*)\n  - A double wildcard (**)","ref":"Jido.Signal.Router.Validator.html#validate_path/1-rules"},{"type":"function","title":"Returns - Jido.Signal.Router.Validator.validate_path/1","doc":"- `{:ok, path}` if valid\n- `{:error, reason}` if invalid","ref":"Jido.Signal.Router.Validator.html#validate_path/1-returns"},{"type":"function","title":"Jido.Signal.Router.Validator.validate_priority/1","doc":"Validates that a priority value is within allowed bounds.","ref":"Jido.Signal.Router.Validator.html#validate_priority/1"},{"type":"function","title":"Parameters - Jido.Signal.Router.Validator.validate_priority/1","doc":"- priority: An integer between -100 and 100, or nil for default priority","ref":"Jido.Signal.Router.Validator.html#validate_priority/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.Validator.validate_priority/1","doc":"- `{:ok, priority}` if valid\n- `{:error, reason}` if invalid","ref":"Jido.Signal.Router.Validator.html#validate_priority/1-returns"},{"type":"function","title":"Jido.Signal.Router.Validator.validate_target/1","doc":"Validates that a target is a valid term.","ref":"Jido.Signal.Router.Validator.html#validate_target/1"},{"type":"function","title":"Valid Targets - Jido.Signal.Router.Validator.validate_target/1","doc":"- Any term","ref":"Jido.Signal.Router.Validator.html#validate_target/1-valid-targets"},{"type":"function","title":"Returns - Jido.Signal.Router.Validator.validate_target/1","doc":"- `{:ok, target}` - Always succeeds","ref":"Jido.Signal.Router.Validator.html#validate_target/1-returns"},{"type":"module","title":"Jido.Signal.Router.WildcardHandlers","doc":"Router Helper struct to store wildcard handler metadata","ref":"Jido.Signal.Router.WildcardHandlers.html"},{"type":"type","title":"Jido.Signal.Router.WildcardHandlers.t/0","doc":"","ref":"Jido.Signal.Router.WildcardHandlers.html#t:t/0"},{"type":"module","title":"Jido.Signal.Serialization.Config","doc":"Configuration management for serialization settings.\n\nThis module provides functions to get and set default serialization\nconfiguration for the Jido application.","ref":"Jido.Signal.Serialization.Config.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Serialization.Config","doc":"The following configuration keys are supported under the `:jido` application:\n\n- `:default_serializer` - The default serializer module to use (default: JsonSerializer)\n- `:default_type_provider` - The default type provider module to use (default: ModuleNameTypeProvider)","ref":"Jido.Signal.Serialization.Config.html#module-configuration-options"},{"type":"module","title":"Example Configuration - Jido.Signal.Serialization.Config","doc":"# In your config.exs or runtime.exs\n    config :jido,\n      default_serializer: Jido.Signal.Serialization.ErlangTermSerializer,\n      default_type_provider: Jido.Signal.Serialization.ModuleNameTypeProvider","ref":"Jido.Signal.Serialization.Config.html#module-example-configuration"},{"type":"module","title":"Runtime Configuration - Jido.Signal.Serialization.Config","doc":"You can also change the configuration at runtime:\n\n    Jido.Signal.Serialization.Config.set_default_serializer(MyCustomSerializer)\n    Jido.Signal.Serialization.Config.set_default_type_provider(MyCustomTypeProvider)","ref":"Jido.Signal.Serialization.Config.html#module-runtime-configuration"},{"type":"function","title":"Jido.Signal.Serialization.Config.all/0","doc":"Get all serialization configuration as a keyword list.","ref":"Jido.Signal.Serialization.Config.html#all/0"},{"type":"function","title":"Jido.Signal.Serialization.Config.default_serializer/0","doc":"Get the configured default serializer.","ref":"Jido.Signal.Serialization.Config.html#default_serializer/0"},{"type":"function","title":"Jido.Signal.Serialization.Config.default_type_provider/0","doc":"Get the configured default type provider.","ref":"Jido.Signal.Serialization.Config.html#default_type_provider/0"},{"type":"function","title":"Jido.Signal.Serialization.Config.set_default_serializer/1","doc":"Set the default serializer at runtime.","ref":"Jido.Signal.Serialization.Config.html#set_default_serializer/1"},{"type":"function","title":"Jido.Signal.Serialization.Config.set_default_type_provider/1","doc":"Set the default type provider at runtime.","ref":"Jido.Signal.Serialization.Config.html#set_default_type_provider/1"},{"type":"function","title":"Jido.Signal.Serialization.Config.validate/0","doc":"Validate the current configuration.","ref":"Jido.Signal.Serialization.Config.html#validate/0"},{"type":"function","title":"Jido.Signal.Serialization.Config.validate_serializer/1","doc":"Validate that the configured serializer implements the Serializer behaviour.","ref":"Jido.Signal.Serialization.Config.html#validate_serializer/1"},{"type":"function","title":"Jido.Signal.Serialization.Config.validate_type_provider/1","doc":"Validate that the configured type provider implements the TypeProvider behaviour.","ref":"Jido.Signal.Serialization.Config.html#validate_type_provider/1"},{"type":"module","title":"Jido.Signal.Serialization.ErlangTermSerializer","doc":"A serializer that uses Erlang's built-in term format.\n\nThis serializer is particularly useful for Erlang/Elixir clusters where\ndata needs to be passed between nodes efficiently. The Erlang term format\npreserves the exact structure and types of Elixir/Erlang data.","ref":"Jido.Signal.Serialization.ErlangTermSerializer.html"},{"type":"module","title":"Features - Jido.Signal.Serialization.ErlangTermSerializer","doc":"- Preserves exact data types (atoms, tuples, etc.)\n- Efficient for inter-node communication\n- Compact binary representation\n- No intermediate transformations needed","ref":"Jido.Signal.Serialization.ErlangTermSerializer.html#module-features"},{"type":"module","title":"Usage - Jido.Signal.Serialization.ErlangTermSerializer","doc":"# Configure as default serializer\n    config :jido, :default_serializer, Jido.Signal.Serialization.ErlangTermSerializer\n\n    # Or use explicitly\n    Signal.serialize(signal, serializer: Jido.Signal.Serialization.ErlangTermSerializer)","ref":"Jido.Signal.Serialization.ErlangTermSerializer.html#module-usage"},{"type":"function","title":"Jido.Signal.Serialization.ErlangTermSerializer.deserialize/2","doc":"Deserialize given Erlang binary data back to the original term.\n\nFor Erlang terms, type conversion is handled automatically since\nthe format preserves the original structure. However, if a specific\ntype is requested, we can still convert it.","ref":"Jido.Signal.Serialization.ErlangTermSerializer.html#deserialize/2"},{"type":"function","title":"Jido.Signal.Serialization.ErlangTermSerializer.serialize/2","doc":"Serialize given term to Erlang binary format.","ref":"Jido.Signal.Serialization.ErlangTermSerializer.html#serialize/2"},{"type":"function","title":"Jido.Signal.Serialization.ErlangTermSerializer.valid_erlang_term?/1","doc":"Checks if the given binary is a valid Erlang term.","ref":"Jido.Signal.Serialization.ErlangTermSerializer.html#valid_erlang_term?/1"},{"type":"protocol","title":"Jido.Signal.Serialization.JsonDecoder","doc":"Protocol to allow additional decoding of a value that has been deserialized from JSON.\n\nThis protocol enables custom decoding logic to be applied after JSON deserialization.\nIt is particularly useful when you need to transform or validate data after it has been\ndeserialized from JSON format.","ref":"Jido.Signal.Serialization.JsonDecoder.html"},{"type":"protocol","title":"Example - Jido.Signal.Serialization.JsonDecoder","doc":"defmodule MyStruct do\n      @derive Jason.Encoder\n      defstruct [:value]\n    end\n\n    # Custom decoder implementation that doubles numeric values\n    defimpl Jido.Signal.Serialization.JsonDecoder, for: MyStruct do\n      def decode(%MyStruct{value: value} = data) when is_number(value) do\n        %MyStruct{value: value * 2}\n      end\n\n      def decode(data), do: data\n    end\n\nThe protocol is optional with a fallback implementation for Any that returns the data unchanged.","ref":"Jido.Signal.Serialization.JsonDecoder.html#module-example"},{"type":"function","title":"Jido.Signal.Serialization.JsonDecoder.decode/1","doc":"Decodes data that has been deserialized using the `Jido.Signal.Serialization.JsonSerializer`.\n\nThis function is called after JSON deserialization to perform any additional data transformations\nor validations needed for the specific data type.","ref":"Jido.Signal.Serialization.JsonDecoder.html#decode/1"},{"type":"function","title":"Parameters - Jido.Signal.Serialization.JsonDecoder.decode/1","doc":"* `data` - The deserialized data to be decoded","ref":"Jido.Signal.Serialization.JsonDecoder.html#decode/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Serialization.JsonDecoder.decode/1","doc":"The decoded data, which can be transformed in any way appropriate for the implementing type.\n  The default implementation for Any simply returns the data unchanged.","ref":"Jido.Signal.Serialization.JsonDecoder.html#decode/1-returns"},{"type":"type","title":"Jido.Signal.Serialization.JsonDecoder.t/0","doc":"All the types that implement this protocol.","ref":"Jido.Signal.Serialization.JsonDecoder.html#t:t/0"},{"type":"module","title":"Jido.Signal.Serialization.JsonSerializer","doc":"A serializer that uses the JSON format and Jason library.","ref":"Jido.Signal.Serialization.JsonSerializer.html"},{"type":"function","title":"Jido.Signal.Serialization.JsonSerializer.deserialize/2","doc":"Deserialize given JSON binary data to the expected type.","ref":"Jido.Signal.Serialization.JsonSerializer.html#deserialize/2"},{"type":"function","title":"Jido.Signal.Serialization.JsonSerializer.serialize/2","doc":"Serialize given term to JSON binary data.","ref":"Jido.Signal.Serialization.JsonSerializer.html#serialize/2"},{"type":"module","title":"Jido.Signal.Serialization.ModuleNameTypeProvider","doc":"A type provider that uses the Elixir module name\n\nExample:\n\n  - %An.Event{} module mapped to \"Elixir.An.Event\".","ref":"Jido.Signal.Serialization.ModuleNameTypeProvider.html"},{"type":"function","title":"Jido.Signal.Serialization.ModuleNameTypeProvider.to_string/1","doc":"Convert a struct to a type string.","ref":"Jido.Signal.Serialization.ModuleNameTypeProvider.html#to_string/1"},{"type":"function","title":"Jido.Signal.Serialization.ModuleNameTypeProvider.to_struct/1","doc":"Convert a type string to a struct.","ref":"Jido.Signal.Serialization.ModuleNameTypeProvider.html#to_struct/1"},{"type":"module","title":"Jido.Signal.Serialization.MsgpackSerializer","doc":"A serializer that uses the MessagePack format via the Msgpax library.\n\nMessagePack is a binary serialization format that is more compact than JSON\nbut still platform-independent, making it ideal for network communication\nand storage.","ref":"Jido.Signal.Serialization.MsgpackSerializer.html"},{"type":"module","title":"Features - Jido.Signal.Serialization.MsgpackSerializer","doc":"- More compact than JSON\n- Preserves more data types than JSON\n- Cross-platform compatibility\n- Fast serialization/deserialization","ref":"Jido.Signal.Serialization.MsgpackSerializer.html#module-features"},{"type":"module","title":"Usage - Jido.Signal.Serialization.MsgpackSerializer","doc":"# Configure as default serializer\n    config :jido, :default_serializer, Jido.Signal.Serialization.MsgpackSerializer\n\n    # Or use explicitly\n    Signal.serialize(signal, serializer: Jido.Signal.Serialization.MsgpackSerializer)","ref":"Jido.Signal.Serialization.MsgpackSerializer.html#module-usage"},{"type":"module","title":"Type Handling - Jido.Signal.Serialization.MsgpackSerializer","doc":"MessagePack has limitations compared to Erlang terms:\n- Atoms are converted to strings\n- Tuples are converted to arrays\n- Custom structs need explicit handling","ref":"Jido.Signal.Serialization.MsgpackSerializer.html#module-type-handling"},{"type":"function","title":"Jido.Signal.Serialization.MsgpackSerializer.deserialize/2","doc":"Deserialize given MessagePack binary data back to the original format.","ref":"Jido.Signal.Serialization.MsgpackSerializer.html#deserialize/2"},{"type":"function","title":"Jido.Signal.Serialization.MsgpackSerializer.serialize/2","doc":"Serialize given term to MessagePack binary format.","ref":"Jido.Signal.Serialization.MsgpackSerializer.html#serialize/2"},{"type":"function","title":"Jido.Signal.Serialization.MsgpackSerializer.valid_msgpack?/1","doc":"Checks if the given binary is valid MessagePack data.","ref":"Jido.Signal.Serialization.MsgpackSerializer.html#valid_msgpack?/1"},{"type":"behaviour","title":"Jido.Signal.Serialization.Serializer","doc":"Behaviour for serialization strategies.\n\nThis behavior defines a common interface for different serialization formats\nsuch as JSON, Erlang Term format, and MessagePack.","ref":"Jido.Signal.Serialization.Serializer.html"},{"type":"behaviour","title":"Implementation Examples - Jido.Signal.Serialization.Serializer","doc":"defmodule MySerializer do\n      @behaviour Jido.Signal.Serialization.Serializer\n\n      def serialize(data, _opts \\ []) do\n        # Your serialization logic here\n        {:ok, serialized_data}\n      end\n\n      def deserialize(binary, opts \\ []) do\n        # Your deserialization logic here\n        {:ok, deserialized_data}\n      end\n    end","ref":"Jido.Signal.Serialization.Serializer.html#module-implementation-examples"},{"type":"behaviour","title":"Configuration - Jido.Signal.Serialization.Serializer","doc":"The default serializer can be configured in your application config:\n\n    config :jido, :default_serializer, MySerializer\n\nYou can also override the serializer per operation by passing the `:serializer` option:\n\n    Signal.serialize(signal, serializer: MySerializer)\n    Signal.deserialize(binary, serializer: MySerializer)","ref":"Jido.Signal.Serialization.Serializer.html#module-configuration"},{"type":"function","title":"Jido.Signal.Serialization.Serializer.default_serializer/0","doc":"Get the configured default serializer.","ref":"Jido.Signal.Serialization.Serializer.html#default_serializer/0"},{"type":"function","title":"Jido.Signal.Serialization.Serializer.deserialize/2","doc":"Deserialize data using the specified or default serializer.","ref":"Jido.Signal.Serialization.Serializer.html#deserialize/2"},{"type":"callback","title":"Jido.Signal.Serialization.Serializer.deserialize/2","doc":"Deserialize the given binary data back to the original format.","ref":"Jido.Signal.Serialization.Serializer.html#c:deserialize/2"},{"type":"callback","title":"Parameters - Jido.Signal.Serialization.Serializer.deserialize/2","doc":"* `binary` - The serialized binary data\n  * `opts` - Optional configuration (e.g., type information, type provider)","ref":"Jido.Signal.Serialization.Serializer.html#c:deserialize/2-parameters"},{"type":"callback","title":"Returns - Jido.Signal.Serialization.Serializer.deserialize/2","doc":"* `{:ok, term}` - Success with deserialized data\n  * `{:error, reason}` - Failure with reason","ref":"Jido.Signal.Serialization.Serializer.html#c:deserialize/2-returns"},{"type":"type","title":"Jido.Signal.Serialization.Serializer.opts/0","doc":"","ref":"Jido.Signal.Serialization.Serializer.html#t:opts/0"},{"type":"type","title":"Jido.Signal.Serialization.Serializer.result/0","doc":"","ref":"Jido.Signal.Serialization.Serializer.html#t:result/0"},{"type":"type","title":"Jido.Signal.Serialization.Serializer.serializable/0","doc":"","ref":"Jido.Signal.Serialization.Serializer.html#t:serializable/0"},{"type":"function","title":"Jido.Signal.Serialization.Serializer.serialize/2","doc":"Serialize data using the specified or default serializer.","ref":"Jido.Signal.Serialization.Serializer.html#serialize/2"},{"type":"callback","title":"Jido.Signal.Serialization.Serializer.serialize/2","doc":"Serialize the given term to binary format.","ref":"Jido.Signal.Serialization.Serializer.html#c:serialize/2"},{"type":"callback","title":"Parameters - Jido.Signal.Serialization.Serializer.serialize/2","doc":"* `data` - The data to serialize\n  * `opts` - Optional configuration (e.g., type information)","ref":"Jido.Signal.Serialization.Serializer.html#c:serialize/2-parameters"},{"type":"callback","title":"Returns - Jido.Signal.Serialization.Serializer.serialize/2","doc":"* `{:ok, binary}` - Success with serialized binary\n  * `{:error, reason}` - Failure with reason","ref":"Jido.Signal.Serialization.Serializer.html#c:serialize/2-returns"},{"type":"type","title":"Jido.Signal.Serialization.Serializer.serialized/0","doc":"","ref":"Jido.Signal.Serialization.Serializer.html#t:serialized/0"},{"type":"behaviour","title":"Jido.Signal.Serialization.TypeProvider","doc":"Specification to convert between an Elixir struct and a corresponding string type.","ref":"Jido.Signal.Serialization.TypeProvider.html"},{"type":"type","title":"Jido.Signal.Serialization.TypeProvider.t/0","doc":"","ref":"Jido.Signal.Serialization.TypeProvider.html#t:t/0"},{"type":"callback","title":"Jido.Signal.Serialization.TypeProvider.to_string/1","doc":"Type of the given Elixir struct as a string","ref":"Jido.Signal.Serialization.TypeProvider.html#c:to_string/1"},{"type":"callback","title":"Jido.Signal.Serialization.TypeProvider.to_struct/1","doc":"Convert the given type string to an Elixir struct","ref":"Jido.Signal.Serialization.TypeProvider.html#c:to_struct/1"},{"type":"type","title":"Jido.Signal.Serialization.TypeProvider.type/0","doc":"","ref":"Jido.Signal.Serialization.TypeProvider.html#t:type/0"},{"type":"function","title":"Jido.Signal.Serialization.TypeProvider.type_provider/0","doc":"Get the configured type provider","ref":"Jido.Signal.Serialization.TypeProvider.html#type_provider/0"},{"type":"module","title":"Jido.Signal.Topology","doc":"Provides a data structure for tracking process hierarchies and their states.\n\nThis module supports:\n- Registering processes with their PIDs\n- Establishing parent-child relationships between processes\n- Querying the process hierarchy in various ways\n- Accessing process state through GenServer calls","ref":"Jido.Signal.Topology.html"},{"type":"function","title":"Jido.Signal.Topology.count/1","doc":"Returns the number of processes in the topology.","ref":"Jido.Signal.Topology.html#count/1"},{"type":"function","title":"Jido.Signal.Topology.get_all_states/1","doc":"Retrieves states from all processes in the topology.","ref":"Jido.Signal.Topology.html#get_all_states/1"},{"type":"function","title":"Returns - Jido.Signal.Topology.get_all_states/1","doc":"* Map of process IDs to their states","ref":"Jido.Signal.Topology.html#get_all_states/1-returns"},{"type":"function","title":"Jido.Signal.Topology.get_descendant_states/2","doc":"Retrieves states from all descendants of a process.","ref":"Jido.Signal.Topology.html#get_descendant_states/2"},{"type":"function","title":"Returns - Jido.Signal.Topology.get_descendant_states/2","doc":"* Map of process IDs to their states","ref":"Jido.Signal.Topology.html#get_descendant_states/2-returns"},{"type":"function","title":"Jido.Signal.Topology.get_state/2","doc":"Retrieves state from the GenServer process.","ref":"Jido.Signal.Topology.html#get_state/2"},{"type":"function","title":"Parameters - Jido.Signal.Topology.get_state/2","doc":"* topology - The current topology\n  * id - ID of the process","ref":"Jido.Signal.Topology.html#get_state/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Topology.get_state/2","doc":"* `{:ok, state}` - State was retrieved successfully\n  * `{:error, :not_found}` - No process with this ID exists\n  * `{:error, reason}` - GenServer call failed","ref":"Jido.Signal.Topology.html#get_state/2-returns"},{"type":"function","title":"Jido.Signal.Topology.list_all/1","doc":"Returns a list of all processes in the topology.","ref":"Jido.Signal.Topology.html#list_all/1"},{"type":"function","title":"Jido.Signal.Topology.list_children/2","doc":"Returns a list of all direct children of a process.","ref":"Jido.Signal.Topology.html#list_children/2"},{"type":"function","title":"Jido.Signal.Topology.list_descendants/2","doc":"Returns all descendants of a process (children, grandchildren, etc.).","ref":"Jido.Signal.Topology.html#list_descendants/2"},{"type":"function","title":"Jido.Signal.Topology.list_roots/1","doc":"Returns a list of all root processes (those without parents).","ref":"Jido.Signal.Topology.html#list_roots/1"},{"type":"function","title":"Jido.Signal.Topology.new/0","doc":"Creates a new empty topology.","ref":"Jido.Signal.Topology.html#new/0"},{"type":"function","title":"Jido.Signal.Topology.print/2","doc":"Prints a visual representation of the hierarchy to the console.\nIf no process ID is provided, prints the entire forest from all root processes.","ref":"Jido.Signal.Topology.html#print/2"},{"type":"function","title":"Jido.Signal.Topology.register/4","doc":"Registers a new process in the topology.","ref":"Jido.Signal.Topology.html#register/4"},{"type":"function","title":"Parameters - Jido.Signal.Topology.register/4","doc":"* topology - The current topology\n  * id - Unique identifier for the process\n  * pid - Process ID\n  * opts - Optional parameters:\n    * name - Human-readable name\n    * metadata - Additional information\n    * parent_id - ID of the parent process (if any)","ref":"Jido.Signal.Topology.html#register/4-parameters"},{"type":"function","title":"Returns - Jido.Signal.Topology.register/4","doc":"* `{:ok, updated_topology}` - Process was registered successfully\n  * `{:error, :already_exists}` - A process with this ID already exists\n  * `{:error, :parent_not_found}` - The specified parent ID doesn't exist","ref":"Jido.Signal.Topology.html#register/4-returns"},{"type":"function","title":"Jido.Signal.Topology.set_parent/3","doc":"Establishes a parent-child relationship between two processes.","ref":"Jido.Signal.Topology.html#set_parent/3"},{"type":"function","title":"Parameters - Jido.Signal.Topology.set_parent/3","doc":"* topology - The current topology\n  * parent_id - ID of the parent process\n  * child_id - ID of the child process","ref":"Jido.Signal.Topology.html#set_parent/3-parameters"},{"type":"function","title":"Returns - Jido.Signal.Topology.set_parent/3","doc":"* `{:ok, updated_topology}` - Relationship established successfully\n  * `{:error, :not_found}` - One of the processes doesn't exist\n  * `{:error, :cycle_detected}` - This would create a cycle in the hierarchy","ref":"Jido.Signal.Topology.html#set_parent/3-returns"},{"type":"type","title":"Jido.Signal.Topology.t/0","doc":"The complete process hierarchy structure","ref":"Jido.Signal.Topology.html#t:t/0"},{"type":"function","title":"Jido.Signal.Topology.to_tree/2","doc":"Returns a tree representation of the hierarchy starting from the given process.\nIf no process ID is provided, returns the entire forest from all root processes.","ref":"Jido.Signal.Topology.html#to_tree/2"},{"type":"function","title":"Returns - Jido.Signal.Topology.to_tree/2","doc":"A tree structure where each node is a map containing:\n  * `:process` - The process node\n  * `:state` - The process state (if available)\n  * `:children` - A list of child tree nodes","ref":"Jido.Signal.Topology.html#to_tree/2-returns"},{"type":"function","title":"Jido.Signal.Topology.unregister/2","doc":"Unregisters a process and all its descendants from the topology.","ref":"Jido.Signal.Topology.html#unregister/2"},{"type":"function","title":"Parameters - Jido.Signal.Topology.unregister/2","doc":"* topology - The current topology\n  * id - ID of the process to remove","ref":"Jido.Signal.Topology.html#unregister/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Topology.unregister/2","doc":"* `{:ok, updated_topology}` - Process was removed successfully\n  * `{:error, :not_found}` - No process with this ID exists","ref":"Jido.Signal.Topology.html#unregister/2-returns"},{"type":"function","title":"Jido.Signal.Topology.update_metadata/3","doc":"Updates a process's metadata.","ref":"Jido.Signal.Topology.html#update_metadata/3"},{"type":"function","title":"Returns - Jido.Signal.Topology.update_metadata/3","doc":"* `{:ok, updated_topology}` - Metadata was updated successfully\n  * `{:error, :not_found}` - No process with this ID exists","ref":"Jido.Signal.Topology.html#update_metadata/3-returns"},{"type":"module","title":"Jido.Signal.Topology.ProcessNode","doc":"Represents a single process node in the process hierarchy topology.\n\nContains information about a process including its PID, relationships to other\nprocesses, and associated metadata. Used to track parent-child relationships\nand query the process hierarchy structure.","ref":"Jido.Signal.Topology.ProcessNode.html"},{"type":"type","title":"Jido.Signal.Topology.ProcessNode.t/0","doc":"Represents a single process in the hierarchy","ref":"Jido.Signal.Topology.ProcessNode.html#t:t/0"},{"type":"module","title":"Jido.Signal.Util","doc":"A collection of utility functions for the Jido framework.\n\nThis module provides various helper functions that are used throughout the Jido framework,\nincluding:\n\n- ID generation\n- Name validation\n- Error handling\n- Logging utilities\n\nThese utilities are designed to support common operations and maintain consistency\nacross the Jido ecosystem. They encapsulate frequently used patterns and provide\na centralized location for shared functionality.\n\nMany of the functions in this module are used internally by other Jido modules,\nbut they can also be useful for developers building applications with Jido.","ref":"Jido.Signal.Util.html"},{"type":"type","title":"Jido.Signal.Util.server/0","doc":"","ref":"Jido.Signal.Util.html#t:server/0"},{"type":"function","title":"Jido.Signal.Util.via_tuple/2","doc":"Creates a via tuple for process registration with a registry.","ref":"Jido.Signal.Util.html#via_tuple/2"},{"type":"function","title":"Parameters - Jido.Signal.Util.via_tuple/2","doc":"- name: The name to register (atom, string, or {name, registry} tuple)\n- opts: Options list\n  - :registry - The registry module to use (defaults to Jido.Signal.Registry)","ref":"Jido.Signal.Util.html#via_tuple/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Util.via_tuple/2","doc":"A via tuple for use with process registration","ref":"Jido.Signal.Util.html#via_tuple/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Util.via_tuple/2","doc":"iex> Jido.Signal.Util.via_tuple(:my_process)\n    {:via, Registry, {Jido.Signal.Registry, \"my_process\"}}\n\n    iex> Jido.Signal.Util.via_tuple(:my_process, registry: MyRegistry)\n    {:via, Registry, {MyRegistry, \"my_process\"}}\n\n    iex> Jido.Signal.Util.via_tuple({:my_process, MyRegistry})\n    {:via, Registry, {MyRegistry, \"my_process\"}}","ref":"Jido.Signal.Util.html#via_tuple/2-examples"},{"type":"function","title":"Jido.Signal.Util.whereis/2","doc":"Finds a process by name, pid, or {name, registry} tuple.","ref":"Jido.Signal.Util.html#whereis/2"},{"type":"function","title":"Parameters - Jido.Signal.Util.whereis/2","doc":"- server: The process identifier (pid, name, or {name, registry} tuple)\n- opts: Options list\n  - :registry - The registry module to use (defaults to Jido.Signal.Registry)","ref":"Jido.Signal.Util.html#whereis/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Util.whereis/2","doc":"- `{:ok, pid}` if process is found\n- `{:error, :not_found}` if process is not found","ref":"Jido.Signal.Util.html#whereis/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Util.whereis/2","doc":"iex> Jido.Signal.Util.whereis(pid)\n    {:ok, #PID<0.123.0>}\n\n    iex> Jido.Signal.Util.whereis(:my_process)\n    {:ok, #PID<0.124.0>}\n\n    iex> Jido.Signal.Util.whereis({:my_process, MyRegistry})\n    {:ok, #PID<0.125.0>}","ref":"Jido.Signal.Util.html#whereis/2-examples"},{"type":"extras","title":"Jido.Signal","doc":"# Jido.Signal\n\n[![Hex.pm](https://img.shields.io/hexpm/v/jido_signal.svg)](https://hex.pm/packages/jido_signal)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/jido_signal/)\n[![CI](https://github.com/agentjido/jido_signal/actions/workflows/elixir-ci.yml/badge.svg)](https://github.com/agentjido/jido_signal/actions/workflows/elixir-ci.yml)\n[![License](https://img.shields.io/hexpm/l/jido_signal.svg)](https://github.com/agentjido/jido_signal/blob/main/LICENSE.md)\n[![Coverage Status](https://coveralls.io/repos/github/agentjido/jido_signal/badge.svg?branch=main)](https://coveralls.io/github/agentjido/jido_signal?branch=main)\n\n> **Agent Communication Envelope and Utilities**\n\n_`Jido.Signal` is part of the [Jido](https://github.com/agentjido/jido) project. Learn more about Jido at [agentjido.xyz](https://agentjido.xyz)._","ref":"readme.html"},{"type":"extras","title":"Overview - Jido.Signal","doc":"`Jido.Signal` is a sophisticated toolkit for building event-driven and agent-based systems in Elixir. It provides a complete ecosystem for defining, routing, dispatching, and tracking signals throughout your application, built on the CloudEvents v1.0.2 specification with powerful Jido-specific extensions.\n\nWhether you're building microservices that need reliable event communication, implementing complex agent-based systems, or creating observable distributed applications, Jido.Signal provides the foundation for robust, traceable, and scalable event-driven architecture.","ref":"readme.html#overview"},{"type":"extras","title":"Why Do I Need Signals? - Jido.Signal","doc":"**Agent Communication in Elixir's Process-Driven World**\n\nElixir's strength lies in lightweight processes that communicate via message passing, but raw message passing has limitations when building complex systems:\n\n- **Phoenix Channels** need structured event broadcasting across connections\n- **GenServers** require reliable inter-process communication with context\n- **Agent Systems** demand traceable conversations between autonomous processes\n- **Distributed Services** need standardized message formats across nodes\n\nTraditional Elixir messaging (`send`, `GenServer.cast/call`) works great for simple scenarios, but falls short when you need:\n\n- **Standardized Message Format**: Raw tuples and maps lack structure and metadata\n- **Event Routing**: Broadcasting to multiple interested processes based on patterns\n- **Conversation Tracking**: Understanding which message caused which response\n- **Reliable Delivery**: Ensuring critical messages aren't lost if a process crashes\n- **Cross-System Integration**: Communicating with external services via webhooks/HTTP\n\n```elixir\n# Traditional Elixir messaging\nGenServer.cast(my_server, {:user_created, user_id, email})  # Unstructured\nsend(pid, {:event, data})  # No routing or reliability\n\n# With Jido.Signal\n{:ok, signal} = UserCreated.new(%{user_id: user_id, email: email})\nBus.publish(:app_bus, [signal])  # Structured, routed, traceable, reliable\n```\n\nJido.Signal transforms Elixir's message passing into a sophisticated communication system that scales from simple GenServer interactions to complex multi-agent orchestration across distributed systems.","ref":"readme.html#why-do-i-need-signals"},{"type":"extras","title":"Key Features - Jido.Signal","doc":"### **Standardized Signal Structure**\n- CloudEvents v1.0.2 compliant message format\n- Custom signal types with data validation\n- Rich metadata and context tracking\n- Flexible serialization (JSON, MessagePack, Erlang Term Format)\n\n### **High-Performance Signal Bus**\n- In-memory GenServer-based pub/sub system\n- Persistent subscriptions with acknowledgment\n- Middleware pipeline for cross-cutting concerns\n- Complete signal history with replay capabilities\n\n### **Advanced Routing Engine**\n- Trie-based pattern matching for optimal performance\n- Wildcard support (`*` single-level, `**` multi-level)\n- Priority-based execution ordering\n- Custom pattern matching functions\n\n### **Pluggable Dispatch System**\n- Multiple delivery adapters (PID, PubSub, HTTP, Logger, Console)\n- Synchronous and asynchronous delivery modes\n- Batch processing for high-throughput scenarios\n- Configurable timeout and retry mechanisms\n\n### **Causality & Conversation Tracking**\n- Complete signal relationship graphs\n- Cause-effect chain analysis\n- Conversation grouping and temporal ordering\n- Comprehensive system traceability for debugging and auditing","ref":"readme.html#key-features"},{"type":"extras","title":"Installation - Jido.Signal","doc":"Add `jido_signal` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:jido_signal, \"~> 1.0\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - Jido.Signal","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"1. Start a Signal Bus - Jido.Signal","doc":"Add to your application's supervision tree:\n\n```elixir\n# In your application.ex\nchildren = [\n  {Jido.Signal.Bus, name: :my_app_bus}\n]\n\nSupervisor.start_link(children, strategy: :one_for_one)\n```","ref":"readme.html#1-start-a-signal-bus"},{"type":"extras","title":"2. Create a Subscriber - Jido.Signal","doc":"```elixir\ndefmodule MySubscriber do\n  use GenServer\n\n  def start_link(_opts), do: GenServer.start_link(__MODULE__, %{})\n  def init(state), do: {:ok, state}\n\n  # Handle incoming signals\n  def handle_info({:signal, signal}, state) do\n    IO.puts(\"Received: #{signal.type}\")\n    {:noreply, state}\n  end\nend\n```","ref":"readme.html#2-create-a-subscriber"},{"type":"extras","title":"3. Subscribe and Publish - Jido.Signal","doc":"```elixir\nalias Jido.Signal.Bus\nalias Jido.Signal\n\n# Start subscriber and subscribe to user events\n{:ok, sub_pid} = MySubscriber.start_link([])\n{:ok, _sub_id} = Bus.subscribe(:my_app_bus, \"user.*\", dispatch: {:pid, target: sub_pid})\n\n# Create and publish a signal\n{:ok, signal} = Signal.new(%{\n  type: \"user.created\",\n  source: \"/auth/registration\",\n  data: %{user_id: \"123\", email: \"user@example.com\"}\n})\n\nBus.publish(:my_app_bus, [signal])\n# Output: \"Received: user.created\"\n```","ref":"readme.html#3-subscribe-and-publish"},{"type":"extras","title":"Core Concepts - Jido.Signal","doc":"","ref":"readme.html#core-concepts"},{"type":"extras","title":"The Signal - Jido.Signal","doc":"Signals are CloudEvents-compliant message envelopes that carry your application's events:\n\n```elixir\n# Basic signal\n{:ok, signal} = Signal.new(%{\n  type: \"order.created\",\n  source: \"/ecommerce/orders\",\n  data: %{order_id: \"ord_123\", amount: 99.99}\n})\n\n# With dispatch configuration\n{:ok, signal} = Signal.new(%{\n  type: \"payment.processed\",\n  source: \"/payments\",\n  data: %{payment_id: \"pay_456\"},\n  jido_dispatch: [\n    {:pubsub, topic: \"payments\"},\n    {:webhook, url: \"https://api.partner.com/webhook\", secret: \"secret123\"}\n  ]\n})\n```","ref":"readme.html#the-signal"},{"type":"extras","title":"Custom Signal Types - Jido.Signal","doc":"Define strongly-typed signals with validation:\n\n```elixir\ndefmodule UserCreated do\n  use Jido.Signal,\n    type: \"user.created.v1\",\n    default_source: \"/users\",\n    schema: [\n      user_id: [type: :string, required: true],\n      email: [type: :string, required: true, format: ~r/@/],\n      name: [type: :string, required: true]\n    ]\nend\n\n# Usage\n{:ok, signal} = UserCreated.new(%{\n  user_id: \"u_123\",\n  email: \"john@example.com\",\n  name: \"John Doe\"\n})\n\n# Validation errors\n{:error, reason} = UserCreated.new(%{user_id: \"u_123\"})\n# => {:error, \"Invalid data for Signal: Required key :email not found\"}\n```","ref":"readme.html#custom-signal-types"},{"type":"extras","title":"The Router - Jido.Signal","doc":"Powerful pattern matching for signal routing:\n\n```elixir\nalias Jido.Signal.Router\n\nroutes = [\n  # Exact matches have highest priority\n  {\"user.created\", :handle_user_creation},\n  \n  # Single-level wildcards\n  {\"user.*.updated\", :handle_user_updates},\n  \n  # Multi-level wildcards\n  {\"audit.**\", :audit_logger, 100},  # High priority\n  \n  # Pattern matching functions\n  {fn signal -> String.contains?(signal.type, \"error\") end, :error_handler}\n]\n\n{:ok, router} = Router.new(routes)\n\n# Route signals to handlers\n{:ok, targets} = Router.route(router, %Signal{type: \"user.profile.updated\"})\n# => {:ok, [:handle_user_updates, :audit_logger]}\n```","ref":"readme.html#the-router"},{"type":"extras","title":"Dispatch System - Jido.Signal","doc":"Flexible delivery to multiple destinations:\n\n```elixir\nalias Jido.Signal.Dispatch\n\ndispatch_configs = [\n  # Send to process\n  {:pid, target: my_process_pid},\n  \n  # Publish via Phoenix.PubSub\n  {:pubsub, target: MyApp.PubSub, topic: \"events\"},\n  \n  # HTTP webhook with signature\n  {:webhook, url: \"https://api.example.com/webhook\", secret: \"secret123\"},\n  \n  # Log structured data\n  {:logger, level: :info, structured: true},\n  \n  # Console output\n  {:console, format: :pretty}\n]\n\n# Synchronous dispatch\n:ok = Dispatch.dispatch(signal, dispatch_configs)\n\n# Asynchronous dispatch\n{:ok, task} = Dispatch.dispatch_async(signal, dispatch_configs)\n```","ref":"readme.html#dispatch-system"},{"type":"extras","title":"Advanced Features - Jido.Signal","doc":"","ref":"readme.html#advanced-features"},{"type":"extras","title":"Persistent Subscriptions - Jido.Signal","doc":"Ensure reliable message delivery with acknowledgments:\n\n```elixir\n# Create persistent subscription\n{:ok, sub_id} = Bus.subscribe(:my_app_bus, \"payment.*\", \n  persistent?: true, \n  dispatch: {:pid, target: self()}\n)\n\n# Receive and acknowledge signals\nreceive do\n  {:signal, signal} ->\n    # Process the signal\n    process_payment(signal)\n    \n    # Acknowledge successful processing\n    Bus.ack(:my_app_bus, sub_id, signal.id)\nend\n\n# If subscriber crashes and restarts\nBus.reconnect(:my_app_bus, sub_id, self())\n# Unacknowledged signals are automatically replayed\n```","ref":"readme.html#persistent-subscriptions"},{"type":"extras","title":"Middleware Pipeline - Jido.Signal","doc":"Add cross-cutting concerns with middleware:\n\n```elixir\nmiddleware = [\n  # Built-in logging middleware\n  {Jido.Signal.Bus.Middleware.Logger, [\n    level: :info,\n    include_signal_data: true\n  ]},\n  \n  # Custom middleware\n  {MyApp.AuthMiddleware, []},\n  {MyApp.MetricsMiddleware, []}\n]\n\n{:ok, _pid} = Jido.Signal.Bus.start_link(\n  name: :my_bus, \n  middleware: middleware\n)\n```","ref":"readme.html#middleware-pipeline"},{"type":"extras","title":"Causality Tracking - Jido.Signal","doc":"Track signal relationships for complete system observability:\n\n```elixir\nalias Jido.Signal.Journal\n\n# Create journal\njournal = Journal.new()\n\n# Record causal relationships\nJournal.record(journal, initial_signal, nil)  # Root cause\nJournal.record(journal, response_signal, initial_signal.id)  # Caused by initial_signal\nJournal.record(journal, side_effect, initial_signal.id)     # Also caused by initial_signal\n\n# Analyze relationships\neffects = Journal.get_effects(journal, initial_signal.id)\n# => [response_signal, side_effect]\n\ncause = Journal.get_cause(journal, response_signal.id)\n# => initial_signal\n```","ref":"readme.html#causality-tracking"},{"type":"extras","title":"Signal History & Replay - Jido.Signal","doc":"Access complete signal history:\n\n```elixir\n# Get recent signals matching pattern\n{:ok, signals} = Bus.replay(:my_app_bus, \"user.*\", \n  since: DateTime.utc_now() |> DateTime.add(-3600, :second),\n  limit: 100\n)\n\n# Replay to new subscriber\n{:ok, new_sub} = Bus.subscribe(:my_app_bus, \"user.*\", \n  dispatch: {:pid, target: new_process_pid},\n  replay_since: DateTime.utc_now() |> DateTime.add(-1800, :second)\n)\n```","ref":"readme.html#signal-history-replay"},{"type":"extras","title":"Snapshots - Jido.Signal","doc":"Create point-in-time views of your signal log:\n\n```elixir\n# Create filtered snapshot\n{:ok, snapshot_id} = Bus.snapshot_create(:my_app_bus, %{\n  path_pattern: \"order.**\",\n  since: ~U[2024-01-01 00:00:00Z],\n  until: ~U[2024-01-31 23:59:59Z]\n})\n\n# Read snapshot data\n{:ok, signals} = Bus.snapshot_read(:my_app_bus, snapshot_id)\n\n# Export or analyze the signals\nEnum.each(signals, &analyze_order_signal/1)\n```","ref":"readme.html#snapshots"},{"type":"extras","title":"Use Cases - Jido.Signal","doc":"","ref":"readme.html#use-cases"},{"type":"extras","title":"Microservices Communication - Jido.Signal","doc":"```elixir\n# Service A publishes order events\n{:ok, signal} = OrderCreated.new(%{order_id: \"123\", customer_id: \"456\"})\nBus.publish(:event_bus, [signal])\n\n# Service B processes inventory\n# Service C sends notifications  \n# Service D updates analytics\n```","ref":"readme.html#microservices-communication"},{"type":"extras","title":"Agent-Based Systems - Jido.Signal","doc":"```elixir\n# Agents communicate via signals\n{:ok, signal} = AgentMessage.new(%{\n  from_agent: \"agent_1\",\n  to_agent: \"agent_2\", \n  action: \"negotiate_price\",\n  data: %{product_id: \"prod_123\", offered_price: 99.99}\n})\n```","ref":"readme.html#agent-based-systems"},{"type":"extras","title":"Event Sourcing - Jido.Signal","doc":"```elixir\n# Commands become events\n{:ok, command_signal} = CreateUser.new(user_data)\n{:ok, event_signal} = UserCreated.new(user_data, cause: command_signal.id)\n\n# Store in journal for complete audit trail\nJournal.record(journal, event_signal, command_signal.id)\n```","ref":"readme.html#event-sourcing"},{"type":"extras","title":"Distributed Workflows - Jido.Signal","doc":"```elixir\n# Coordinate multi-step processes\nworkflow_signals = [\n  %Signal{type: \"workflow.started\", data: %{workflow_id: \"wf_123\"}},\n  %Signal{type: \"step.completed\", data: %{step: 1, workflow_id: \"wf_123\"}},\n  %Signal{type: \"step.completed\", data: %{step: 2, workflow_id: \"wf_123\"}},\n  %Signal{type: \"workflow.completed\", data: %{workflow_id: \"wf_123\"}}\n]\n```","ref":"readme.html#distributed-workflows"},{"type":"extras","title":"Documentation - Jido.Signal","doc":"- **[Getting Started Guide](https://hexdocs.pm/jido_signal/getting-started.html)** - Step-by-step tutorial\n- **[Core Concepts](https://hexdocs.pm/jido_signal/core-concepts.html)** - Deep dive into architecture  \n- **[API Reference](https://hexdocs.pm/jido_signal)** - Complete function documentation\n- **[Guides](https://hexdocs.pm/jido_signal/guides.html)** - Advanced usage patterns","ref":"readme.html#documentation"},{"type":"extras","title":"Development - Jido.Signal","doc":"","ref":"readme.html#development"},{"type":"extras","title":"Prerequisites - Jido.Signal","doc":"- Elixir 1.17+\n- Erlang/OTP 26+","ref":"readme.html#prerequisites"},{"type":"extras","title":"Setup - Jido.Signal","doc":"```bash\ngit clone https://github.com/agentjido/jido_signal.git\ncd jido_signal\nmix deps.get\n```","ref":"readme.html#setup"},{"type":"extras","title":"Running Tests - Jido.Signal","doc":"```bash\nmix test\n```","ref":"readme.html#running-tests"},{"type":"extras","title":"Quality Checks - Jido.Signal","doc":"```bash\nmix quality  # Runs formatter, dialyzer, and credo\n```","ref":"readme.html#quality-checks"},{"type":"extras","title":"Generate Documentation - Jido.Signal","doc":"```bash\nmix docs\n```","ref":"readme.html#generate-documentation"},{"type":"extras","title":"Contributing - Jido.Signal","doc":"We welcome contributions! Please see our [Contributing Guide](CONTRIBUTING.md) for details on:\n\n- Setting up your development environment\n- Running tests and quality checks\n- Submitting pull requests\n- Code style guidelines","ref":"readme.html#contributing"},{"type":"extras","title":"License - Jido.Signal","doc":"This project is licensed under the Apache License 2.0 - see the [LICENSE.md](LICENSE.md) file for details.","ref":"readme.html#license"},{"type":"extras","title":"Related Projects - Jido.Signal","doc":"- **[Jido](https://github.com/agentjido/jido)** - The main Jido agent framework\n- **[Jido Workbench](https://github.com/agentjido/jido_workbench)** - Development tools and utilities","ref":"readme.html#related-projects"},{"type":"extras","title":"Links - Jido.Signal","doc":"- [Hex Package](https://hex.pm/packages/jido_signal)\n- [Documentation](https://hexdocs.pm/jido_signal)\n- [GitHub Repository](https://github.com/agentjido/jido_signal)\n- [Agent Jido Website](https://agentjido.xyz)\n- [CloudEvents Specification](https://cloudevents.io/)\n\n---\n\n**Built with ❤️ by the Jido team**","ref":"readme.html#links"},{"type":"extras","title":"Contributing to Jido","doc":"# Contributing to Jido\n\nWelcome to the Jido contributor's guide! We're excited that you're interested in contributing to Jido, our framework for building autonomous, distributed agent systems in Elixir. This guide will help you understand our development process and standards.","ref":"contributing.html"},{"type":"extras","title":"Getting Started - Contributing to Jido","doc":"","ref":"contributing.html#getting-started"},{"type":"extras","title":"Development Environment - Contributing to Jido","doc":"1. **Elixir Version Requirements**\n   - Jido requires Elixir ~> 1.17\n   - We recommend using asdf or similar version manager\n\n2. **Initial Setup**\n   ```bash\n   # Clone the repository\n   git clone https://github.com/agentjido/jido.git\n   cd jido\n\n   # Install dependencies\n   mix deps.get\n\n   # Run tests to verify your setup\n   mix test\n   ```\n\n3. **Quality Checks**\n   ```bash\n   # Run the full quality check suite\n   mix quality\n\n   # Or individual checks\n   mix format                    # Format code\n   mix compile --warnings-as-errors  # Check compilation\n   mix dialyzer                  # Type checking\n   mix credo --all              # Static analysis\n   ```","ref":"contributing.html#development-environment"},{"type":"extras","title":"Code Organization - Contributing to Jido","doc":"","ref":"contributing.html#code-organization"},{"type":"extras","title":"Project Structure - Contributing to Jido","doc":"```\n.\n├── lib/\n│   ├── jido/\n│   │   ├── actions/     # Core action implementations\n│   │   ├── agents/      # Agent behaviors and implementations\n│   │   ├── sensors/     # Sensor system components\n│   │   ├── workflows/   # Workflow execution engines\n│   │   └── utils/       # Utility functions\n│   └── jido.ex          # Main entry point\n├── test/\n│   ├── jido/\n│   │   └── ...         # Tests mirroring lib structure\n│   ├── support/        # Test helpers and shared fixtures\n│   └── test_helper.exs\n├── guides/            # Documentation guides\n└── mix.exs\n```","ref":"contributing.html#project-structure"},{"type":"extras","title":"Core Components - Contributing to Jido","doc":"- **Actions**: Discrete, composable units of work\n- **Workflows**: Sequences of actions that accomplish larger goals\n- **Agents**: Stateful entities that can plan and execute workflows\n- **Sensors**: Real-time monitoring and data gathering components","ref":"contributing.html#core-components"},{"type":"extras","title":"Development Guidelines - Contributing to Jido","doc":"","ref":"contributing.html#development-guidelines"},{"type":"extras","title":"Code Style - Contributing to Jido","doc":"1. **Formatting**\n   - Run `mix format` before committing\n   - Follow standard Elixir style guide\n   - Use `snake_case` for functions and variables\n   - Use `PascalCase` for module names\n\n2. **Documentation**\n   - Add `@moduledoc` to every module\n   - Document all public functions with `@doc`\n   - Include examples when helpful\n   - Keep guides up-to-date in the `guides/` directory\n   - Use doctests for simple examples\n\n3. **Type Specifications**\n   ```elixir\n   @type validation_error :: :invalid_name | :invalid_status\n   \n   @spec process(String.t()) :: {:ok, term()} | {:error, validation_error()}\n   def process(input) do\n     # Implementation\n   end\n   ```","ref":"contributing.html#code-style"},{"type":"extras","title":"Testing - Contributing to Jido","doc":"1. **Test Organization**\n   ```elixir\n   defmodule Jido.Test.Actions.FormatUserTest do\n     use ExUnit.Case, async: true\n     \n     describe \"run/2\" do\n       test \"formats user data correctly\" do\n         # Test implementation\n       end\n       \n       test \"handles invalid input\" do\n         # Error case testing\n       end\n     end\n   end\n   ```\n\n2. **Coverage Requirements**\n   - Maintain high test coverage\n   - Test both success and error paths\n   - Include property-based tests for complex logic\n   - Test async behavior where applicable\n\n3. **Running Tests**\n   ```bash\n   # Run full test suite\n   mix test\n\n   # Run with coverage\n   mix test --cover\n\n   # Run specific test file\n   mix test test/jido/actions/format_user_test.exs\n   ```","ref":"contributing.html#testing"},{"type":"extras","title":"Error Handling - Contributing to Jido","doc":"1. **Use With Patterns**\n   ```elixir\n   def complex_operation(input) do\n     with {:ok, validated} <- validate(input),\n          {:ok, processed} <- process(validated) do\n       {:ok, processed}\n     end\n   end\n   ```\n\n2. **Return Values**\n   - Use tagged tuples: `{:ok, result}` or `{:error, reason}`\n   - Create specific error types for different failures\n   - Never intentionally raise Exceptions, the Jido framework intentionally minimizes the use of exceptions.\n   - Avoid silent failures\n   - Document error conditions","ref":"contributing.html#error-handling"},{"type":"extras","title":"Performance Considerations - Contributing to Jido","doc":"1. **Optimization**\n   - Profile before optimizing\n   - Document performance characteristics\n   - Consider resource usage in distributed environments\n   - Implement appropriate timeouts\n\n2. **Resource Management**\n   - Clean up resources properly\n   - Handle large data sets efficiently\n   - Consider memory usage in long-running processes","ref":"contributing.html#performance-considerations"},{"type":"extras","title":"Pull Request Process - Contributing to Jido","doc":"1. **Before Submitting**\n   - Run the full quality check suite: `mix quality`\n   - Ensure all tests pass\n   - Update documentation if needed\n   - Add tests for new functionality\n\n2. **PR Guidelines**\n   - Create a feature branch from `main`\n   - Use descriptive commit messages\n   - Reference any related issues\n   - Keep changes focused and atomic\n\n3. **Review Process**\n   - PRs require at least one review\n   - Address all review comments\n   - Maintain a clean commit history\n   - Update your branch if needed","ref":"contributing.html#pull-request-process"},{"type":"extras","title":"Release Process - Contributing to Jido","doc":"1. **Version Numbers**\n   - Follow semantic versioning\n   - Update version in `mix.exs`\n   - Update CHANGELOG.md\n\n2. **Documentation**\n   - Update guides if needed\n   - Check all docstrings\n   - Verify README is current","ref":"contributing.html#release-process"},{"type":"extras","title":"Additional Resources - Contributing to Jido","doc":"- [Hex Documentation](https://hexdocs.pm/jido)\n- [GitHub Issues](https://github.com/agentjido/jido/issues)\n- [GitHub Discussions](https://github.com/agentjido/jido/discussions)","ref":"contributing.html#additional-resources"},{"type":"extras","title":"Questions or Problems? - Contributing to Jido","doc":"If you have questions about contributing:\n- Open a GitHub Discussion\n- Check existing issues\n- Review the guides directory\n\nThank you for contributing to Jido!","ref":"contributing.html#questions-or-problems"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2019 Parker Selbert\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"},{"type":"extras","title":"What is Jido.Signal?","doc":"# What is Jido.Signal?\n\nJido.Signal is a comprehensive toolkit for building event-driven and agent-based systems in Elixir. It provides a standardized approach to inter-service communication through a powerful signal-based architecture.","ref":"what-is-jido-signal.html"},{"type":"extras","title":"Core Value Propositions - What is Jido.Signal?","doc":"- **Standardized Signals**: Universal message envelopes that follow the CloudEvents specification\n- **High-Performance Routing**: Intelligent, trie-based engine for matching signals to handlers\n- **Pluggable Dispatch**: Flexible system for sending signals to various destinations\n- **System Traceability**: Built-in causality tracking for understanding signal relationships","ref":"what-is-jido-signal.html#core-value-propositions"},{"type":"extras","title":"Key Features - What is Jido.Signal?","doc":"- **In-Process Pub/Sub**: Fast, reliable message passing within your application\n- **External Integration**: Send signals to HTTP endpoints, webhooks, and other services\n- **Persistent Subscriptions**: Ensure at-least-once delivery for critical events\n- **Middleware Support**: Interceptor pattern for cross-cutting concerns\n- **Journaling**: Track causality and audit trails across your system","ref":"what-is-jido-signal.html#key-features"},{"type":"extras","title":"CloudEvents Compatibility - What is Jido.Signal?","doc":"Jido.Signal aligns with the [CloudEvents specification](https://cloudevents.io/), ensuring your signals are portable and interoperable with other event-driven systems.","ref":"what-is-jido-signal.html#cloudevents-compatibility"},{"type":"extras","title":"Core Components at a Glance","doc":"# Core Components at a Glance\n\nJido.Signal is built around five main components that work together to provide a complete event-driven architecture:","ref":"core-components.html"},{"type":"extras","title":"Signal - Core Components at a Glance","doc":"The **Signal** is the universal message envelope that carries your data throughout the system. Every signal contains:\n\n- Metadata (id, source, type, timestamp)\n- Payload data\n- Routing information\n- Causality tracking","ref":"core-components.html#signal"},{"type":"extras","title":"Bus - Core Components at a Glance","doc":"The **Bus** serves as the central pub/sub hub for in-process communication. It:\n\n- Manages subscriptions and routing\n- Maintains a signal log for replay capabilities\n- Handles persistent subscriptions\n- Provides middleware integration points","ref":"core-components.html#bus"},{"type":"extras","title":"Router - Core Components at a Glance","doc":"The **Router** is an intelligent, trie-based engine that:\n\n- Matches signals to handlers using path patterns\n- Supports wildcards (`*` and `**`)\n- Prioritizes handlers by specificity and priority\n- Enables complex routing logic","ref":"core-components.html#router"},{"type":"extras","title":"Dispatch - Core Components at a Glance","doc":"The **Dispatch** system is a pluggable architecture for sending signals to external destinations:\n\n- Built-in adapters for HTTP, webhooks, PubSub, and more\n- Asynchronous and batch processing capabilities\n- Configurable retry and error handling\n- Custom adapter support","ref":"core-components.html#dispatch"},{"type":"extras","title":"Journal - Core Components at a Glance","doc":"The **Journal** acts as a causality ledger that:\n\n- Tracks relationships between signals\n- Enables audit trails and debugging\n- Supports complex workflow analysis\n- Provides signal ancestry tracking\n\nThese components work together seamlessly to provide a robust foundation for event-driven applications.","ref":"core-components.html#journal"},{"type":"extras","title":"When to Use Jido.Signal","doc":"# When to Use Jido.Signal\n\nJido.Signal is designed to solve common challenges in distributed and event-driven systems. Here are the primary use cases where it excels:","ref":"when-to-use.html"},{"type":"extras","title":"Decoupling Services - When to Use Jido.Signal","doc":"Use Jido.Signal when you need to reduce tight coupling between components:\n\n- **Microservices Communication**: Enable services to communicate without direct dependencies\n- **Module Isolation**: Keep different parts of your application loosely coupled\n- **Plugin Architecture**: Build extensible systems where components can be added/removed dynamically","ref":"when-to-use.html#decoupling-services"},{"type":"extras","title":"Agent-to-Agent Communication - When to Use Jido.Signal","doc":"Perfect for multi-agent systems where autonomous components need to coordinate:\n\n- **Distributed AI Systems**: Enable agents to share information and coordinate actions\n- **Workflow Orchestration**: Chain together different processing steps\n- **Event-Driven Automation**: Trigger actions based on system events","ref":"when-to-use.html#agent-to-agent-communication"},{"type":"extras","title":"Building Observable Systems - When to Use Jido.Signal","doc":"Gain visibility into your system's behavior:\n\n- **Audit Trails**: Track every action and decision in your system\n- **Debugging Complex Flows**: Understand how data flows through your application\n- **Performance Monitoring**: Measure and optimize system performance\n- **Compliance Reporting**: Generate reports for regulatory requirements","ref":"when-to-use.html#building-observable-systems"},{"type":"extras","title":"Event Sourcing Patterns - When to Use Jido.Signal","doc":"Implement event sourcing and CQRS architectures:\n\n- **State Reconstruction**: Rebuild application state from events\n- **Temporal Queries**: Query your system's state at any point in time\n- **Event Replay**: Reprocess events to fix bugs or add new features\n- **Read Model Projection**: Build optimized views from event streams","ref":"when-to-use.html#event-sourcing-patterns"},{"type":"extras","title":"Real-Time Applications - When to Use Jido.Signal","doc":"Handle time-sensitive operations:\n\n- **Live Updates**: Push changes to connected clients in real-time\n- **Notification Systems**: Deliver alerts and messages instantly\n- **Collaborative Features**: Synchronize state across multiple users\n- **IoT Data Processing**: Handle high-volume sensor data streams","ref":"when-to-use.html#real-time-applications"},{"type":"extras","title":"Integration Scenarios - When to Use Jido.Signal","doc":"Connect with external systems:\n\n- **Webhook Processing**: Handle incoming webhooks from third-party services\n- **API Gateway Events**: Process and route API requests\n- **Legacy System Integration**: Bridge old and new systems\n- **Third-Party Service Coordination**: Orchestrate calls to external APIs\n\nChoose Jido.Signal when you need a robust, scalable foundation for event-driven architecture that grows with your application's complexity.","ref":"when-to-use.html#integration-scenarios"},{"type":"extras","title":"Getting Started: Your First Signal Bus","doc":"# Getting Started: Your First Signal Bus\n\nThis guide will get you up and running with Jido.Signal in minutes. We'll create a simple pub/sub system to demonstrate the core concepts.","ref":"getting-started.html"},{"type":"extras","title":"Installation - Getting Started: Your First Signal Bus","doc":"Add `jido_signal` to your dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:jido_signal, \"~> 1.0\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"getting-started.html#installation"},{"type":"extras","title":"Starting a Bus - Getting Started: Your First Signal Bus","doc":"Add a named bus to your application's supervision tree. In your `application.ex`:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      # Other supervisors...\n      {Jido.Signal.Bus, name: :my_bus}\n    ]\n    \n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"getting-started.html#starting-a-bus"},{"type":"extras","title":"Creating a Subscriber - Getting Started: Your First Signal Bus","doc":"Build a simple `GenServer` that can receive signals:\n\n```elixir\ndefmodule MyApp.UserHandler do\n  use GenServer\n  alias Jido.Signal.Bus\n\n  def start_link(opts) do\n    GenServer.start_link(__MODULE__, opts, name: __MODULE__)\n  end\n\n  def init(_opts) do\n    # Subscribe to all user-related signals\n    Bus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: self()})\n    {:ok, %{}}\n  end\n\n  # Handle incoming signals\n  def handle_info({:signal, signal}, state) do\n    IO.puts(\"Received signal: #{signal.type}\")\n    IO.inspect(signal.data)\n    {:noreply, state}\n  end\nend\n```","ref":"getting-started.html#creating-a-subscriber"},{"type":"extras","title":"Subscribing to Signals - Getting Started: Your First Signal Bus","doc":"The `Bus.subscribe/3` function allows you to listen for signals matching a path pattern:\n\n```elixir\n# Subscribe to specific signal types\nBus.subscribe(:my_bus, \"user.created\", dispatch: {:pid, target: self()})\n\n# Use wildcards to match multiple types\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: self()})\n\n# Match all signals under a path\nBus.subscribe(:my_bus, \"user.**\", dispatch: {:pid, target: self()})\n```","ref":"getting-started.html#subscribing-to-signals"},{"type":"extras","title":"Publishing a Signal - Getting Started: Your First Signal Bus","doc":"Create and publish signals to the bus:\n\n```elixir\n# Create a basic signal\nsignal = Jido.Signal.new(%{\n  type: \"user.created\",\n  source: \"user_service\",\n  data: %{\n    user_id: 123,\n    email: \"user@example.com\",\n    name: \"John Doe\"\n  }\n})\n\n# Publish to the bus\nBus.publish(:my_bus, signal)\n```","ref":"getting-started.html#publishing-a-signal"},{"type":"extras","title":"Putting It All Together - Getting Started: Your First Signal Bus","doc":"Here's a complete example that demonstrates the entire flow:\n\n```elixir\ndefmodule MyApp.Example do\n  use GenServer\n  alias Jido.Signal.Bus\n\n  def start_link do\n    GenServer.start_link(__MODULE__, [], name: __MODULE__)\n  end\n\n  def init(_) do\n    # Subscribe to user signals\n    Bus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: self()})\n    \n    # Publish a test signal after a short delay\n    Process.send_after(self(), :publish_test, 1000)\n    \n    {:ok, %{}}\n  end\n\n  def handle_info(:publish_test, state) do\n    signal = Jido.Signal.new(%{\n      type: \"user.created\",\n      source: \"example\",\n      data: %{user_id: 123, name: \"Test User\"}\n    })\n    \n    Bus.publish(:my_bus, signal)\n    {:noreply, state}\n  end\n\n  def handle_info({:signal, signal}, state) do\n    IO.puts(\"✓ Received signal: #{signal.type}\")\n    IO.puts(\"  Data: #{inspect(signal.data)}\")\n    {:noreply, state}\n  end\nend\n```\n\nStart this in your IEx session to see it work:\n\n```elixir\n# Start the example GenServer\n{:ok, _pid} = MyApp.Example.start_link()\n\n# You should see output like:\n# ✓ Received signal: user.created\n#   Data: %{user_id: 123, name: \"Test User\"}\n```","ref":"getting-started.html#putting-it-all-together"},{"type":"extras","title":"Next Steps - Getting Started: Your First Signal Bus","doc":"Now that you have a basic signal bus running, explore these topics:\n\n- [Signal Structure](core-concepts/signal-structure.md) - Learn about signal anatomy\n- [Signal Bus](core-concepts/signal-bus.md) - Dive deeper into bus features\n- [Router](core-concepts/router.md) - Understand routing patterns\n- [Dispatch System](core-concepts/dispatch-system.md) - Send signals to external destinations","ref":"getting-started.html#next-steps"},{"type":"extras","title":"Guide: The Signal Structure","doc":"# Guide: The Signal Structure\n\nA `Jido.Signal` is the fundamental message unit in the system. It provides a standardized envelope that aligns with the CloudEvents specification, ensuring interoperability and consistency across your event-driven architecture.","ref":"signal-structure.html"},{"type":"extras","title":"Anatomy of a Signal - Guide: The Signal Structure","doc":"Every signal contains the following fields:","ref":"signal-structure.html#anatomy-of-a-signal"},{"type":"extras","title":"Required Fields - Guide: The Signal Structure","doc":"- **`id`** - Unique identifier for the signal (auto-generated if not provided)\n- **`source`** - Identifies the context in which the signal occurred (e.g., \"user_service\", \"payment_processor\")\n- **`type`** - Describes the type of event (e.g., \"user.created\", \"payment.processed\")\n- **`data`** - The actual payload/content of the signal","ref":"signal-structure.html#required-fields"},{"type":"extras","title":"Optional Fields - Guide: The Signal Structure","doc":"- **`time`** - Timestamp when the signal was created (auto-generated if not provided)\n- **`subject`** - The subject of the event (e.g., user ID, order ID)\n- **`datacontenttype`** - MIME type of the data (defaults to \"application/json\")\n- **`dataschema`** - URI reference to the schema of the data\n- **`metadata`** - Additional key-value pairs for extensibility","ref":"signal-structure.html#optional-fields"},{"type":"extras","title":"Creating Signals - Guide: The Signal Structure","doc":"","ref":"signal-structure.html#creating-signals"},{"type":"extras","title":"Basic Signal Creation - Guide: The Signal Structure","doc":"```elixir\n# Simple signal with minimal required fields\nsignal = Jido.Signal.new(%{\n  type: \"user.created\",\n  source: \"user_service\",\n  data: %{\n    user_id: 123,\n    email: \"user@example.com\"\n  }\n})\n```","ref":"signal-structure.html#basic-signal-creation"},{"type":"extras","title":"Signal with Full Metadata - Guide: The Signal Structure","doc":"```elixir\nsignal = Jido.Signal.new(%{\n  id: \"custom-id-123\",\n  type: \"user.profile.updated\",\n  source: \"user_service\",\n  subject: \"user:123\",\n  time: DateTime.utc_now(),\n  data: %{\n    user_id: 123,\n    changes: %{email: \"new@example.com\"}\n  },\n  datacontenttype: \"application/json\",\n  metadata: %{\n    version: \"1.0\",\n    region: \"us-west-2\"\n  }\n})\n```","ref":"signal-structure.html#signal-with-full-metadata"},{"type":"extras","title":"Defining Custom Signal Types - Guide: The Signal Structure","doc":"For strongly-typed signals with built-in validation, use the `use Jido.Signal` macro:\n\n```elixir\ndefmodule MyApp.UserCreatedSignal do\n  use Jido.Signal,\n    type: \"user.created\",\n    schema: [\n      user_id: [type: :integer, required: true],\n      email: [type: :string, required: true],\n      name: [type: :string, required: false],\n      created_at: [type: :string, required: false]\n    ]\nend\n```","ref":"signal-structure.html#defining-custom-signal-types"},{"type":"extras","title":"Using Custom Signal Types - Guide: The Signal Structure","doc":"```elixir\n# This will validate the data against the schema\n{:ok, signal} = MyApp.UserCreatedSignal.new(%{\n  source: \"user_service\",\n  data: %{\n    user_id: 123,\n    email: \"user@example.com\",\n    name: \"John Doe\"\n  }\n})\n\n# Invalid data will return an error\n{:error, reason} = MyApp.UserCreatedSignal.new(%{\n  source: \"user_service\",\n  data: %{\n    # Missing required user_id\n    email: \"user@example.com\"\n  }\n})\n```","ref":"signal-structure.html#using-custom-signal-types"},{"type":"extras","title":"Schema Validation - Guide: The Signal Structure","doc":"The `:schema` option in custom signal types provides compile-time and runtime validation:\n\n```elixir\ndefmodule MyApp.PaymentSignal do\n  use Jido.Signal,\n    type: \"payment.processed\",\n    schema: [\n      payment_id: [type: :string, required: true],\n      amount: [type: :integer, required: true],\n      currency: [type: :string, required: true, default: \"USD\"],\n      customer_id: [type: :string, required: true],\n      status: [type: :string, required: true, \n               in: [\"pending\", \"completed\", \"failed\"]]\n    ]\nend\n```","ref":"signal-structure.html#schema-validation"},{"type":"extras","title":"Schema Options - Guide: The Signal Structure","doc":"- **`type`** - Data type (`:string`, `:integer`, `:boolean`, `:map`, `:list`)\n- **`required`** - Whether the field is mandatory\n- **`default`** - Default value if not provided\n- **`in`** - List of valid values\n- **`format`** - Format validation (e.g., email, UUID)","ref":"signal-structure.html#schema-options"},{"type":"extras","title":"Signal Paths and Routing - Guide: The Signal Structure","doc":"The signal's `type` field is used for routing and subscription matching:\n\n```elixir\n# Exact match\n\"user.created\"\n\n# Single-level wildcard (matches one segment)\n\"user.*\"          # Matches \"user.created\", \"user.updated\", etc.\n\n# Multi-level wildcard (matches any number of segments)\n\"user.**\"         # Matches \"user.created\", \"user.profile.updated\", etc.\n```","ref":"signal-structure.html#signal-paths-and-routing"},{"type":"extras","title":"Best Practices - Guide: The Signal Structure","doc":"","ref":"signal-structure.html#best-practices"},{"type":"extras","title":"Naming Conventions - Guide: The Signal Structure","doc":"Use hierarchical naming with dots as separators:\n\n```elixir\n# Good examples\n\"user.created\"\n\"user.profile.updated\"\n\"payment.transaction.completed\"\n\"inventory.item.stock.low\"\n\n# Avoid\n\"userCreated\"\n\"user_created\"\n\"USER.CREATED\"\n```","ref":"signal-structure.html#naming-conventions"},{"type":"extras","title":"Source Identification - Guide: The Signal Structure","doc":"Make sources descriptive and consistent:\n\n```elixir\n# Good examples\n\"user_service\"\n\"payment_processor\"\n\"inventory_manager\"\n\"notification_service\"\n\n# Avoid generic sources\n\"app\"\n\"system\"\n\"service\"\n```","ref":"signal-structure.html#source-identification"},{"type":"extras","title":"Data Structure - Guide: The Signal Structure","doc":"Keep data focused and avoid deeply nested structures:\n\n```elixir\n# Good - flat and focused\ndata: %{\n  user_id: 123,\n  email: \"user@example.com\",\n  plan: \"premium\"\n}\n\n# Less ideal - deeply nested\ndata: %{\n  user: %{\n    profile: %{\n      contact: %{\n        email: \"user@example.com\"\n      }\n    }\n  }\n}\n```","ref":"signal-structure.html#data-structure"},{"type":"extras","title":"CloudEvents Compatibility - Guide: The Signal Structure","doc":"Jido.Signal maps directly to CloudEvents fields:\n\n| Jido.Signal | CloudEvents | Description |\n|-------------|-------------|-------------|\n| `id` | `id` | Unique event identifier |\n| `source` | `source` | Event source context |\n| `type` | `type` | Event type |\n| `data` | `data` | Event payload |\n| `time` | `time` | Event timestamp |\n| `subject` | `subject` | Event subject |\n| `datacontenttype` | `datacontenttype` | Content type |\n| `dataschema` | `dataschema` | Schema reference |\n| `metadata` | Extensions | Additional attributes |\n\nThis compatibility ensures your signals can be easily integrated with other CloudEvents-compliant systems.","ref":"signal-structure.html#cloudevents-compatibility"},{"type":"extras","title":"Guide: The Signal Bus","doc":"# Guide: The Signal Bus\n\nThe Signal Bus is the heart of Jido.Signal's pub/sub system. It's a `GenServer`-based hub that manages subscriptions, routes signals, and maintains a comprehensive signal log for replay capabilities.","ref":"signal-bus.html"},{"type":"extras","title":"Bus Architecture - Guide: The Signal Bus","doc":"The bus operates as a centralized message broker that:\n\n- Maintains active subscriptions and their routing rules\n- Logs every signal for replay and audit purposes\n- Handles both synchronous and asynchronous signal delivery\n- Provides middleware integration points\n- Supports persistent subscriptions with acknowledgment","ref":"signal-bus.html#bus-architecture"},{"type":"extras","title":"Managing Subscriptions - Guide: The Signal Bus","doc":"","ref":"signal-bus.html#managing-subscriptions"},{"type":"extras","title":"Basic Subscription - Guide: The Signal Bus","doc":"```elixir\nalias Jido.Signal.Bus\n\n# Subscribe to specific signal types\nBus.subscribe(:my_bus, \"user.created\", dispatch: {:pid, target: self()})\n\n# Subscribe with wildcards\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: self()})\nBus.subscribe(:my_bus, \"payment.**\", dispatch: {:pid, target: self()})\n```","ref":"signal-bus.html#basic-subscription"},{"type":"extras","title":"Subscription Options - Guide: The Signal Bus","doc":"```elixir\nBus.subscribe(:my_bus, \"user.*\", [\n  dispatch: {:pid, target: self()},\n  priority: 100,                    # Higher priority handlers run first\n  persistent?: true,                # Enable persistent delivery\n  filter: fn signal ->              # Optional signal filtering\n    signal.data.active == true\n  end\n])\n```","ref":"signal-bus.html#subscription-options"},{"type":"extras","title":"Unsubscribing - Guide: The Signal Bus","doc":"```elixir\n# Get subscription ID when subscribing\n{:ok, sub_id} = Bus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: self()})\n\n# Unsubscribe using the ID\nBus.unsubscribe(:my_bus, sub_id)\n```","ref":"signal-bus.html#unsubscribing"},{"type":"extras","title":"Publishing Signals - Guide: The Signal Bus","doc":"","ref":"signal-bus.html#publishing-signals"},{"type":"extras","title":"Basic Publishing - Guide: The Signal Bus","doc":"```elixir\nsignal = Jido.Signal.new(%{\n  type: \"user.created\",\n  source: \"user_service\",\n  data: %{user_id: 123}\n})\n\nBus.publish(:my_bus, signal)\n```","ref":"signal-bus.html#basic-publishing"},{"type":"extras","title":"Publishing Options - Guide: The Signal Bus","doc":"```elixir\n# Publish with metadata\nBus.publish(:my_bus, signal, %{priority: :high, async: true})\n\n# Publish multiple signals\nsignals = [signal1, signal2, signal3]\nBus.publish_batch(:my_bus, signals)\n```","ref":"signal-bus.html#publishing-options"},{"type":"extras","title":"The Internal Signal Log - Guide: The Signal Bus","doc":"The bus maintains a comprehensive log of every signal that passes through it:","ref":"signal-bus.html#the-internal-signal-log"},{"type":"extras","title":"Log Structure - Guide: The Signal Bus","doc":"```elixir\n%{\n  signal: %Jido.Signal{},           # The original signal\n  timestamp: ~U[2024-01-01 12:00:00Z], # When it was logged\n  metadata: %{},                    # Additional context\n  cause_id: \"parent-signal-id\"      # Causality tracking\n}\n```","ref":"signal-bus.html#log-structure"},{"type":"extras","title":"Querying the Log - Guide: The Signal Bus","doc":"```elixir\n# Get recent signals\n{:ok, entries} = Bus.get_log(:my_bus, limit: 10)\n\n# Get signals by path pattern\n{:ok, entries} = Bus.get_log(:my_bus, path: \"user.*\", limit: 50)\n\n# Get signals since a timestamp\nsince = DateTime.add(DateTime.utc_now(), -3600, :second)\n{:ok, entries} = Bus.get_log(:my_bus, since: since)\n```","ref":"signal-bus.html#querying-the-log"},{"type":"extras","title":"Replaying History - Guide: The Signal Bus","doc":"The replay system allows you to retrieve and reprocess signals from the log:","ref":"signal-bus.html#replaying-history"},{"type":"extras","title":"Basic Replay - Guide: The Signal Bus","doc":"```elixir\n# Replay all signals matching a pattern\nBus.replay(:my_bus, \"user.*\", \n  since: DateTime.add(DateTime.utc_now(), -1800, :second),\n  dispatch: {:pid, target: self()}\n)\n```","ref":"signal-bus.html#basic-replay"},{"type":"extras","title":"Advanced Replay Options - Guide: The Signal Bus","doc":"```elixir\nBus.replay(:my_bus, \"payment.**\", [\n  since: ~U[2024-01-01 00:00:00Z],\n  until: ~U[2024-01-01 23:59:59Z],\n  limit: 1000,\n  dispatch: {:pid, target: replay_handler_pid},\n  filter: fn entry ->\n    entry.signal.data.amount > 1000\n  end\n])\n```","ref":"signal-bus.html#advanced-replay-options"},{"type":"extras","title":"Replay to Multiple Destinations - Guide: The Signal Bus","doc":"```elixir\nBus.replay(:my_bus, \"user.*\", [\n  since: DateTime.add(DateTime.utc_now(), -3600, :second),\n  dispatch: [\n    {:pid, target: processor1_pid},\n    {:webhook, url: \"https://api.example.com/webhook\"},\n    {:logger, level: :info}\n  ]\n])\n```","ref":"signal-bus.html#replay-to-multiple-destinations"},{"type":"extras","title":"Persistent Subscriptions - Guide: The Signal Bus","doc":"Persistent subscriptions ensure \"at-least-once\" delivery even if the subscriber temporarily disconnects:","ref":"signal-bus.html#persistent-subscriptions"},{"type":"extras","title":"Creating Persistent Subscriptions - Guide: The Signal Bus","doc":"```elixir\n{:ok, sub_id} = Bus.subscribe(:my_bus, \"critical.alerts\", [\n  dispatch: {:pid, target: self()},\n  persistent?: true,\n  subscriber_id: \"alert_processor_1\"  # Stable identifier\n])\n```","ref":"signal-bus.html#creating-persistent-subscriptions"},{"type":"extras","title":"Acknowledgment Flow - Guide: The Signal Bus","doc":"```elixir\ndefmodule AlertProcessor do\n  use GenServer\n\n  def handle_info({:signal, signal, delivery_info}, state) do\n    case process_alert(signal) do\n      :ok ->\n        # Acknowledge successful processing\n        Bus.ack(:my_bus, delivery_info.subscription_id, delivery_info.message_id)\n        {:noreply, state}\n      \n      {:error, reason} ->\n        # Don't acknowledge - signal will be redelivered\n        Logger.error(\"Failed to process alert: #{inspect(reason)}\")\n        {:noreply, state}\n    end\n  end\n  \n  defp process_alert(signal) do\n    # Your processing logic here\n    :ok\n  end\nend\n```","ref":"signal-bus.html#acknowledgment-flow"},{"type":"extras","title":"Handling Reconnections - Guide: The Signal Bus","doc":"```elixir\n# Reconnect and replay missed signals\nBus.reconnect(:my_bus, \"alert_processor_1\", [\n  dispatch: {:pid, target: self()},\n  replay_missed: true\n])\n```","ref":"signal-bus.html#handling-reconnections"},{"type":"extras","title":"Bus Configuration - Guide: The Signal Bus","doc":"","ref":"signal-bus.html#bus-configuration"},{"type":"extras","title":"Starting with Custom Options - Guide: The Signal Bus","doc":"```elixir\n{Jido.Signal.Bus, [\n  name: :my_bus,\n  log_retention: :timer.hours(24),    # Keep logs for 24 hours\n  max_log_size: 100_000,              # Maximum log entries\n  middleware: [MyApp.AuditMiddleware], # Custom middleware\n  serializer: Jido.Signal.Serialization.JsonSerializer\n]}\n```","ref":"signal-bus.html#starting-with-custom-options"},{"type":"extras","title":"Runtime Configuration - Guide: The Signal Bus","doc":"```elixir\n# Update bus configuration\nBus.configure(:my_bus, log_retention: :timer.hours(48))\n\n# Get current configuration\nconfig = Bus.get_config(:my_bus)\n```","ref":"signal-bus.html#runtime-configuration"},{"type":"extras","title":"Monitoring and Introspection - Guide: The Signal Bus","doc":"","ref":"signal-bus.html#monitoring-and-introspection"},{"type":"extras","title":"Bus Status - Guide: The Signal Bus","doc":"```elixir\nstatus = Bus.status(:my_bus)\n# Returns:\n# %{\n#   active_subscriptions: 15,\n#   log_size: 1250,\n#   uptime: 3600,\n#   last_signal: ~U[2024-01-01 12:30:00Z]\n# }\n```","ref":"signal-bus.html#bus-status"},{"type":"extras","title":"Subscription Listing - Guide: The Signal Bus","doc":"```elixir\nsubscriptions = Bus.list_subscriptions(:my_bus)\n# Returns list of active subscriptions with their patterns and options\n```","ref":"signal-bus.html#subscription-listing"},{"type":"extras","title":"Error Handling - Guide: The Signal Bus","doc":"","ref":"signal-bus.html#error-handling"},{"type":"extras","title":"Delivery Failures - Guide: The Signal Bus","doc":"```elixir\n# Configure delivery retry behavior\nBus.subscribe(:my_bus, \"user.*\", [\n  dispatch: {:http, url: \"https://api.example.com/webhook\"},\n  retry_policy: %{\n    max_retries: 3,\n    backoff: :exponential,\n    base_delay: 1000\n  }\n])\n```","ref":"signal-bus.html#delivery-failures"},{"type":"extras","title":"Dead Letter Handling - Guide: The Signal Bus","doc":"```elixir\n# Signals that fail delivery can be routed to a dead letter queue\nBus.subscribe(:my_bus, \"user.*\", [\n  dispatch: {:pid, target: self()},\n  dead_letter: {:logger, level: :error}\n])\n```","ref":"signal-bus.html#dead-letter-handling"},{"type":"extras","title":"Performance Considerations - Guide: The Signal Bus","doc":"","ref":"signal-bus.html#performance-considerations"},{"type":"extras","title":"Batch Operations - Guide: The Signal Bus","doc":"For high-throughput scenarios, use batch operations:\n\n```elixir\n# Batch publish multiple signals\nsignals = Enum.map(1..1000, fn i ->\n  Jido.Signal.new(%{\n    type: \"batch.item\",\n    source: \"batch_processor\",\n    data: %{item_id: i}\n  })\nend)\n\nBus.publish_batch(:my_bus, signals)\n```","ref":"signal-bus.html#batch-operations"},{"type":"extras","title":"Asynchronous Processing - Guide: The Signal Bus","doc":"```elixir\n# Publish without blocking\nBus.publish_async(:my_bus, signal)\n\n# Subscribe with async dispatch\nBus.subscribe(:my_bus, \"user.*\", [\n  dispatch: {:pid, target: self(), async: true}\n])\n```\n\nThe Signal Bus provides a robust foundation for building scalable, observable event-driven systems with comprehensive replay and persistence capabilities.","ref":"signal-bus.html#asynchronous-processing"},{"type":"extras","title":"Guide: The Router","doc":"# Guide: The Router\n\nThe Router is Jido.Signal's intelligent routing engine that uses a trie-based data structure to efficiently match signals to handlers. It supports complex path patterns and sophisticated prioritization rules.","ref":"router.html"},{"type":"extras","title":"Trie-Based Routing Engine - Guide: The Router","doc":"The router uses a prefix tree (trie) to organize and match signal paths efficiently:\n\n```\nuser\n├── created\n├── updated\n├── deleted\n└── profile\n    ├── updated\n    └── verified\n```\n\nThis structure allows for O(k) lookup time where k is the path depth, making routing extremely fast even with thousands of subscriptions.","ref":"router.html#trie-based-routing-engine"},{"type":"extras","title":"Path Patterns - Guide: The Router","doc":"The router supports three types of path matching:","ref":"router.html#path-patterns"},{"type":"extras","title":"Exact Matches - Guide: The Router","doc":"```elixir\n# Matches only \"user.created\"\n\"user.created\"\n```","ref":"router.html#exact-matches"},{"type":"extras","title":"Single-Level Wildcards (*) - Guide: The Router","doc":"The `*` wildcard matches exactly one path segment:\n\n```elixir\n# Matches: \"user.created\", \"user.updated\", \"user.deleted\"\n# Does NOT match: \"user.profile.updated\"\n\"user.*\"\n\n# Matches: \"order.item.added\", \"order.item.removed\"  \n# Does NOT match: \"order.created\" or \"order.item.stock.updated\"\n\"order.item.*\"\n```","ref":"router.html#single-level-wildcards"},{"type":"extras","title":"Multi-Level Wildcards (**) - Guide: The Router","doc":"The `**` wildcard matches any number of path segments:\n\n```elixir\n# Matches: \"user.created\", \"user.profile.updated\", \"user.settings.privacy.changed\"\n\"user.**\"\n\n# Matches anything under the payment namespace\n\"payment.**\"\n\n# Matches all signals\n\"**\"\n```","ref":"router.html#multi-level-wildcards"},{"type":"extras","title":"Pattern Examples - Guide: The Router","doc":"```elixir\nalias Jido.Signal.Bus\n\n# Exact subscription\nBus.subscribe(:my_bus, \"user.created\", dispatch: {:pid, target: self()})\n\n# Single-level wildcard - all direct user events\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: self()})\n\n# Multi-level wildcard - all user-related events\nBus.subscribe(:my_bus, \"user.**\", dispatch: {:pid, target: self()})\n\n# Complex patterns\nBus.subscribe(:my_bus, \"payment.transaction.*\", dispatch: {:pid, target: self()})\nBus.subscribe(:my_bus, \"inventory.**.low\", dispatch: {:pid, target: self()})\n```","ref":"router.html#pattern-examples"},{"type":"extras","title":"Execution Order and Priority - Guide: The Router","doc":"When multiple patterns match a signal, the router determines execution order using three criteria:","ref":"router.html#execution-order-and-priority"},{"type":"extras","title":"1. Path Specificity (Complexity) - Guide: The Router","doc":"More specific patterns execute first:\n\n```elixir\n# For signal \"user.profile.updated\", execution order is:\n# 1. \"user.profile.updated\"  (exact match - highest specificity)\n# 2. \"user.profile.*\"        (single wildcard)\n# 3. \"user.*\"                (single wildcard, less specific)\n# 4. \"user.**\"               (multi-level wildcard)\n# 5. \"**\"                    (catch-all - lowest specificity)\n```","ref":"router.html#1-path-specificity-complexity"},{"type":"extras","title":"2. Explicit Priority - Guide: The Router","doc":"You can override natural ordering with explicit priority:\n\n```elixir\n# Higher priority numbers execute first\nBus.subscribe(:my_bus, \"user.**\", dispatch: {:pid, target: audit_logger}, priority: 1000)\nBus.subscribe(:my_bus, \"user.created\", dispatch: {:pid, target: welcome_sender}, priority: 500)\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: analytics}, priority: 100)\n```","ref":"router.html#2-explicit-priority"},{"type":"extras","title":"3. Registration Order - Guide: The Router","doc":"For handlers with the same specificity and priority, first-registered executes first:\n\n```elixir\n# These will execute in registration order\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: handler_a})  # First\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: handler_b})  # Second\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: handler_c})  # Third\n```","ref":"router.html#3-registration-order"},{"type":"extras","title":"Standalone Router Usage - Guide: The Router","doc":"You can use the router directly for advanced scenarios:","ref":"router.html#standalone-router-usage"},{"type":"extras","title":"Creating a Router - Guide: The Router","doc":"```elixir\nalias Jido.Signal.Router\n\n# Create a new router\n{:ok, router} = Router.new()\n\n# Add routes\n{:ok, route_id} = Router.add_route(router, \"user.*\", %{handler: :user_handler})\n{:ok, route_id} = Router.add_route(router, \"payment.**\", %{handler: :payment_handler})\n```","ref":"router.html#creating-a-router"},{"type":"extras","title":"Matching Routes - Guide: The Router","doc":"```elixir\n# Find matching routes for a signal path\nmatches = Router.match(router, \"user.created\")\n# Returns: [%{pattern: \"user.*\", handler: :user_handler, priority: 0}]\n\n# Get all routes\nall_routes = Router.list_routes(router)\n```","ref":"router.html#matching-routes"},{"type":"extras","title":"Route Management - Guide: The Router","doc":"```elixir\n# Remove a route\nRouter.remove_route(router, route_id)\n\n# Update route priority\nRouter.update_route(router, route_id, %{priority: 500})\n\n# Clear all routes\nRouter.clear(router)\n```","ref":"router.html#route-management"},{"type":"extras","title":"Advanced Routing Patterns - Guide: The Router","doc":"","ref":"router.html#advanced-routing-patterns"},{"type":"extras","title":"Conditional Routing - Guide: The Router","doc":"Combine path patterns with filters for sophisticated routing:\n\n```elixir\nBus.subscribe(:my_bus, \"user.*\", [\n  dispatch: {:pid, target: premium_handler},\n  filter: fn signal ->\n    signal.data.subscription_type == \"premium\"\n  end,\n  priority: 100\n])\n\nBus.subscribe(:my_bus, \"user.*\", [\n  dispatch: {:pid, target: standard_handler},\n  filter: fn signal ->\n    signal.data.subscription_type != \"premium\"\n  end,\n  priority: 50\n])\n```","ref":"router.html#conditional-routing"},{"type":"extras","title":"Hierarchical Processing - Guide: The Router","doc":"Use wildcards to create processing hierarchies:\n\n```elixir\n# Catch-all logging (lowest priority)\nBus.subscribe(:my_bus, \"**\", [\n  dispatch: {:logger, level: :debug},\n  priority: 1\n])\n\n# Domain-specific logging\nBus.subscribe(:my_bus, \"user.**\", [\n  dispatch: {:logger, level: :info, prefix: \"USER\"},\n  priority: 10\n])\n\n# Critical event alerting\nBus.subscribe(:my_bus, \"user.deleted\", [\n  dispatch: {:webhook, url: \"https://alerts.example.com\"},\n  priority: 1000\n])\n```","ref":"router.html#hierarchical-processing"},{"type":"extras","title":"Fan-Out Patterns - Guide: The Router","doc":"Route single signals to multiple handlers:\n\n```elixir\n# A single signal can trigger multiple handlers\nBus.subscribe(:my_bus, \"order.completed\", dispatch: {:pid, target: inventory_updater})\nBus.subscribe(:my_bus, \"order.completed\", dispatch: {:pid, target: email_sender})\nBus.subscribe(:my_bus, \"order.completed\", dispatch: {:pid, target: analytics_tracker})\nBus.subscribe(:my_bus, \"order.completed\", dispatch: {:webhook, url: \"https://api.crm.com\"})\n```","ref":"router.html#fan-out-patterns"},{"type":"extras","title":"Pattern Matching Behavior - Guide: The Router","doc":"","ref":"router.html#pattern-matching-behavior"},{"type":"extras","title":"Edge Cases - Guide: The Router","doc":"```elixir\n# Empty segments are significant\n\"user..created\"     # Matches signal with empty segment: [\"user\", \"\", \"created\"]\n\"user.*.\"          # Matches: \"user.something.\" (with trailing empty segment)\n\n# Multiple wildcards\n\"*.user.*\"         # Matches: \"app.user.created\", \"service.user.updated\"\n\"**.critical.**\"   # Matches any path containing \"critical\"\n```","ref":"router.html#edge-cases"},{"type":"extras","title":"Escape Sequences - Guide: The Router","doc":"Special characters in literal paths:\n\n```elixir\n# To match literal dots, use escape sequences in custom implementations\n# The standard router treats all dots as path separators\n```","ref":"router.html#escape-sequences"},{"type":"extras","title":"Performance Characteristics - Guide: The Router","doc":"","ref":"router.html#performance-characteristics"},{"type":"extras","title":"Lookup Performance - Guide: The Router","doc":"- **O(k)** where k is the path depth\n- **Memory efficient** with shared prefixes\n- **Fast wildcard matching** using trie traversal","ref":"router.html#lookup-performance"},{"type":"extras","title":"Optimization Tips - Guide: The Router","doc":"```elixir\n# More specific patterns first for better cache locality\nBus.subscribe(:my_bus, \"user.profile.updated\", ...)  # Specific\nBus.subscribe(:my_bus, \"user.*\", ...)                # General\n\n# Use appropriate wildcard types\n\"user.*\"     # When you only want direct children\n\"user.**\"    # When you want all descendants\n```","ref":"router.html#optimization-tips"},{"type":"extras","title":"Benchmarking - Guide: The Router","doc":"```elixir\n# Test routing performance\nsignal = Jido.Signal.new(%{type: \"user.profile.updated\", source: \"test\"})\n\n{time, _result} = :timer.tc(fn ->\n  Router.match(router, signal.type)\nend)\n\nIO.puts(\"Routing took #{time} microseconds\")\n```","ref":"router.html#benchmarking"},{"type":"extras","title":"Best Practices - Guide: The Router","doc":"","ref":"router.html#best-practices"},{"type":"extras","title":"Pattern Design - Guide: The Router","doc":"1. **Use hierarchical naming**: `domain.entity.action`\n2. **Be consistent**: Choose a naming convention and stick to it\n3. **Avoid deep nesting**: Keep paths readable and manageable\n4. **Use meaningful names**: Make patterns self-documenting","ref":"router.html#pattern-design"},{"type":"extras","title":"Performance Optimization - Guide: The Router","doc":"1. **Register specific patterns first** for better cache performance\n2. **Use single-level wildcards** when you don't need deep matching\n3. **Batch route additions** when possible\n4. **Monitor routing performance** in high-throughput scenarios","ref":"router.html#performance-optimization"},{"type":"extras","title":"Error Handling - Guide: The Router","doc":"```elixir\n# Handle routing errors gracefully\ncase Router.add_route(router, pattern, handler_config) do\n  {:ok, route_id} -> \n    # Success\n  {:error, :invalid_pattern} -> \n    # Handle invalid pattern\n  {:error, reason} -> \n    # Handle other errors\nend\n```\n\nThe Router provides the foundation for building sophisticated, high-performance event routing systems that can scale to handle complex signal distribution patterns.","ref":"router.html#error-handling"},{"type":"extras","title":"Guide: The Dispatch System","doc":"# Guide: The Dispatch System\n\nThe Dispatch System is Jido.Signal's pluggable architecture for sending signals to various destinations. It provides a unified interface for delivering signals to processes, external services, logging systems, and custom endpoints.","ref":"dispatch-system.html"},{"type":"extras","title":"Adapter-Based Architecture - Guide: The Dispatch System","doc":"The dispatch system uses adapters to handle different delivery mechanisms. Each adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour, providing consistent interfaces for validation and delivery.\n\n```elixir\n# Basic dispatch configuration\ndispatch_config = {:pid, target: self()}\n\n# Dispatch with options\ndispatch_config = {:http, url: \"https://api.example.com/webhook\", headers: %{\"Authorization\" => \"Bearer token\"}}\n```","ref":"dispatch-system.html#adapter-based-architecture"},{"type":"extras","title":"Built-in Adapters - Guide: The Dispatch System","doc":"### `:pid` - Process Delivery\n\nSends signals directly to Elixir processes:\n\n```elixir\n# Send to a specific PID\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: self()})\n\n# Send to a named process\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: :user_handler})\n\n# Async delivery to avoid blocking\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pid, target: self(), async: true})\n```\n\n**Options:**\n- `target` - PID or registered process name (required)\n- `async` - Send asynchronously (default: false)\n- `timeout` - Delivery timeout in milliseconds (default: 5000)\n\n### `:named` - Named Process Delivery\n\nSends signals to processes registered with specific names:\n\n```elixir\n# Send to locally registered process\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:named, name: MyApp.UserHandler})\n\n# Send to globally registered process\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:named, name: {:global, :user_handler}})\n\n# Send via Registry\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:named, name: {:via, Registry, {MyApp.Registry, :handler}}})\n```\n\n**Options:**\n- `name` - Process name specification (required)\n- `async` - Send asynchronously (default: false)\n- `timeout` - Delivery timeout (default: 5000)\n\n### `:pubsub` - Phoenix.PubSub Integration\n\nDistributes signals across clustered nodes using Phoenix.PubSub:\n\n```elixir\n# Broadcast to a topic\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pubsub, topic: \"user_events\"})\n\n# Use custom PubSub instance\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pubsub, \n  pubsub: MyApp.PubSub, \n  topic: \"critical_events\"\n})\n\n# Local broadcast only\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:pubsub, \n  topic: \"local_events\", \n  local?: true\n})\n```\n\n**Options:**\n- `topic` - PubSub topic name (required)\n- `pubsub` - PubSub instance name (default: app's configured PubSub)\n- `local?` - Broadcast only to local node (default: false)\n\n### `:logger` - Structured Logging\n\nLogs signals using Elixir's Logger:\n\n```elixir\n# Basic logging\nBus.subscribe(:my_bus, \"error.**\", dispatch: {:logger, level: :error})\n\n# With custom formatting\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:logger, \n  level: :info,\n  format: \"User event: $type from $source\",\n  metadata: [:user_id, :timestamp]\n})\n```\n\n**Options:**\n- `level` - Log level (`:debug`, `:info`, `:warn`, `:error`)\n- `format` - Custom log format string\n- `metadata` - Additional metadata fields to include\n\n### `:console` - Console Output\n\nPrints signals to stdout/stderr for development and debugging:\n\n```elixir\n# Simple console output\nBus.subscribe(:my_bus, \"debug.**\", dispatch: {:console})\n\n# Formatted output\nBus.subscribe(:my_bus, \"user.*\", dispatch: {:console, \n  format: :pretty,\n  colors: true\n})\n\n# Output to stderr\nBus.subscribe(:my_bus, \"error.**\", dispatch: {:console, \n  device: :stderr,\n  format: :json\n})\n```\n\n**Options:**\n- `format` - Output format (`:json`, `:pretty`, `:compact`)\n- `colors` - Enable colored output (default: true)\n- `device` - Output device (`:stdio`, `:stderr`)\n\n### `:noop` - No Operation\n\nDiscards signals (useful for testing and conditional routing):\n\n```elixir\n# Conditional dispatch based on environment\ndispatch_config = if Mix.env() == :test do\n  {:noop}\nelse\n  {:logger, level: :info}\nend\n\nBus.subscribe(:my_bus, \"test.**\", dispatch: dispatch_config)\n```\n\n### `:http` - HTTP Delivery\n\nSends signals as HTTP requests:\n\n```elixir\n# Basic HTTP POST\nBus.subscribe(:my_bus, \"webhook.**\", dispatch: {:http, \n  url: \"https://api.example.com/events\"\n})\n\n# With authentication and custom headers\nBus.subscribe(:my_bus, \"critical.**\", dispatch: {:http,\n  url: \"https://alerts.example.com/webhook\",\n  method: :post,\n  headers: %{\n    \"Authorization\" => \"Bearer #{token}\",\n    \"Content-Type\" => \"application/json\",\n    \"X-Source\" => \"jido-signal\"\n  },\n  timeout: 10_000\n})\n```\n\n**Options:**\n- `url` - Target URL (required)\n- `method` - HTTP method (default: `:post`)\n- `headers` - Custom headers map\n- `timeout` - Request timeout in milliseconds (default: 5000)\n- `retry_policy` - Retry configuration for failed requests\n\n### `:webhook` - Webhook Delivery\n\nSpecialized HTTP delivery with webhook-specific features:\n\n```elixir\n# Webhook with signature verification\nBus.subscribe(:my_bus, \"payment.**\", dispatch: {:webhook,\n  url: \"https://merchant.example.com/webhook\",\n  secret: \"webhook_secret_key\",\n  signature_header: \"X-Signature\"\n})\n\n# Multiple webhook endpoints\nBus.subscribe(:my_bus, \"order.**\", dispatch: {:webhook,\n  urls: [\n    \"https://inventory.example.com/webhook\",\n    \"https://analytics.example.com/webhook\",\n    \"https://notifications.example.com/webhook\"\n  ],\n  parallel: true\n})\n```\n\n**Options:**\n- `url` or `urls` - Webhook endpoint(s)\n- `secret` - Secret for HMAC signature generation\n- `signature_header` - Header name for signature (default: \"X-Signature\")\n- `parallel` - Send to multiple URLs concurrently (default: false)","ref":"dispatch-system.html#built-in-adapters"},{"type":"extras","title":"Dispatching to Multiple Targets - Guide: The Dispatch System","doc":"Send signals to multiple destinations simultaneously:\n\n```elixir\nBus.subscribe(:my_bus, \"user.created\", dispatch: [\n  {:logger, level: :info},\n  {:http, url: \"https://crm.example.com/webhook\"},\n  {:pid, target: :email_service},\n  {:pubsub, topic: \"user_events\"}\n])\n```","ref":"dispatch-system.html#dispatching-to-multiple-targets"},{"type":"extras","title":"Asynchronous and Batch Dispatching - Guide: The Dispatch System","doc":"","ref":"dispatch-system.html#asynchronous-and-batch-dispatching"},{"type":"extras","title":"Asynchronous Dispatch - Guide: The Dispatch System","doc":"For non-blocking delivery:\n\n```elixir\n# Individual async dispatch\nDispatch.dispatch_async(signal, {:http, url: \"https://slow-api.example.com\"})\n\n# Async subscription\nBus.subscribe(:my_bus, \"analytics.**\", dispatch: {:http, \n  url: \"https://analytics.example.com\", \n  async: true\n})\n```","ref":"dispatch-system.html#asynchronous-dispatch"},{"type":"extras","title":"Batch Dispatch - Guide: The Dispatch System","doc":"For high-throughput scenarios:\n\n```elixir\nsignals = [signal1, signal2, signal3]\n\n# Batch dispatch to single target\nDispatch.dispatch_batch(signals, {:http, url: \"https://api.example.com/batch\"})\n\n# Batch dispatch with different targets\ndispatch_configs = [\n  {:logger, level: :info},\n  {:http, url: \"https://api.example.com/webhook\"}\n]\n\nDispatch.dispatch_batch(signals, dispatch_configs)\n```","ref":"dispatch-system.html#batch-dispatch"},{"type":"extras","title":"Error Handling and Retries - Guide: The Dispatch System","doc":"","ref":"dispatch-system.html#error-handling-and-retries"},{"type":"extras","title":"Retry Policies - Guide: The Dispatch System","doc":"Configure automatic retries for failed deliveries:\n\n```elixir\nBus.subscribe(:my_bus, \"critical.**\", dispatch: {:http,\n  url: \"https://critical-api.example.com/webhook\",\n  retry_policy: %{\n    max_retries: 3,\n    backoff: :exponential,\n    base_delay: 1000,        # 1 second\n    max_delay: 30_000,       # 30 seconds\n    jitter: true\n  }\n})\n```","ref":"dispatch-system.html#retry-policies"},{"type":"extras","title":"Dead Letter Handling - Guide: The Dispatch System","doc":"Route failed signals to alternative destinations:\n\n```elixir\nBus.subscribe(:my_bus, \"payment.**\", dispatch: [\n  # Primary delivery\n  {:http, url: \"https://payment-processor.example.com/webhook\"},\n  \n  # Dead letter queue for failures\n  {:logger, level: :error, on_failure: true},\n  {:http, url: \"https://dead-letter-queue.example.com\", on_failure: true}\n])\n```","ref":"dispatch-system.html#dead-letter-handling"},{"type":"extras","title":"Circuit Breaker Pattern - Guide: The Dispatch System","doc":"Protect against cascading failures:\n\n```elixir\nBus.subscribe(:my_bus, \"external.**\", dispatch: {:http,\n  url: \"https://external-service.example.com\",\n  circuit_breaker: %{\n    failure_threshold: 5,\n    recovery_timeout: 60_000,\n    half_open_max_calls: 3\n  }\n})\n```","ref":"dispatch-system.html#circuit-breaker-pattern"},{"type":"extras","title":"Custom Dispatch Adapters - Guide: The Dispatch System","doc":"Create custom adapters for specialized delivery needs:\n\n```elixir\ndefmodule MyApp.SlackAdapter do\n  @behaviour Jido.Signal.Dispatch.Adapter\n  \n  @impl true\n  def validate_opts(opts) do\n    required = [:channel, :token]\n    case NimbleOptions.validate(opts, required) do\n      {:ok, validated} -> {:ok, validated}\n      {:error, error} -> {:error, error}\n    end\n  end\n  \n  @impl true\n  def deliver(signal, opts) do\n    case post_to_slack(signal, opts) do\n      {:ok, response} -> {:ok, response}\n      {:error, reason} -> {:error, reason}\n    end\n  end\n  \n  defp post_to_slack(signal, opts) do\n    # Implementation details\n  end\nend\n```\n\nUse custom adapters:\n\n```elixir\nBus.subscribe(:my_bus, \"alerts.**\", dispatch: {MyApp.SlackAdapter,\n  channel: \"#alerts\",\n  token: \"slack-bot-token\"\n})\n```","ref":"dispatch-system.html#custom-dispatch-adapters"},{"type":"extras","title":"Performance Optimization - Guide: The Dispatch System","doc":"","ref":"dispatch-system.html#performance-optimization"},{"type":"extras","title":"Connection Pooling - Guide: The Dispatch System","doc":"For HTTP-based adapters, use connection pooling:\n\n```elixir\n# Configure HTTP client with pooling\nBus.subscribe(:my_bus, \"api.**\", dispatch: {:http,\n  url: \"https://api.example.com/webhook\",\n  pool: :api_pool,\n  pool_size: 10,\n  pool_timeout: 5000\n})\n```","ref":"dispatch-system.html#connection-pooling"},{"type":"extras","title":"Batching Strategies - Guide: The Dispatch System","doc":"Optimize for throughput with batching:\n\n```elixir\n# Time-based batching\nBus.subscribe(:my_bus, \"metrics.**\", dispatch: {:http,\n  url: \"https://metrics.example.com/batch\",\n  batch_size: 100,\n  batch_timeout: 5000\n})\n\n# Size-based batching\nBus.subscribe(:my_bus, \"logs.**\", dispatch: {:http,\n  url: \"https://logs.example.com/bulk\",\n  batch_size: 1000,\n  flush_interval: 10_000\n})\n```","ref":"dispatch-system.html#batching-strategies"},{"type":"extras","title":"Monitoring and Observability - Guide: The Dispatch System","doc":"","ref":"dispatch-system.html#monitoring-and-observability"},{"type":"extras","title":"Telemetry Integration - Guide: The Dispatch System","doc":"The dispatch system emits telemetry events for monitoring:\n\n```elixir\n:telemetry.attach(\"dispatch-metrics\", [:jido_signal, :dispatch, :deliver], fn event, measurements, metadata, _config ->\n  # Log delivery metrics\n  Logger.info(\"Signal delivered\", [\n    adapter: metadata.adapter,\n    duration: measurements.duration,\n    success: metadata.success\n  ])\nend)\n```","ref":"dispatch-system.html#telemetry-integration"},{"type":"extras","title":"Health Checks - Guide: The Dispatch System","doc":"Monitor adapter health:\n\n```elixir\n# Check adapter connectivity\n{:ok, status} = Dispatch.health_check({:http, url: \"https://api.example.com\"})\n# Returns: %{status: :healthy, latency: 45, last_success: ~U[...]}\n```\n\nThe Dispatch System provides a flexible, robust foundation for integrating Jido.Signal with any external system or service, ensuring reliable signal delivery across your entire architecture.","ref":"dispatch-system.html#health-checks"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}